@@ -13,7 +13,7 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- * 
+ *
  */
 
 package org.apache.jmeter.assertions;
@@ -46,6 +46,8 @@ import org.apache.oro.text.regex.Perl5Matcher;
 public class ResponseAssertion extends AbstractScopedAssertion implements Serializable, Assertion {
     private static final Logger log = LoggingManager.getLoggerForClass();
 
+    private static final long serialVersionUID = 240L;
+
     private final static String TEST_FIELD = "Assertion.test_field";  // $NON-NLS-1$
 
     // Values for TEST_FIELD
@@ -72,7 +74,7 @@ public class ResponseAssertion extends AbstractScopedAssertion implements Serial
      */
     private final static int MATCH = 1 << 0;
 
-    final static int CONTAINS = 1 << 1;
+    private final static int CONTAINS = 1 << 1;
 
     private final static int NOT = 1 << 2;
 
@@ -82,7 +84,7 @@ public class ResponseAssertion extends AbstractScopedAssertion implements Serial
 
     // Mask should contain all types (but not NOT)
     private final static int TYPE_MASK = CONTAINS | EQUALS | MATCH | SUBSTRING;
-    
+
     private static final int  EQUALS_SECTION_DIFF_LEN
             = JMeterUtils.getPropDefault("assertion.equals_section_diff_len", 100);
 
@@ -97,12 +99,13 @@ public class ResponseAssertion extends AbstractScopedAssertion implements Serial
             = JMeterUtils.getPropDefault("assertion.equals_diff_delta_end", "]]]");
 
     public ResponseAssertion() {
-        setProperty(new CollectionProperty(TEST_STRINGS, new ArrayList()));
+        setProperty(new CollectionProperty(TEST_STRINGS, new ArrayList<String>()));
     }
 
+    @Override
     public void clear() {
         super.clear();
-        setProperty(new CollectionProperty(TEST_STRINGS, new ArrayList()));
+        setProperty(new CollectionProperty(TEST_STRINGS, new ArrayList<String>()));
     }
 
     private void setTestField(String testField) {
@@ -189,7 +192,7 @@ public class ResponseAssertion extends AbstractScopedAssertion implements Serial
 
     /***************************************************************************
      * !ToDoo (Method description)
-     * 
+     *
      * @return !ToDo (Return description)
      **************************************************************************/
     public String getTestField() {
@@ -198,7 +201,7 @@ public class ResponseAssertion extends AbstractScopedAssertion implements Serial
 
     /***************************************************************************
      * !ToDoo (Method description)
-     * 
+     *
      * @return !ToDo (Return description)
      **************************************************************************/
     public int getTestType() {
@@ -211,7 +214,7 @@ public class ResponseAssertion extends AbstractScopedAssertion implements Serial
 
     /***************************************************************************
      * !ToDoo (Method description)
-     * 
+     *
      * @return !ToDo (Return description)
      **************************************************************************/
     public CollectionProperty getTestStrings() {
@@ -272,7 +275,7 @@ public class ResponseAssertion extends AbstractScopedAssertion implements Serial
 
     /**
      * Make sure the response satisfies the specified assertion requirements.
-     * 
+     *
      * @param response
      *            an instance of SampleResult
      * @return an instance of AssertionResult
@@ -288,7 +291,9 @@ public class ResponseAssertion extends AbstractScopedAssertion implements Serial
         }
 
         // What are we testing against?
-        if (isTestFieldResponseData()) {
+        if (isScopeVariable()){
+            toCheck = getThreadContext().getVariables().get(getVariableName());
+        } else if (isTestFieldResponseData()) {
             toCheck = response.getResponseDataAsString(); // (bug25052)
         } else if (isTestFieldResponseCode()) {
             toCheck = response.getResponseCode();
@@ -297,10 +302,10 @@ public class ResponseAssertion extends AbstractScopedAssertion implements Serial
         } else if (isTestFieldResponseHeaders()) {
             toCheck = response.getResponseHeaders();
         } else { // Assume it is the URL
-            toCheck = "";                
+            toCheck = "";
             final URL url = response.getURL();
             if (url != null){
-                toCheck = url.toString();                
+                toCheck = url.toString();
             }
         }
 
@@ -322,7 +327,7 @@ public class ResponseAssertion extends AbstractScopedAssertion implements Serial
         if (debugEnabled){
             log.debug("Type:" + (contains?"Contains":"Match") + (notTest? "(not)": ""));
         }
-        
+
         try {
             // Get the Matcher for this thread
             Perl5Matcher localMatcher = JMeterUtils.getMatcher();
@@ -362,17 +367,19 @@ public class ResponseAssertion extends AbstractScopedAssertion implements Serial
 
     /**
      * Generate the failure reason from the TestType
-     * 
+     *
      * @param stringPattern
-     * @return the message for the assertion report 
+     * @return the message for the assertion report
      */
     // TODO strings should be resources
     private String getFailText(String stringPattern, String toCheck) {
-        
-        StringBuffer sb = new StringBuffer(200);
+
+        StringBuilder sb = new StringBuilder(200);
         sb.append("Test failed: ");
 
-        if (isTestFieldResponseData()) {
+        if (isScopeVariable()){
+            sb.append("variable(").append(getVariableName()).append(')');
+        } else if (isTestFieldResponseData()) {
             sb.append("text");
         } else if (isTestFieldResponseCode()) {
             sb.append("code");
@@ -411,15 +418,15 @@ public class ResponseAssertion extends AbstractScopedAssertion implements Serial
         }
 
         sb.append("/");
-        
+
         if (isEqualsType()){
             sb.append(equalsComparisonText(toCheck, stringPattern));
         } else {
             sb.append(stringPattern);
         }
-        
+
         sb.append("/");
-        
+
         return sb.toString();
     }
 
@@ -445,9 +452,8 @@ public class ResponseAssertion extends AbstractScopedAssertion implements Serial
      * @return  Two lines of text separated by newlines, and then forward and backward pointers
      *      denoting first position of difference.
      */
-    private static StringBuffer equalsComparisonText(final String received, final String comparison)
+    private static StringBuilder equalsComparisonText(final String received, final String comparison)
     {
-        final StringBuffer      text;
         int                     firstDiff;
         int                     lastRecDiff = -1;
         int                     lastCompDiff = -1;
@@ -458,10 +464,8 @@ public class ResponseAssertion extends AbstractScopedAssertion implements Serial
         String                  recDeltaSeq = "";
         String                  compDeltaSeq = "";
         String                  endingEqSeq = "";
-        final StringBuffer      pad;
 
-
-        text = new StringBuffer(Math.max(recLength, compLength) * 2);
+        final StringBuilder text = new StringBuilder(Math.max(recLength, compLength) * 2);
         for (firstDiff = 0; firstDiff < minLength; firstDiff++) {
             if (received.charAt(firstDiff) != comparison.charAt(firstDiff)){
                 break;
@@ -493,7 +497,7 @@ public class ResponseAssertion extends AbstractScopedAssertion implements Serial
             recDeltaSeq = trunc(true, received.substring(firstDiff, lastRecDiff + 1));
             compDeltaSeq = trunc(true, comparison.substring(firstDiff, lastCompDiff + 1));
         }
-        pad = new StringBuffer(Math.abs(recDeltaSeq.length() - compDeltaSeq.length()));
+        final StringBuilder pad = new StringBuilder(Math.abs(recDeltaSeq.length() - compDeltaSeq.length()));
         for (int i = 0; i < pad.capacity(); i++){
             pad.append(' ');
         }
@@ -502,7 +506,7 @@ public class ResponseAssertion extends AbstractScopedAssertion implements Serial
         } else {
             recDeltaSeq += pad.toString();
         }
-        
+
         text.append("\n\n");
         text.append(RECEIVED_STR);
         text.append(startingEqSeq);
