@@ -222,7 +222,7 @@ public class JMeter implements JMeterPlugin {
         GuiPackage.getInstance(treeLis, treeModel);
         MainFrame main = new MainFrame(ActionRouter.getInstance(), treeModel, treeLis);
         ComponentUtil.centerComponentInWindow(main, 80);
-        main.show();
+        main.setVisible(true);
         ActionRouter.getInstance().actionPerformed(new ActionEvent(main, 1, ActionNames.ADD_ALL));
         if (testFile != null) {
             FileInputStream reader = null;
@@ -547,11 +547,11 @@ public class JMeter implements JMeterPlugin {
         // Process command line property definitions
         // These can potentially occur multiple times
 
-        List clOptions = parser.getArguments();
+        List<CLOption> clOptions = parser.getArguments();
         int size = clOptions.size();
 
         for (int i = 0; i < size; i++) {
-            CLOption option = (CLOption) clOptions.get(i);
+            CLOption option = clOptions.get(i);
             String name = option.getArgument(0);
             String value = option.getArgument(1);
             FileInputStream fis = null;
@@ -719,16 +719,17 @@ public class JMeter implements JMeterPlugin {
 
             HashTree tree = SaveService.loadTree(reader);
 
+            @SuppressWarnings("deprecation") // Deliberate use of deprecated ctor
             JMeterTreeModel treeModel = new JMeterTreeModel(new Object());// Create non-GUI version to avoid headless problems
             JMeterTreeNode root = (JMeterTreeNode) treeModel.getRoot();
             treeModel.addSubTree(tree, root);
 
             // Hack to resolve ModuleControllers in non GUI mode
-            SearchByClass replaceableControllers = new SearchByClass(ReplaceableController.class);
+            SearchByClass<ReplaceableController> replaceableControllers = new SearchByClass<ReplaceableController>(ReplaceableController.class);
             tree.traverse(replaceableControllers);
-            Collection replaceableControllersRes = replaceableControllers.getSearchResults();
-            for (Iterator iter = replaceableControllersRes.iterator(); iter.hasNext();) {
-                ReplaceableController replaceableController = (ReplaceableController) iter.next();
+            Collection<ReplaceableController> replaceableControllersRes = replaceableControllers.getSearchResults();
+            for (Iterator<ReplaceableController> iter = replaceableControllersRes.iterator(); iter.hasNext();) {
+                ReplaceableController replaceableController = iter.next();
                 replaceableController.resolveReplacementSubTree(root);
             }
 
@@ -736,19 +737,27 @@ public class JMeter implements JMeterPlugin {
             // For GUI runs this is done in Start.java
             convertSubTree(tree);
 
-            if (logFile != null) {
-                ResultCollector logger = new ResultCollector();
-                logger.setFilename(logFile);
-                tree.add(tree.getArray()[0], logger);
-            }
+            Summariser summer = null;
             String summariserName = JMeterUtils.getPropDefault("summariser.name", "");//$NON-NLS-1$
             if (summariserName.length() > 0) {
                 log.info("Creating summariser <" + summariserName + ">");
                 println("Creating summariser <" + summariserName + ">");
-                Summariser summer = new Summariser(summariserName);
-                tree.add(tree.getArray()[0], summer);
+                summer = new Summariser(summariserName);
             }
-            List engines = new LinkedList();
+
+            if (logFile != null) {
+                ResultCollector logger = new ResultCollector(summer);
+                logger.setFilename(logFile);
+                tree.add(tree.getArray()[0], logger);
+            }
+            else {
+                // only add Summariser if it can not be shared with the ResultCollector
+                if (summer != null) {
+                    tree.add(tree.getArray()[0], summer);
+                }
+            }
+
+            List<JMeterEngine> engines = new LinkedList<JMeterEngine>();
             tree.add(tree.getArray()[0], new ListenToTest(parent, (remoteStart && remoteStop) ? engines : null));
             println("Created the tree successfully using "+testFile);
             JMeterEngine engine = null;
@@ -769,9 +778,9 @@ public class JMeter implements JMeterPlugin {
                 println("Starting remote engines");
                 long now=System.currentTimeMillis();
                 println("Starting the test @ "+new Date(now)+" ("+now+")");
-                Iterator iter = engines.iterator();
+                Iterator<JMeterEngine> iter = engines.iterator();
                 while (iter.hasNext()) {
-                    engine = (JMeterEngine) iter.next();
+                    engine = iter.next();
                     if (engine != null){
                         engine.runTest();
                     }
@@ -792,7 +801,7 @@ public class JMeter implements JMeterPlugin {
      * @param tree
      */
     public static void convertSubTree(HashTree tree) {
-        Iterator iter = new LinkedList(tree.list()).iterator();
+        Iterator<Object> iter = new LinkedList<Object>(tree.list()).iterator();
         while (iter.hasNext()) {
             Object o = iter.next();
             if (o instanceof TestElement) {
@@ -833,7 +842,7 @@ public class JMeter implements JMeterPlugin {
                     // Replacement only needs to occur when starting the engine
                     // @see StandardJMeterEngine.run()
                     if (item.getUserObject() instanceof ReplaceableController) {
-                        ReplaceableController rc = 
+                        ReplaceableController rc =
                             (ReplaceableController) item.getTestElement();
                         HashTree subTree = tree.getTree(item);
 
@@ -883,9 +892,9 @@ public class JMeter implements JMeterPlugin {
 
         //NOT YET USED private JMeter _parent;
 
-        private List engines;
+        private List<JMeterEngine> engines;
 
-        public ListenToTest(JMeter parent, List engines) {
+        public ListenToTest(JMeter parent, List<JMeterEngine> engines) {
             //_parent = parent;
             this.engines=engines;
         }
@@ -934,9 +943,9 @@ public class JMeter implements JMeterPlugin {
              */
             if (engines!=null){ // it will be null unless remoteStop = true
                 System.out.println("Exitting remote servers");
-                Iterator it = engines.iterator();
+                Iterator<JMeterEngine> it = engines.iterator();
                 while(it.hasNext()){
-                    JMeterEngine e = (JMeterEngine) it.next();
+                    JMeterEngine e = it.next();
                     e.exit();
                 }
             }
@@ -989,7 +998,7 @@ public class JMeter implements JMeterPlugin {
         }
         log.info("Loaded icon properties from " + iconProp);
         String[][] iconlist = new String[p.size()][3];
-        Enumeration pe = p.keys();
+        Enumeration<?> pe = p.keys();
         int i = 0;
         while (pe.hasMoreElements()) {
             String key = (String) pe.nextElement();
@@ -1010,7 +1019,7 @@ public class JMeter implements JMeterPlugin {
 
     /**
      * Check if JMeter is running in non-GUI mode.
-     * 
+     *
      * @return true if JMeter is running in non-GUI mode.
      */
     public static boolean isNonGUI(){
