@@ -24,15 +24,14 @@ import java.net.DatagramPacket;
 import java.net.DatagramSocket;
 import java.net.InetAddress;
 import java.util.ArrayList;
-import java.util.Collections;
 import java.util.Date;
-import java.util.HashMap;
 import java.util.HashSet;
 import java.util.Iterator;
 import java.util.LinkedList;
 import java.util.List;
 import java.util.Map;
 import java.util.Properties;
+import java.util.concurrent.ConcurrentHashMap;
 
 import org.apache.jmeter.JMeter;
 import org.apache.jmeter.testbeans.TestBean;
@@ -45,7 +44,7 @@ import org.apache.jmeter.threads.JMeterThread;
 import org.apache.jmeter.threads.JMeterThreadMonitor;
 import org.apache.jmeter.threads.ListenerNotifier;
 import org.apache.jmeter.threads.TestCompiler;
-import org.apache.jmeter.threads.ThreadGroup;
+import org.apache.jmeter.threads.AbstractThreadGroup;
 import org.apache.jmeter.util.JMeterUtils;
 import org.apache.jorphan.collections.HashTree;
 import org.apache.jorphan.collections.ListedHashTree;
@@ -90,10 +89,10 @@ public class StandardJMeterEngine implements JMeterEngine, JMeterThreadMonitor,
      * Only used by the function parser so far.
      * The list is merged with the testListeners and then cleared.
      */
-    private static final List testList = new ArrayList();
+    private static final List<TestListener> testList = new ArrayList<TestListener>();
 
     /** JMeterThread => its JVM thread */
-    private final Map/*<JMeterThread, Thread>*/ allThreads;
+    private final Map<JMeterThread, Thread> allThreads;
 
     /** flag to show that groups are still being created, i.e test plan is not complete */
     private volatile boolean startingGroups;
@@ -106,7 +105,7 @@ public class StandardJMeterEngine implements JMeterEngine, JMeterThreadMonitor,
 
     private HashTree test;
 
-    private volatile SearchByClass testListenersSave;
+    private volatile SearchByClass<TestListener> testListenersSave;
 
     private final String host;
 
@@ -140,9 +139,9 @@ public class StandardJMeterEngine implements JMeterEngine, JMeterThreadMonitor,
         }
         JMeterThread thrd=null;
         synchronized (engine.allThreads) { // Protect iterator
-            Iterator iter = engine.allThreads.keySet().iterator();
+            Iterator<JMeterThread> iter = engine.allThreads.keySet().iterator();
             while(iter.hasNext()){
-                thrd = (JMeterThread) iter.next();
+                thrd = iter.next();
                 if (thrd.getThreadName().equals(threadName)){
                     break; // Found matching thread
                 }
@@ -152,7 +151,7 @@ public class StandardJMeterEngine implements JMeterEngine, JMeterThreadMonitor,
             thrd.stop();
             thrd.interrupt();
             if (now) {
-                Thread t = (Thread) engine.allThreads.get(thrd);
+                Thread t = engine.allThreads.get(thrd);
                 if (t != null) {
                     t.interrupt();
                 }
@@ -170,7 +169,7 @@ public class StandardJMeterEngine implements JMeterEngine, JMeterThreadMonitor,
 
     public StandardJMeterEngine(String host) {
         this.host = host;
-        this.allThreads = Collections.synchronizedMap(new HashMap());
+        this.allThreads = new ConcurrentHashMap<JMeterThread, Thread>();
         // Hack to allow external control
         engine = this;
     }
@@ -185,7 +184,8 @@ public class StandardJMeterEngine implements JMeterEngine, JMeterThreadMonitor,
             super(s);
           }
 
-          public void uncaughtException(Thread t, Throwable e) {
+          @Override
+        public void uncaughtException(Thread t, Throwable e) {
             if (!(e instanceof ThreadDeath)) {
                 log.error("Uncaught exception: ", e);
                 System.err.println("Uncaught Exception " + e + ". See log file for details.");
@@ -203,6 +203,7 @@ public class StandardJMeterEngine implements JMeterEngine, JMeterThreadMonitor,
             runningThread.start();
             if (JMeter.isNonGUI() && UDP_PORT > 1000){
                 Thread waiter = new Thread(){
+                    @Override
                     public void run() {
                         waitForSignals();
                     }
@@ -249,14 +250,14 @@ public class StandardJMeterEngine implements JMeterEngine, JMeterThreadMonitor,
                 socket.close();
             }
         }
-        
+
     }
 
-    private void removeThreadGroups(List elements) {
-        Iterator iter = elements.iterator();
+    private void removeThreadGroups(List<?> elements) {
+        Iterator<?> iter = elements.iterator();
         while (iter.hasNext()) {
             Object item = iter.next();
-            if (item instanceof ThreadGroup) {
+            if (item instanceof AbstractThreadGroup) {
                 iter.remove();
             } else if (!(item instanceof TestElement)) {
                 iter.remove();
@@ -264,10 +265,11 @@ public class StandardJMeterEngine implements JMeterEngine, JMeterThreadMonitor,
         }
     }
 
-    private void notifyTestListenersOfStart(SearchByClass testListeners) {
-        Iterator iter = testListeners.getSearchResults().iterator();
+    @SuppressWarnings("deprecation") // Deliberate use of deprecated method
+    private void notifyTestListenersOfStart(SearchByClass<TestListener> testListeners) {
+        Iterator<TestListener> iter = testListeners.getSearchResults().iterator();
         while (iter.hasNext()) {
-            TestListener tl = (TestListener) iter.next();
+            TestListener tl = iter.next();
             if (tl instanceof TestBean) {
                 TestBeanHelper.prepare((TestElement) tl);
             }
@@ -279,11 +281,11 @@ public class StandardJMeterEngine implements JMeterEngine, JMeterThreadMonitor,
         }
     }
 
-    private void notifyTestListenersOfEnd(SearchByClass testListeners) {
+    private void notifyTestListenersOfEnd(SearchByClass<TestListener> testListeners) {
         log.info("Notifying test listeners of end of test");
-        Iterator iter = testListeners.getSearchResults().iterator();
+        Iterator<TestListener> iter = testListeners.getSearchResults().iterator();
         while (iter.hasNext()) {
-            TestListener tl = (TestListener) iter.next();
+            TestListener tl = iter.next();
             try {
                 if (host == null) {
                     tl.testEnded();
@@ -322,10 +324,11 @@ public class StandardJMeterEngine implements JMeterEngine, JMeterThreadMonitor,
         allThreads.remove(thread);
         if (!startingGroups && allThreads.size() == 0 ) {// All threads have exitted
             new Thread(){// Ensure that the current sampler thread can exit cleanly
+                @Override
                 public void run() {
                     log.info("Stopping test");
                     notifyTestListenersOfEnd(testListenersSave);
-                }                    
+                }
             }.start();
         }
     }
@@ -349,17 +352,17 @@ public class StandardJMeterEngine implements JMeterEngine, JMeterThreadMonitor,
         public void run() {
             running = false;
             engine = null;
+            boolean stopped = false;
             if (now) {
                 tellThreadsToStop();
                 pause(10 * allThreads.size());
-                boolean stopped = verifyThreadsStopped();
+                stopped = verifyThreadsStopped();
                 if (!stopped) {
-                    notifyTestListenersOfEnd(testListenersSave);
                     if (JMeter.isNonGUI()) {
                         exit();
                     } else {
                         JMeterUtils.reportErrorToUser(
-                                JMeterUtils.getResString("stopping_test_failed"), 
+                                JMeterUtils.getResString("stopping_test_failed"),
                                 JMeterUtils.getResString("stopping_test_title"));
                         // TODO - perhaps allow option to stop them?
                     }
@@ -367,6 +370,10 @@ public class StandardJMeterEngine implements JMeterEngine, JMeterThreadMonitor,
             } else {
                 stopAllThreads();
             }
+            //  for TGs which run consecutively or when thread don't stop 
+            if (serialized || !stopped) {
+                notifyTestListenersOfEnd(testListenersSave);
+            }
         }
     }
 
@@ -374,7 +381,7 @@ public class StandardJMeterEngine implements JMeterEngine, JMeterThreadMonitor,
         log.info("Running the test!");
         running = true;
 
-        SearchByClass testPlan = new SearchByClass(TestPlan.class);
+        SearchByClass<TestPlan> testPlan = new SearchByClass<TestPlan>(TestPlan.class);
         test.traverse(testPlan);
         Object[] plan = testPlan.getSearchResults().toArray();
         if (plan.length == 0) {
@@ -398,7 +405,7 @@ public class StandardJMeterEngine implements JMeterEngine, JMeterThreadMonitor,
          * Notification of test listeners needs to happen after function
          * replacement, but before setting RunningVersion to true.
          */
-        SearchByClass testListeners = new SearchByClass(TestListener.class);
+        SearchByClass<TestListener> testListeners = new SearchByClass<TestListener>(TestListener.class);
         test.traverse(testListeners);
 
         // Merge in any additional test listeners
@@ -412,15 +419,15 @@ public class StandardJMeterEngine implements JMeterEngine, JMeterThreadMonitor,
         test.traverse(new TurnElementsOn());
         if (startListenersLater) { notifyTestListenersOfStart(testListeners); }
 
-        List testLevelElements = new LinkedList(test.list(test.getArray()[0]));
+        List<?> testLevelElements = new LinkedList<Object>(test.list(test.getArray()[0]));
         removeThreadGroups(testLevelElements);
-        SearchByClass searcher = new SearchByClass(ThreadGroup.class);
+        SearchByClass<AbstractThreadGroup> searcher = new SearchByClass<AbstractThreadGroup>(AbstractThreadGroup.class);
         test.traverse(searcher);
         TestCompiler.initialize();
         // for each thread group, generate threads
         // hand each thread the sampler controller
         // and the listeners, and the timer
-        Iterator iter = searcher.getSearchResults().iterator();
+        Iterator<AbstractThreadGroup> iter = searcher.getSearchResults().iterator();
 
         /*
          * Here's where the test really starts. Run a Full GC now: it's no harm
@@ -437,16 +444,14 @@ public class StandardJMeterEngine implements JMeterEngine, JMeterThreadMonitor,
         startingGroups = true;
         while (running && iter.hasNext()) {// for each thread group
             groupCount++;
-            ThreadGroup group = (ThreadGroup) iter.next();
+            AbstractThreadGroup group = iter.next();
             int numThreads = group.getNumThreads();
             JMeterContextService.addTotalThreads(numThreads);
             boolean onErrorStopTest = group.getOnErrorStopTest();
             boolean onErrorStopTestNow = group.getOnErrorStopTestNow();
             boolean onErrorStopThread = group.getOnErrorStopThread();
             String groupName = group.getName();
-            int rampUp = group.getRampUp();
-            float perThreadDelay = ((float) (rampUp * 1000) / (float) numThreads);
-            log.info("Starting " + numThreads + " threads for group " + groupName + ". Ramp up = " + rampUp + ".");
+            log.info("Starting " + numThreads + " threads for group " + groupName + ".");
 
             if (onErrorStopTest) {
                 log.info("Test will stop on error");
@@ -457,7 +462,6 @@ public class StandardJMeterEngine implements JMeterEngine, JMeterThreadMonitor,
             } else {
                 log.info("Thread will continue on error");
             }
-
             ListedHashTree threadGroupTree = (ListedHashTree) searcher.getSubTree(group);
             threadGroupTree.add(group, testLevelElements);
             for (int i = 0; running && i < numThreads; i++) {
@@ -465,18 +469,15 @@ public class StandardJMeterEngine implements JMeterEngine, JMeterThreadMonitor,
                 jmeterThread.setThreadNum(i);
                 jmeterThread.setThreadGroup(group);
                 jmeterThread.setInitialContext(JMeterContextService.getContext());
-                jmeterThread.setInitialDelay((int) (perThreadDelay * i));
                 final String threadName = groupName + " " + (groupCount) + "-" + (i + 1);
                 jmeterThread.setThreadName(threadName);
-
-                scheduleThread(jmeterThread, group);
-
-                // Set up variables for stop handling
                 jmeterThread.setEngine(this);
                 jmeterThread.setOnErrorStopTest(onErrorStopTest);
                 jmeterThread.setOnErrorStopTestNow(onErrorStopTestNow);
                 jmeterThread.setOnErrorStopThread(onErrorStopThread);
 
+                group.scheduleThread(jmeterThread);
+
                 Thread newThread = new Thread(jmeterThread);
                 newThread.setName(threadName);
                 allThreads.put(jmeterThread, newThread);
@@ -489,61 +490,33 @@ public class StandardJMeterEngine implements JMeterEngine, JMeterThreadMonitor,
                 }
             }
         } // end of thread groups
-        if (running) {
-            log.info("All threads have been started");
-        } else {
-            log.info("Test stopped - no more threads will be started");
-        }
         startingGroups = false;
-    }
-
-    /**
-     * This will schedule the time for the JMeterThread.
-     *
-     * @param thread
-     * @param group
-     */
-    private void scheduleThread(JMeterThread thread, ThreadGroup group) {
-        // if true the Scheduler is enabled
-        if (group.getScheduler()) {
-            long now = System.currentTimeMillis();
-            // set the start time for the Thread
-            if (group.getDelay() > 0) {// Duration is in seconds
-                thread.setStartTime(group.getDelay() * 1000 + now);
+        if (groupCount == 0){ // No TGs found
+            log.info("No enabled thread groups found");
+            notifyTestListenersOfEnd(testListenersSave);
+        } else {
+            if (running) {
+                log.info("All threads have been started");
             } else {
-                long start = group.getStartTime();
-                if (start < now) {
-                    start = now; // Force a sensible start time
-                }
-                thread.setStartTime(start);
+                log.info("Test stopped - no more threads will be started");
             }
-
-            // set the endtime for the Thread
-            if (group.getDuration() > 0) {// Duration is in seconds
-                thread.setEndTime(group.getDuration() * 1000 + (thread.getStartTime()));
-            } else {
-                thread.setEndTime(group.getEndTime());
-            }
-
-            // Enables the scheduler
-            thread.setScheduled(true);
         }
     }
 
     private boolean verifyThreadsStopped() {
         boolean stoppedAll = true;
-        List/*<Thread>*/ threadsToCheck = new ArrayList/*<Thread>*/(allThreads.size());
+        List<Thread> threadsToCheck = new ArrayList<Thread>(allThreads.size());
         synchronized (allThreads) { // Protect iterator
-            Iterator/*<Thread>*/ iter = allThreads.keySet().iterator();
+            Iterator<JMeterThread> iter = allThreads.keySet().iterator();
             while (iter.hasNext()) {
-                Thread t = (Thread) allThreads.get(iter.next());
+                Thread t = allThreads.get(iter.next());
                 if (t != null) {
                     threadsToCheck.add(t); // Do work later to reduce time in synch block.
                 }
             }
         }
         for(int i=0; i < threadsToCheck.size(); i++) {
-            Thread t = (Thread) threadsToCheck.get(i);
+            Thread t = threadsToCheck.get(i);
             if (t.isAlive()) {
                 try {
                     t.join(WAIT_TO_DIE);
@@ -560,26 +533,28 @@ public class StandardJMeterEngine implements JMeterEngine, JMeterThreadMonitor,
 
     private void tellThreadsToStop() {
         synchronized (allThreads) { // Protect iterator
-            Iterator iter = new HashSet(allThreads.keySet()).iterator();
+            Iterator<JMeterThread> iter = new HashSet<JMeterThread>(allThreads.keySet()).iterator();
             while (iter.hasNext()) {
-                JMeterThread item = (JMeterThread) iter.next();
+                JMeterThread item = iter.next();
                 item.stop(); // set stop flag
                 item.interrupt(); // interrupt sampler if possible
-                Thread t = (Thread) allThreads.get(item);
+                Thread t = allThreads.get(item);
                 t.interrupt(); // also interrupt JVM thread
             }
         }
     }
 
     public void askThreadsToStop() {
-        engine.stopTest(false);
+        if (engine != null) { // Will be null if StopTest thread has started
+            engine.stopTest(false);
+        }
     }
 
     private void stopAllThreads() {
         synchronized (allThreads) {// Protect iterator
-            Iterator iter = new HashSet(allThreads.keySet()).iterator();
+            Iterator<JMeterThread> iter = new HashSet<JMeterThread>(allThreads.keySet()).iterator();
             while (iter.hasNext()) {
-                JMeterThread item = (JMeterThread) iter.next();
+                JMeterThread item = iter.next();
                 item.stop(); // This is quick
             }
         }
@@ -589,6 +564,7 @@ public class StandardJMeterEngine implements JMeterEngine, JMeterThreadMonitor,
     public void exit() {
         // Needs to be run in a separate thread to allow RMI call to return OK
         Thread t = new Thread() {
+            @Override
             public void run() {
                 // log.info("Pausing");
                 pause(1000); // Allow RMI to complete
@@ -604,7 +580,7 @@ public class StandardJMeterEngine implements JMeterEngine, JMeterThreadMonitor,
         try {
             Thread.sleep(ms);
         } catch (InterruptedException e) {
-        }        
+        }
     }
 
     public void setProperties(Properties p) {
