@@ -74,6 +74,8 @@ import org.apache.jorphan.gui.ComponentUtil;
  */
 public class MainFrame extends JFrame implements TestListener, Remoteable {
 
+    private static final long serialVersionUID = 240L;
+
     // This is used to keep track of local (non-remote) tests
     // The name is chosen to be an unlikely host-name
     private static final String LOCAL = "*local*"; // $NON-NLS-1$
@@ -110,7 +112,7 @@ public class MainFrame extends JFrame implements TestListener, Remoteable {
     private int previousDragYLocation = 0;
 
     /** The set of currently running hosts. */
-    private Set hosts = new HashSet();
+    private Set<String> hosts = new HashSet<String>();
 
     /** A message dialog shown while JMeter threads are stopping. */
     private JDialog stoppingMessage;
@@ -154,6 +156,7 @@ public class MainFrame extends JFrame implements TestListener, Remoteable {
      *
      * @deprecated Do not use - only needed for JUnit tests
      */
+    @Deprecated
     public MainFrame() {
     }
 
@@ -289,7 +292,7 @@ public class MainFrame extends JFrame implements TestListener, Remoteable {
         SwingUtilities.invokeLater(new Runnable() {
             public void run() {
                 if (stoppingMessage != null) {// TODO - how can this be null?
-                    stoppingMessage.show();
+                    stoppingMessage.setVisible(true);
                 }
             }
         });
@@ -468,6 +471,9 @@ public class MainFrame extends JFrame implements TestListener, Remoteable {
      */
     private JTree makeTree(TreeModel treeModel, JMeterTreeListener treeListener) {
         JTree treevar = new JTree(treeModel) {
+            private static final long serialVersionUID = 240L;
+
+            @Override
             public String getToolTipText(MouseEvent event) {
                 TreePath path = this.getPathForLocation(event.getX(), event.getY());
                 if (path != null) {
@@ -542,6 +548,7 @@ public class MainFrame extends JFrame implements TestListener, Remoteable {
          * @param event
          *            the WindowEvent to handle
          */
+        @Override
         public void windowClosing(WindowEvent event) {
             ActionRouter.getInstance().actionPerformed(new ActionEvent(this, event.getID(), ActionNames.EXIT));
         }
