@@ -48,6 +48,8 @@ import org.apache.jorphan.logging.LoggingManager;
 import org.apache.log.Logger;
 
 public class JMeterMenuBar extends JMenuBar implements LocaleChangeListener {
+    private static final long serialVersionUID = 240L;
+
     private static final Logger log = LoggingManager.getLoggerForClass();
 
     private JMenu fileMenu;
@@ -60,7 +62,7 @@ public class JMeterMenuBar extends JMenuBar implements LocaleChangeListener {
 
     private JMenuItem file_load;
 
-    private List file_load_recent_files;
+    private List<JComponent> file_load_recent_files;
 
     private JMenuItem file_merge;
 
@@ -83,7 +85,7 @@ public class JMeterMenuBar extends JMenuBar implements LocaleChangeListener {
 
     private JMenuItem remote_start_all;
 
-    private Collection remote_engine_start;
+    private Collection<JMenuItem> remote_engine_start;
 
     private JMenuItem run_stop;
 
@@ -93,7 +95,7 @@ public class JMeterMenuBar extends JMenuBar implements LocaleChangeListener {
 
     private JMenuItem remote_stop_all;
 
-    private Collection remote_engine_stop;
+    private Collection<JMenuItem> remote_engine_stop;
 
     private JMenuItem run_clear;
 
@@ -117,15 +119,15 @@ public class JMeterMenuBar extends JMenuBar implements LocaleChangeListener {
 
     private JMenuItem remote_exit_all;
 
-    private Collection remote_engine_exit;
+    private Collection<JMenuItem> remote_engine_exit;
 
     public JMeterMenuBar() {
         // List for recent files menu items
-        file_load_recent_files = new LinkedList();
+        file_load_recent_files = new LinkedList<JComponent>();
         // Lists for remote engines menu items
-        remote_engine_start = new LinkedList();
-        remote_engine_stop = new LinkedList();
-        remote_engine_exit = new LinkedList();
+        remote_engine_start = new LinkedList<JMenuItem>();
+        remote_engine_stop = new LinkedList<JMenuItem>();
+        remote_engine_exit = new LinkedList<JMenuItem>();
         remoteHosts = JOrphanUtils.split(JMeterUtils.getPropDefault("remote_hosts", ""), ","); //$NON-NLS-1$
         if (remoteHosts.length == 1 && remoteHosts[0].equals("")) {
             remoteHosts = new String[0];
@@ -254,7 +256,7 @@ public class JMeterMenuBar extends JMenuBar implements LocaleChangeListener {
     private void makeHelpMenu() {
         // HELP MENU
         helpMenu = makeMenuRes("help",'H'); //$NON-NLS-1$
-        
+
         JMenuItem contextHelp = makeMenuItemRes("help", 'H', ActionNames.HELP, KeyStrokes.HELP); //$NON-NLS-1$
 
         JMenuItem whatClass = makeMenuItemRes("help_node", 'W', ActionNames.WHAT_CLASS, KeyStrokes.WHAT_CLASS);//$NON-NLS-1$
@@ -305,14 +307,14 @@ public class JMeterMenuBar extends JMenuBar implements LocaleChangeListener {
     private static class LangMenuHelper{
         final ActionRouter actionRouter = ActionRouter.getInstance();
         final JMenu languageMenu;
-        
+
         LangMenuHelper(JMenu _languageMenu){
             languageMenu = _languageMenu;
         }
-        
+
         /**
          * Create a language entry from the locale name.
-         * 
+         *
          * @param locale - must also be a valid resource name
          */
         void addLang(String locale){
@@ -321,19 +323,19 @@ public class JMeterMenuBar extends JMenuBar implements LocaleChangeListener {
             language.addActionListener(actionRouter);
             language.setActionCommand(ActionNames.CHANGE_LANGUAGE);
             language.setName(locale); // This is used by the ChangeLanguage class to define the Locale
-            languageMenu.add(language);                
+            languageMenu.add(language);
         }
 
    }
 
     /**
      * Generate the list of supported languages.
-     * 
+     *
      * @return list of languages
      */
     // Also used by org.apache.jmeter.resources.PackageTest
     public static String[] getLanguages(){
-        List lang = new ArrayList(20);
+        List<String> lang = new ArrayList<String>(20);
         lang.add(Locale.ENGLISH.toString()); // en
         lang.add(Locale.FRENCH.toString()); // fr
         lang.add(Locale.GERMAN.toString()); // de
@@ -353,12 +355,12 @@ public class JMeterMenuBar extends JMenuBar implements LocaleChangeListener {
                 lang.add(addLanguages[i]);
             }
         }
-        return (String[]) lang.toArray(new String[lang.size()]);
+        return lang.toArray(new String[lang.size()]);
     }
 
     static JMenu makeLanguageMenu() {
         final JMenu languageMenu = makeMenuRes("choose_language",'C'); //$NON-NLS-1$
-        
+
         LangMenuHelper langMenu = new LangMenuHelper(languageMenu);
 
         /*
@@ -429,7 +431,7 @@ public class JMeterMenuBar extends JMenuBar implements LocaleChangeListener {
     private void makeFileMenu() {
         // FILE MENU
         fileMenu = makeMenuRes("file",'F'); //$NON-NLS-1$
-        
+
         JMenuItem file_save = makeMenuItemRes("save", 'S', ActionNames.SAVE, KeyStrokes.SAVE); //$NON-NLS-1$
         file_save.setEnabled(true);
 
@@ -470,8 +472,8 @@ public class JMeterMenuBar extends JMenuBar implements LocaleChangeListener {
         // Add the recent files, which will also add a separator that is
         // visible when needed
         file_load_recent_files = LoadRecentProject.getRecentFileMenuItems();
-        for(Iterator i = file_load_recent_files.iterator(); i.hasNext();) {
-            fileMenu.add((JComponent)i.next());
+        for(Iterator<JComponent> i = file_load_recent_files.iterator(); i.hasNext();) {
+            fileMenu.add(i.next());
         }
         fileMenu.add(file_exit);
     }
@@ -479,13 +481,13 @@ public class JMeterMenuBar extends JMenuBar implements LocaleChangeListener {
     public void setRunning(boolean running, String host) {
         log.info("setRunning(" + running + "," + host + ")");
 
-        Iterator iter = remote_engine_start.iterator();
-        Iterator iter2 = remote_engine_stop.iterator();
-        Iterator iter3 = remote_engine_exit.iterator();
+        Iterator<JMenuItem> iter = remote_engine_start.iterator();
+        Iterator<JMenuItem> iter2 = remote_engine_stop.iterator();
+        Iterator<JMenuItem> iter3 = remote_engine_exit.iterator();
         while (iter.hasNext() && iter2.hasNext() && iter3.hasNext()) {
-            JMenuItem start = (JMenuItem) iter.next();
-            JMenuItem stop = (JMenuItem) iter2.next();
-            JMenuItem exit = (JMenuItem) iter3.next();
+            JMenuItem start = iter.next();
+            JMenuItem stop = iter2.next();
+            JMenuItem exit = iter3.next();
             if (start.getText().equals(host)) {
                 log.debug("Found start host: " + start.getText());
                 start.setEnabled(!running);
@@ -501,6 +503,8 @@ public class JMeterMenuBar extends JMenuBar implements LocaleChangeListener {
         }
     }
 
+    /** {@inheritDoc} */
+    @Override
     public void setEnabled(boolean enable) {
         run_start.setEnabled(!enable);
         run_stop.setEnabled(enable);
@@ -533,10 +537,7 @@ public class JMeterMenuBar extends JMenuBar implements LocaleChangeListener {
         }
     }
 
-    /**
-     * Processes a locale change notification. 
-     * Changes the texts in all menus to the new language.
-     */
+    /** {@inheritDoc} */
     public void localeChanged(LocaleChangeEvent event) {
         updateMenuElement(fileMenu);
         updateMenuElement(editMenu);
@@ -547,13 +548,13 @@ public class JMeterMenuBar extends JMenuBar implements LocaleChangeListener {
 
     /**
      * <p>Refreshes all texts in the menu and all submenus to a new locale.</p>
-     * 
+     *
      * <p>Assumes that the item name is set to the resource key, so the resource can be retrieved.
      * Certain action types do not follow this rule, @see JMeterMenuBar#isNotResource(String)</p>
      *
      * The Language Change event assumes that the name is the same as the locale name,
      * so this additionally means that all supported locales must be defined as resources.
-     * 
+     *
      */
     private void updateMenuElement(MenuElement menu) {
         Component component = menu.getComponent();
@@ -584,14 +585,14 @@ public class JMeterMenuBar extends JMenuBar implements LocaleChangeListener {
     /**
      * Return true if component name is a resource.<br/>
      * i.e it is not a hostname:<br/>
-     * 
+     *
      * <tt>ActionNames.REMOTE_START</tt><br/>
      * <tt>ActionNames.REMOTE_STOP</tt><br/>
      * <tt>ActionNames.REMOTE_EXIT</tt><br/>
-     * 
+     *
      * nor a filename:<br/>
      * <tt>ActionNames.OPEN_RECENT</tt>
-     * 
+     *
      * nor a look and feel prefix:<br/>
      * <tt>ActionNames.LAF_PREFIX</tt>
      */
@@ -633,7 +634,7 @@ public class JMeterMenuBar extends JMenuBar implements LocaleChangeListener {
 
     /**
      * Make a menu from a resource string and set its mnemonic.
-     * 
+     *
      * @param resource
      * @param mnemonic
      * @return the menu
