@@ -22,6 +22,7 @@ import java.awt.Component;
 import java.io.IOException;
 import java.util.Collection;
 import java.util.Collections;
+import java.util.Comparator;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.Iterator;
@@ -64,6 +65,8 @@ public final class MenuFactory {
      *  These are used as menu categories in the menuMap Hashmap,
      *  and also for resource lookup in messages.properties
     */
+    public static final String THREADS = "menu_threads"; //$NON-NLS-1$
+
     public static final String TIMERS = "menu_timer"; //$NON-NLS-1$
 
     public static final String CONTROLLERS = "menu_logic_controller"; //$NON-NLS-1$
@@ -82,13 +85,15 @@ public final class MenuFactory {
 
     public static final String LISTENERS = "menu_listener"; //$NON-NLS-1$
 
-    private static final Map menuMap = new HashMap();
+    private static final Map<String, List<MenuInfo>> menuMap =
+        new HashMap<String, List<MenuInfo>>();
 
-    private static final Set elementsToSkip = new HashSet();
+    private static final Set<String> elementsToSkip = new HashSet<String>();
 
     // MENU_ADD_xxx - controls which items are in the ADD menu
     // MENU_PARENT_xxx - controls which items are in the Insert Parent menu
     private static final String[] MENU_ADD_CONTROLLER = new String[] {
+        MenuFactory.THREADS,
         MenuFactory.CONTROLLERS,
         MenuFactory.CONFIG_ELEMENTS,
         MenuFactory.TIMERS,
@@ -114,20 +119,22 @@ public final class MenuFactory {
     private static final String[] MENU_PARENT_SAMPLER = new String[] {
         MenuFactory.CONTROLLERS };
 
-    private static final List timers, controllers, samplers, configElements,
-        assertions, listeners, nonTestElements,
+    private static final List<MenuInfo> timers, controllers, samplers, threads,
+        configElements, assertions, listeners, nonTestElements,
         postProcessors, preProcessors;
 
     static {
-        timers = new LinkedList();
-        controllers = new LinkedList();
-        samplers = new LinkedList();
-        configElements = new LinkedList();
-        assertions = new LinkedList();
-        listeners = new LinkedList();
-        postProcessors = new LinkedList();
-        preProcessors = new LinkedList();
-        nonTestElements = new LinkedList();
+        threads = new LinkedList<MenuInfo>();
+        timers = new LinkedList<MenuInfo>();
+        controllers = new LinkedList<MenuInfo>();
+        samplers = new LinkedList<MenuInfo>();
+        configElements = new LinkedList<MenuInfo>();
+        assertions = new LinkedList<MenuInfo>();
+        listeners = new LinkedList<MenuInfo>();
+        postProcessors = new LinkedList<MenuInfo>();
+        preProcessors = new LinkedList<MenuInfo>();
+        nonTestElements = new LinkedList<MenuInfo>();
+        menuMap.put(THREADS, threads);
         menuMap.put(TIMERS, timers);
         menuMap.put(ASSERTIONS, assertions);
         menuMap.put(CONFIG_ELEMENTS, configElements);
@@ -145,8 +152,15 @@ public final class MenuFactory {
             }
 
             initializeMenus();
+            sortPluginMenus();
         } catch (Throwable e) {
             log.error("", e);
+            if (e instanceof Error){
+                throw (Error) e;
+            }
+            if (e instanceof RuntimeException){
+                throw (RuntimeException) e;
+            }
         }
     }
 
@@ -282,6 +296,13 @@ public final class MenuFactory {
         return pop;
     }
 
+    public static JPopupMenu getDefaultMenu() { // if type is unknown
+        JPopupMenu pop = new JPopupMenu();
+        MenuFactory.addEditMenu(pop, true);
+        MenuFactory.addFileMenu(pop);
+        return pop;
+    }
+
     /**
      * Create a menu from a menu category.
      *
@@ -291,7 +312,7 @@ public final class MenuFactory {
      * @return the menu
      */
     public static JMenu makeMenu(String category, String actionCommand) {
-        return makeMenu((Collection) menuMap.get(category), actionCommand, JMeterUtils.getResString(category));
+        return makeMenu(menuMap.get(category), actionCommand, JMeterUtils.getResString(category));
     }
 
     /**
@@ -303,11 +324,11 @@ public final class MenuFactory {
      * @param menuName
      * @return the menu
      */
-    public static JMenu makeMenu(Collection menuInfo, String actionCommand, String menuName) {
-        Iterator iter = menuInfo.iterator();
+    public static JMenu makeMenu(Collection<MenuInfo> menuInfo, String actionCommand, String menuName) {
         JMenu menu = new JMenu(menuName);
+        Iterator<MenuInfo> iter = menuInfo.iterator();
         while (iter.hasNext()) {
-            MenuInfo info = (MenuInfo) iter.next();
+            MenuInfo info = iter.next();
             menu.add(makeMenuItem(info, actionCommand));
         }
         return menu;
@@ -338,7 +359,7 @@ public final class MenuFactory {
 
         return newMenuChoice;
     }
-    
+
     /**
      * Create a single menu item from the resource name.
      *
@@ -391,12 +412,12 @@ public final class MenuFactory {
 
     private static void initializeMenus() {
         try {
-            List guiClasses = ClassFinder.findClassesThatExtend(JMeterUtils.getSearchPaths(), new Class[] {
+            List<String> guiClasses = ClassFinder.findClassesThatExtend(JMeterUtils.getSearchPaths(), new Class[] {
                     JMeterGUIComponent.class, TestBean.class });
             Collections.sort(guiClasses);
-            Iterator iter = guiClasses.iterator();
+            Iterator<String> iter = guiClasses.iterator();
             while (iter.hasNext()) {
-                String name = (String) iter.next();
+                String name = iter.next();
 
                 /*
                  * JMeterTreeNode and TestBeanGUI are special GUI classes, and
@@ -411,7 +432,7 @@ public final class MenuFactory {
 
                 JMeterGUIComponent item;
                 try {
-                    Class c = Class.forName(name);
+                    Class<?> c = Class.forName(name);
                     if (TestBean.class.isAssignableFrom(c)) {
                         item = new TestBeanGUI(c);
                     } else {
@@ -422,6 +443,12 @@ public final class MenuFactory {
                     continue;
                 } catch (Throwable e) {
                     log.warn("Could not instantiate " + name, e);
+                    if (e instanceof Error){
+                        throw (Error) e;
+                    }
+                    if (e instanceof RuntimeException){
+                        throw (RuntimeException) e;
+                    }
                     continue;
                 }
                 if (elementsToSkip.contains(name) || elementsToSkip.contains(item.getStaticLabel())) {
@@ -430,11 +457,14 @@ public final class MenuFactory {
                 } else {
                     elementsToSkip.add(name);
                 }
-                Collection categories = item.getMenuCategories();
+                Collection<String> categories = item.getMenuCategories();
                 if (categories == null) {
                     log.debug(name + " participates in no menus.");
                     continue;
                 }
+                if (categories.contains(THREADS)) {
+                    threads.add(new MenuInfo(item, name));
+                }
                 if (categories.contains(TIMERS)) {
                     timers.add(new MenuInfo(item, name));
                 }
@@ -525,14 +555,14 @@ public final class MenuFactory {
         if (parent instanceof TestPlan) {
             if (foundClass(nodes,
                      new Class[]{Sampler.class, Controller.class}, // Samplers and Controllers need not apply ...
-                     org.apache.jmeter.threads.ThreadGroup.class)  // but ThreadGroup (Controller) is OK
+                     org.apache.jmeter.threads.AbstractThreadGroup.class)  // but AbstractThreadGroup (Controller) is OK
                 ){
                 return false;
             }
             return true;
         }
-        // ThreadGroup is only allowed under a TestPlan
-        if (foundClass(nodes, new Class[]{org.apache.jmeter.threads.ThreadGroup.class})){
+        // AbstractThreadGroup is only allowed under a TestPlan
+        if (foundClass(nodes, new Class[]{org.apache.jmeter.threads.AbstractThreadGroup.class})){
             return false;
         }
         if (parent instanceof Controller) {// Includes thread group; anything goes
@@ -549,7 +579,7 @@ public final class MenuFactory {
     }
 
     // Is any node an instance of one of the classes?
-    private static boolean foundClass(JMeterTreeNode nodes[],Class classes[]){
+    private static boolean foundClass(JMeterTreeNode nodes[],Class<?> classes[]){
         for (int i = 0; i < nodes.length; i++) {
             JMeterTreeNode node = nodes[i];
             for (int j=0; j < classes.length; j++) {
@@ -562,7 +592,7 @@ public final class MenuFactory {
     }
 
     // Is any node an instance of one of the classes, but not an exception?
-    private static boolean foundClass(JMeterTreeNode nodes[],Class classes[], Class except){
+    private static boolean foundClass(JMeterTreeNode nodes[],Class<?> classes[], Class<?> except){
         for (int i = 0; i < nodes.length; i++) {
             JMeterTreeNode node = nodes[i];
             Object userObject = node.getUserObject();
@@ -611,4 +641,24 @@ public final class MenuFactory {
     static int elementsToSkip_size() {
         return elementsToSkip.size();
     }
+
+    /**
+     * Menu sort helper class
+     */
+    private static class MenuInfoComparator implements Comparator<MenuInfo> {
+       public int compare(MenuInfo o1, MenuInfo o2) {
+          return o1.getLabel().compareTo(o2.getLabel());
+}
+    }
+
+    /**
+     * Sort loaded menus
+     */
+    private static void sortPluginMenus() {
+       Iterator<List<MenuInfo>> it = menuMap.values().iterator();
+       while (it.hasNext()) {
+          List<MenuInfo> menuToSort = it.next();
+          Collections.sort(menuToSort, new MenuInfoComparator());
+       }
+    }
 }
