@@ -35,7 +35,6 @@ import java.util.HashMap;
 import java.util.Iterator;
 import java.util.Map;
 
-import org.apache.avalon.framework.configuration.DefaultConfigurationSerializer;
 import org.apache.jmeter.engine.event.LoopIterationEvent;
 import org.apache.jmeter.engine.util.NoThreadClone;
 import org.apache.jmeter.gui.GuiPackage;
@@ -46,7 +45,6 @@ import org.apache.jmeter.samplers.SampleListener;
 import org.apache.jmeter.samplers.SampleResult;
 import org.apache.jmeter.samplers.SampleSaveConfiguration;
 import org.apache.jmeter.save.CSVSaveService;
-import org.apache.jmeter.save.OldSaveService;
 import org.apache.jmeter.save.SaveService;
 import org.apache.jmeter.testelement.TestElement;
 import org.apache.jmeter.testelement.TestListener;
@@ -92,14 +90,14 @@ public class ResultCollector extends AbstractListenerElement implements SampleLi
     private static final String ERROR_LOGGING = "ResultCollector.error_logging"; // $NON-NLS-1$
 
     private static final String SUCCESS_ONLY_LOGGING = "ResultCollector.success_only_logging"; // $NON-NLS-1$
-    
+
     // Static variables
 
     // Lock used to guard static mutable variables
     private static final Object LOCK = new Object();
-    
+
     //@GuardedBy("LOCK")
-    private static final Map files = new HashMap(); // key=filename, entry=FileEntry
+    private static final Map<String, FileEntry> files = new HashMap<String, FileEntry>();
 
     /*
      * Keep track of the file writer and the configuration,
@@ -119,14 +117,12 @@ public class ResultCollector extends AbstractListenerElement implements SampleLi
      * The instance count is used to keep track of whether any tests are currently running.
      * It's not possible to use the constructor or threadStarted etc as tests may overlap
      * e.g. a remote test may be started,
-     * and then a local test started whilst the remote test is still running. 
+     * and then a local test started whilst the remote test is still running.
      */
     //@GuardedBy("LOCK")
     private static int instanceCount; // Keep track of how many instances are active
 
-    // Instance variables
-    
-    private transient volatile DefaultConfigurationSerializer serializer;
+    // Instance variables (guarded by volatile)
 
     private transient volatile PrintWriter out;
 
@@ -134,20 +130,31 @@ public class ResultCollector extends AbstractListenerElement implements SampleLi
 
     private volatile boolean isStats = false;
 
+    /** the summarizer to which this result collector will forward the samples */
+    private volatile Summariser summariser;
+
     /**
      * No-arg constructor.
      */
     public ResultCollector() {
+        this(null);
+    }
+
+    public ResultCollector(Summariser summer) {
         setErrorLogging(false);
         setSuccessOnlyLogging(false);
         setProperty(new ObjectProperty(SAVE_CONFIG, new SampleSaveConfiguration()));
+        summariser = summer;
     }
 
     // Ensure that the sample save config is not shared between copied nodes
     // N.B. clone only seems to be used for client-server tests
+    @Override
     public Object clone(){
         ResultCollector clone = (ResultCollector) super.clone();
         clone.setSaveConfig((SampleSaveConfiguration)clone.getSaveConfig().clone());
+        // Unfortunately AbstractTestElement does not call super.clone()
+        clone.summariser = this.summariser;
         return clone;
     }
 
@@ -163,11 +170,11 @@ public class ResultCollector extends AbstractListenerElement implements SampleLi
         return getPropertyAsBoolean(ERROR_LOGGING);
     }
 
-    public void setErrorLogging(boolean errorLogging) {
+    public final void setErrorLogging(boolean errorLogging) {
         setProperty(new BooleanProperty(ERROR_LOGGING, errorLogging));
     }
 
-    public void setSuccessOnlyLogging(boolean value) {
+    public final void setSuccessOnlyLogging(boolean value) {
         if (value) {
             setProperty(new BooleanProperty(SUCCESS_ONLY_LOGGING, true));
         } else {
@@ -233,11 +240,15 @@ public class ResultCollector extends AbstractListenerElement implements SampleLi
             if (instanceCount <= 0) {
                 finalizeFileOutput();
                 inTest = false;
-            }            
+            }
+        }
+
+        if(summariser != null) {
+            summariser.testEnded(host);
         }
     }
 
-    public synchronized void testStarted(String host) {
+    public void testStarted(String host) {
         synchronized(LOCK){
             instanceCount++;
             try {
@@ -250,6 +261,10 @@ public class ResultCollector extends AbstractListenerElement implements SampleLi
             }
         }
         inTest = true;
+
+        if(summariser != null) {
+            summariser.testStarted(host);
+        }
     }
 
     public void testEnded() {
@@ -294,17 +309,11 @@ public class ResultCollector extends AbstractListenerElement implements SampleLi
                     } else { // We are processing XML
                         try { // Assume XStream
                             bufferedInputStream = new BufferedInputStream(new FileInputStream(file));
-                            SaveService.loadTestResults(bufferedInputStream, 
+                            SaveService.loadTestResults(bufferedInputStream,
                                     new ResultCollectorHelper(this, visualizer));
                             parsedOK = true;
                         } catch (Exception e) {
-                            log.info("Failed to load "+filename+" using XStream, trying old XML format. Error was: "+e);
-                            try {
-                                OldSaveService.processSamples(filename, visualizer, this);
-                                parsedOK = true;
-                            } catch (Exception e1) {
-                                log.warn("Error parsing Avalon XML. " + e1.getLocalizedMessage());
-                            }
+                            log.warn("Failed to load "+filename+" using XStream. Error was: "+e);
                         }
                     }
                 }
@@ -366,7 +375,7 @@ public class ResultCollector extends AbstractListenerElement implements SampleLi
         if (filename == null || filename.length() == 0) {
             return null;
         }
-        FileEntry fe = (FileEntry) files.get(filename);
+        FileEntry fe = files.get(filename);
         PrintWriter writer = null;
         boolean trimmed = true;
 
@@ -467,14 +476,7 @@ public class ResultCollector extends AbstractListenerElement implements SampleLi
                 result.setSaveConfig(config);
                 try {
                     if (config.saveAsXml()) {
-                        if (SaveService.isSaveTestLogFormat20()) {
-                            if (serializer == null) {
-                                serializer = new DefaultConfigurationSerializer();
-                            }
-                            out.write(OldSaveService.getSerializedSampleResult(result, serializer, config));
-                        } else { // !LogFormat20
-                            SaveService.saveSampleResult(event, out);
-                        }
+                        SaveService.saveSampleResult(event, out);
                     } else { // !saveAsXml
                         String savee = CSVSaveService.resultToDelimitedString(event);
                         out.println(savee);
@@ -484,6 +486,10 @@ public class ResultCollector extends AbstractListenerElement implements SampleLi
                 }
             }
         }
+
+        if(summariser != null) {
+            summariser.sampleOccurred(event);
+        }
     }
 
     protected final void sendToVisualizer(SampleResult r) {
@@ -530,11 +536,11 @@ public class ResultCollector extends AbstractListenerElement implements SampleLi
     }
 
     private void finalizeFileOutput() {
-        Iterator it = files.entrySet().iterator();
+        Iterator<Map.Entry<String,ResultCollector.FileEntry>> it = files.entrySet().iterator();
         while(it.hasNext()){
-            Map.Entry me = (Map.Entry) it.next();
+            Map.Entry<String,ResultCollector.FileEntry> me = it.next();
             log.debug("Closing: "+me.getKey());
-            FileEntry fe = (FileEntry) me.getValue();
+            FileEntry fe = me.getValue();
             writeFileEnd(fe.pw, fe.config);
             fe.pw.close();
             if (fe.pw.checkError()){
@@ -544,10 +550,8 @@ public class ResultCollector extends AbstractListenerElement implements SampleLi
         files.clear();
     }
 
-    /*
-     * (non-Javadoc)
-     *
-     * @see TestListener#testIterationStart(LoopIterationEvent)
+    /**
+     * {@inheritDoc}
      */
     public void testIterationStart(LoopIterationEvent event) {
     }
@@ -577,5 +581,4 @@ public class ResultCollector extends AbstractListenerElement implements SampleLi
     // can find the Clearable nodes - the userObject has to implement the interface.
     public void clearData() {
     }
-
 }
\ No newline at end of file
