@@ -22,14 +22,12 @@ import java.io.Serializable;
 import java.io.UnsupportedEncodingException;
 import java.net.HttpURLConnection;
 import java.net.URL;
+import java.nio.charset.Charset;
 import java.util.ArrayList;
 import java.util.HashSet;
 import java.util.List;
 import java.util.Set;
-import java.lang.reflect.Method;
-import java.lang.reflect.InvocationTargetException;
 
-import org.apache.avalon.framework.configuration.Configuration;
 import org.apache.jmeter.assertions.AssertionResult;
 import org.apache.jmeter.util.JMeterUtils;
 import org.apache.jorphan.logging.LoggingManager;
@@ -46,7 +44,7 @@ import org.apache.log.Logger;
 public class SampleResult implements Serializable {
 
     private static final long serialVersionUID = 233L;
-    
+
     // Needs to be accessible from Test code
     static final Logger log = LoggingManager.getLoggerForClass();
 
@@ -70,6 +68,9 @@ public class SampleResult implements Serializable {
             = JMeterUtils.getPropDefault("sampleresult.default.encoding", // $NON-NLS-1$
             DEFAULT_HTTP_ENCODING);
 
+    /* The default used by {@link #setResponseData(String, String)} */
+    private static final String DEFAULT_CHARSET = Charset.defaultCharset().name();
+
     /**
      * Data type value indicating that the response data is text.
      *
@@ -115,9 +116,11 @@ public class SampleResult implements Serializable {
 
     private String label = "";// Never return null
 
-    private String resultFileName = ""; // Filename used by ResultSaver
+    /** Filename used by ResultSaver */
+    private String resultFileName = "";
 
-    private String samplerData; // The data used by the sampler
+    /** The data used by the sampler */
+    private String samplerData;
 
     private String threadName = ""; // Never return null
 
@@ -130,7 +133,8 @@ public class SampleResult implements Serializable {
     private String requestHeaders = "";
 
     // TODO timeStamp == 0 means either not yet initialised or no stamp available (e.g. when loading a results file)
-    private long timeStamp = 0;// the time stamp - can be start or end
+    /** the time stamp - can be start or end */
+    private long timeStamp = 0;
 
     private long startTime = 0;
 
@@ -138,78 +142,60 @@ public class SampleResult implements Serializable {
 
     private long idleTime = 0;// Allow for non-sample time
 
-    private long pauseTime = 0;// Start of pause (if any)
+    /** Start of pause (if any) */
+    private long pauseTime = 0;
 
-    private List assertionResults;
+    private List<AssertionResult> assertionResults;
 
-    private List subResults;
+    private List<SampleResult> subResults;
 
     private String dataType=""; // Don't return null if not set
 
     private boolean success;
 
     //@GuardedBy("this"")
-    private final Set files = new HashSet(); // files that this sample has been saved in
+    /** files that this sample has been saved in */
+    private final Set<String> files = new HashSet<String>();
 
     private String dataEncoding;// (is this really the character set?) e.g.
                                 // ISO-8895-1, UTF-8
 
-    private static Method initNanoTimeMethod() {
-        try {
-            return System.class.getMethod("nanoTime", null);
-        } catch (NoSuchMethodException e) {
-            return null;
-        }
-    }
-
-    private static boolean haveNanoTime() {
-        return nanoTimeMethod != null;
-    }
-
-    private static long nanoTime() {
-        Long result = null;
-        try {
-            result = (Long) nanoTimeMethod.invoke(null, null);
-        } catch (IllegalAccessException e) {
-            throw new RuntimeException(e);
-        } catch (InvocationTargetException e) {
-            throw new RuntimeException(e);
-        }
-        return result.longValue();
-    }
-
-    private static final Method nanoTimeMethod = initNanoTimeMethod();
-
     // a reference time from the nanosecond clock
-    private static final long referenceTimeNsClock = haveNanoTime() ? sampleNsClockInMs() : Long.MIN_VALUE;
+    private static final long referenceTimeNsClock = sampleNsClockInMs();
 
     // a reference time from the millisecond clock
     private static final long referenceTimeMsClock = System.currentTimeMillis();
 
-    private long time = 0; // elapsed time
+    /** elapsed time */
+    private long time = 0;
 
-    private long latency = 0; // time to first response
+    /** time to first response */
+    private long latency = 0;
 
-    private boolean stopThread = false; // Should thread terminate?
+    /** Should thread terminate? */
+    private boolean stopThread = false;
 
-    private boolean stopTest = false; // Should test terminate?
+    /** Should test terminate? */
+    private boolean stopTest = false;
 
-    private boolean stopTestNow = false; // Should test terminate abruptly?
+    /** Should test terminate abruptly? */
+    private boolean stopTestNow = false;
 
+    /** Is the sampler acting as a monitor? */
     private boolean isMonitor = false;
 
     private int sampleCount = 1;
 
     private int bytes = 0; // Allows override of sample size in case sampler does not want to store all the data
 
-    private volatile int groupThreads = 0; // Active threads in this thread group
+    /** Currently active threads in this thread group */
+    private volatile int groupThreads = 0;
 
-    private volatile int allThreads = 0; // Active threads in all thread groups
+    /** Currently active threads in all thread groups */
+    private volatile int allThreads = 0;
 
     // TODO do contentType and/or dataEncoding belong in HTTPSampleResult instead?
 
-    private final static String TOTAL_TIME = "totalTime"; // $NON-NLS-1$
-
     private static final boolean startTimeStamp
         = JMeterUtils.getPropDefault("sampleresult.timestamp.start", false);  // $NON-NLS-1$
 
@@ -227,37 +213,45 @@ public class SampleResult implements Serializable {
     }
 
     /**
-     * Construct a 'parent' result for an already-existing result, essentially
-     * cloning it
-     *
-     * @param res
-     *            existing sample result
+     * Copy constructor.
+     * 
+     * @param res existing sample result
      */
     public SampleResult(SampleResult res) {
-        //TODO - why not just copy all the fields? Do we need the calculations that some of the set() methods perform?
-        //TODO - why are the following not copied:
-        // assertionResults, bytes, idleTime, latency, parent,pauseTime,resultFileName,sampleCount,samplerData,saveConfig
-        // stopTest, stopThread, subResults,threadName
-        setStartTime(res.getStartTime());
-        setEndTime(res.getStartTime());
-        // was setElapsed(0) which is the same as setStartTime=setEndTime=now
-
-        setSampleLabel(res.getSampleLabel());
-        setRequestHeaders(res.getRequestHeaders());
-        setResponseData(res.getResponseData());
-        setResponseCode(res.getResponseCode());
-        setSuccessful(res.isSuccessful());
-        setResponseMessage(res.getResponseMessage());
-        setDataType(res.getDataType());
-        setResponseHeaders(res.getResponseHeaders());
-        setContentType(res.getContentType());
-        setDataEncoding(res.getDataEncodingNoDefault());
-        setURL(res.getURL());
-
-        setGroupThreads(res.getGroupThreads());
-        setAllThreads(res.getAllThreads());
-
-        addSubResult(res); // this will add res.getTime() to getTime().
+        allThreads = res.allThreads;//OK
+        assertionResults = res.assertionResults;// TODO ??
+        bytes = res.bytes;
+        contentType = res.contentType;//OK
+        dataEncoding = res.dataEncoding;//OK
+        dataType = res.dataType;//OK
+        endTime = res.endTime;//OK
+        // files is created automatically, and applies per instance
+        groupThreads = res.groupThreads;//OK
+        idleTime = res.idleTime;
+        isMonitor = res.isMonitor;
+        label = res.label;//OK
+        latency = res.latency;
+        location = res.location;//OK
+        parent = res.parent; // TODO ??
+        pauseTime = res.pauseTime;
+        requestHeaders = res.requestHeaders;//OK
+        responseCode = res.responseCode;//OK
+        responseData = res.responseData;//OK
+        responseHeaders = res.responseHeaders;//OK
+        responseMessage = res.responseMessage;//OK
+        // Don't copy this; it is per instance resultFileName = res.resultFileName;
+        sampleCount = res.sampleCount;
+        samplerData = res.samplerData;
+        saveConfig = res.saveConfig;
+        startTime = res.startTime;//OK
+        stopTest = res.stopTest;
+        stopTestNow = res.stopTestNow;
+        stopThread = res.stopThread;
+        subResults = res.subResults; // TODO ??
+        success = res.success;//OK
+        threadName = res.threadName;//OK
+        time = res.time;
+        timeStamp = res.timeStamp;
     }
 
     public boolean isStampedAtStart() {
@@ -329,17 +323,13 @@ public class SampleResult implements Serializable {
     }
 
     private static long sampleNsClockInMs() {
-        return nanoTime() / 1000000;
+        return System.nanoTime() / 1000000;
     }
 
     // Helper method to get 1 ms resolution timing.
     public static long currentTimeInMs() {
-        if (haveNanoTime()) {
-            long elapsedInMs = sampleNsClockInMs() - referenceTimeNsClock;
-            return referenceTimeMsClock + elapsedInMs;
-        } else {
-            return System.currentTimeMillis();
-        }
+        long elapsedInMs = sampleNsClockInMs() - referenceTimeNsClock;
+        return referenceTimeMsClock + elapsedInMs;
     }
 
     // Helper method to maintain timestamp relationships
@@ -369,31 +359,9 @@ public class SampleResult implements Serializable {
         stampAndTime(stamp, elapsed);
     }
 
-    /**
-     * Method to set the elapsed time for a sample. Retained for backward
-     * compatibility with 3rd party add-ons.
-     * It is assumed that the method is only called at the end of a sample
-     * and that timeStamps are end-times
-     *
-     * Also used by SampleResultConverter when creating results from files.
-     *
-     * Must not be used in conjunction with sampleStart()/End()
-     *
-     * @deprecated use sampleStart() and sampleEnd() instead
-     * @param elapsed
-     *            time in milliseconds
-     */
-    public void setTime(long elapsed) {
-        if (startTime != 0 || endTime != 0){
-            throw new RuntimeException("Calling setTime() after start/end times have been set");
-        }
-        long now = currentTimeInMs();
-        setTimes(now - elapsed, now);
-    }
-
     /**
      * Set the "marked" flag to show that the result has been written to the file.
-     * 
+     *
      * @param filename
      * @return true if the result was previously marked
      */
@@ -405,14 +373,15 @@ public class SampleResult implements Serializable {
         return responseCode;
     }
 
-    private static final String OK = Integer.toString(HttpURLConnection.HTTP_OK);
+    private static final String OK_CODE = Integer.toString(HttpURLConnection.HTTP_OK);
+    private static final String OK_MSG = "OK"; // $NON-NLS-1$
 
     /**
      * Set response code to OK, i.e. "200"
      *
      */
     public void setResponseCodeOK(){
-        responseCode=OK;
+        responseCode=OK_CODE;
     }
 
     public void setResponseCode(String code) {
@@ -420,7 +389,7 @@ public class SampleResult implements Serializable {
     }
 
     public boolean isResponseCodeOK(){
-        return responseCode.equals(OK);
+        return responseCode.equals(OK_CODE);
     }
     public String getResponseMessage() {
         return responseMessage;
@@ -431,7 +400,21 @@ public class SampleResult implements Serializable {
     }
 
     public void setResponseMessageOK() {
-        responseMessage = "OK"; // $NON-NLS-1$
+        responseMessage = OK_MSG;
+    }
+
+    /**
+     * Set result statuses OK - shorthand method to set:
+     * <ul>
+     * <li>ResponseCode</li>
+     * <li>ResponseMessage</li>
+     * <li>Successful status</li>
+     * </ul>
+     */
+    public void setResponseOK(){
+        setResponseCodeOK();
+        setResponseMessageOK();
+        setSuccessful(true);
     }
 
     public String getThreadName() {
@@ -466,7 +449,7 @@ public class SampleResult implements Serializable {
      */
     public String getSampleLabel(boolean includeGroup) {
         if (includeGroup) {
-            StringBuffer sb = new StringBuffer(threadName.substring(0,threadName.lastIndexOf(" "))); //$NON-NLS-1$
+            StringBuilder sb = new StringBuilder(threadName.substring(0,threadName.lastIndexOf(" "))); //$NON-NLS-1$
             return sb.append(":").append(label).toString(); //$NON-NLS-1$
         }
         return label;
@@ -478,7 +461,7 @@ public class SampleResult implements Serializable {
 
     public void addAssertionResult(AssertionResult assertResult) {
         if (assertionResults == null) {
-            assertionResults = new ArrayList();
+            assertionResults = new ArrayList<AssertionResult>();
         }
         assertionResults.add(assertResult);
     }
@@ -493,24 +476,39 @@ public class SampleResult implements Serializable {
         if (assertionResults == null) {
             return EMPTY_AR;
         }
-        return (AssertionResult[]) assertionResults.toArray(new AssertionResult[0]);
+        return assertionResults.toArray(new AssertionResult[0]);
     }
 
+    /**
+     * Add a subresult and adjust the parent byte count and end-time.
+     * 
+     * @param subResult
+     */
     public void addSubResult(SampleResult subResult) {
         String tn = getThreadName();
         if (tn.length()==0) {
             tn=Thread.currentThread().getName();//TODO do this more efficiently
             this.setThreadName(tn);
         }
-        subResult.setThreadName(tn);
-        if (subResults == null) {
-            subResults = new ArrayList();
-        }
-        subResults.add(subResult);
+        subResult.setThreadName(tn); // TODO is this really necessary?
+
         // Extend the time to the end of the added sample
         setEndTime(Math.max(getEndTime(), subResult.getEndTime()));
         // Include the byte count for the added sample
         setBytes(getBytes() + subResult.getBytes());
+        addRawSubResult(subResult);
+    }
+    
+    /**
+     * Add a subresult to the collection without updating any parent fields.
+     * 
+     * @param subResult
+     */
+    public void addRawSubResult(SampleResult subResult){
+        if (subResults == null) {
+            subResults = new ArrayList<SampleResult>();
+        }
+        subResults.add(subResult);
         subResult.setParent(this);
     }
 
@@ -523,7 +521,7 @@ public class SampleResult implements Serializable {
      */
     public void storeSubResult(SampleResult subResult) {
         if (subResults == null) {
-            subResults = new ArrayList();
+            subResults = new ArrayList<SampleResult>();
         }
         subResults.add(subResult);
         subResult.setParent(this);
@@ -539,11 +537,7 @@ public class SampleResult implements Serializable {
         if (subResults == null) {
             return EMPTY_SR;
         }
-        return (SampleResult[]) subResults.toArray(new SampleResult[0]);
-    }
-
-    public void configure(Configuration info) {
-        time = info.getAttributeAsLong(TOTAL_TIME, 0L);
+        return subResults.toArray(new SampleResult[0]);
     }
 
     /**
@@ -568,6 +562,7 @@ public class SampleResult implements Serializable {
      *
      * @deprecated - only intended for use from BeanShell code
      */
+    @Deprecated
     public void setResponseData(String response) {
         try {
             responseData = response.getBytes(getDataEncodingWithDefault());
@@ -577,6 +572,26 @@ public class SampleResult implements Serializable {
         }
     }
 
+    /**
+     * Sets the encoding and responseData attributes of the SampleResult object.
+     *
+     * @param response the new responseData value (String)
+     * @param encoding the encoding to set and then use (if null, use platform default)
+     *
+     */
+    public void setResponseData(final String response, final String encoding) {
+        String encodeUsing = encoding != null? encoding : DEFAULT_CHARSET;
+        try {
+            responseData = response.getBytes(encodeUsing);
+            setDataEncoding(encodeUsing);
+        } catch (UnsupportedEncodingException e) {
+            log.warn("Could not convert string using '"+encodeUsing+
+                    "', using default encoding: "+DEFAULT_CHARSET,e);
+            responseData = response.getBytes();
+            setDataEncoding(DEFAULT_CHARSET);
+        }
+    }
+
     /**
      * Gets the responseData attribute of the SampleResult object.
      * <p>
@@ -638,7 +653,7 @@ public class SampleResult implements Serializable {
      * Extract and save the DataEncoding and DataType from the parameter provided.
      * Does not save the full content Type.
      * @see #setContentType(String) which should be used to save the full content-type string
-     * 
+     *
      * @param ct - content type (may be null)
      */
     public void setEncodingAndType(String ct){
@@ -650,8 +665,12 @@ public class SampleResult implements Serializable {
             final String CS_PFX = "charset="; // $NON-NLS-1$
             int cset = ct.toLowerCase(java.util.Locale.ENGLISH).indexOf(CS_PFX);
             if (cset >= 0) {
-                // TODO - assumes charset is not followed by anything else
                 String charSet = ct.substring(cset + CS_PFX.length());
+                // handle: ContentType: text/plain; charset=ISO-8859-1; format=flowed
+                int semiColon = charSet.indexOf(';');
+                if (semiColon >= 0) {
+                    charSet=charSet.substring(0, semiColon);
+                }
                 // Check for quoted string
                 if (charSet.startsWith("\"")){ // $NON-NLS-1$
                     setDataEncoding(charSet.substring(1, charSet.length()-1)); // remove quotes
@@ -704,31 +723,31 @@ public class SampleResult implements Serializable {
      *
      * @return display name of this sample result
      */
+    @Override
     public String toString() {
         return getSampleLabel();
     }
 
     /**
-     * Returns the dataEncoding or the default if no dataEncoding was provided
-     *
-     * @deprecated use getDataEncodingWithDefault() or getDataEncodingNoDefault() as needed.
+     * Returns the dataEncoding or the default if no dataEncoding was provided.
+     * 
+     * @return the value of the dataEncoding or DEFAULT_ENCODING
      */
-    public String getDataEncoding() {
-        if (dataEncoding != null) {
-            return dataEncoding;
-        }
-        return DEFAULT_ENCODING;
+    public String getDataEncodingWithDefault() {
+        return getDataEncodingWithDefault(DEFAULT_ENCODING);
     }
 
     /**
-     * Returns the dataEncoding or the default if no dataEncoding was provided
-     * @return the value of the dataEncoding or DEFAULT_ENCODING 
+     * Returns the dataEncoding or the default if no dataEncoding was provided.
+     * 
+     * @param defaultEncoding the default to be applied
+     * @return the value of the dataEncoding or the provided default
      */
-    public String getDataEncodingWithDefault() {
+    protected String getDataEncodingWithDefault(String defaultEncoding) {
         if (dataEncoding != null && dataEncoding.length() > 0) {
             return dataEncoding;
         }
-        return DEFAULT_ENCODING;
+        return defaultEncoding;
     }
 
     /**
@@ -830,7 +849,7 @@ public class SampleResult implements Serializable {
     /**
      * Stores the content-type string, e.g. "text/xml; charset=utf-8"
      * @see #setEncodingAndType(String) which can be used to extract the charset.
-     * 
+     *
      * @param string
      */
     public void setContentType(String string) {
@@ -884,6 +903,15 @@ public class SampleResult implements Serializable {
         }
     }
 
+    /**
+     * Set idle time pause.
+     * For use by SampleResultConverter/CSVSaveService.
+     * @param idle long
+     */
+    public void setIdleTime(long idle) {
+        idleTime = idle;
+    }
+
     private void setTimes(long start, long end) {
         setStartTime(start);
         setEndTime(end);
@@ -955,10 +983,10 @@ public class SampleResult implements Serializable {
     }
 
     /**
-     * For the JMS sampler, it can perform multiple samples for greater degree
-     * of accuracy.
-     *
-     * @param count
+     * The statistical sample sender aggregates several samples to save on
+     * transmission costs.
+     * 
+     * @param count number of samples represented by this instance
      */
     public void setSampleCount(int count) {
         sampleCount = count;
@@ -985,8 +1013,10 @@ public class SampleResult implements Serializable {
     public void setErrorCount(int i){// for reading from CSV files
         // ignored currently
     }
+
     /*
-     * TODO: error counting needs to be sorted out after 2.3 final.
+     * TODO: error counting needs to be sorted out.
+     *
      * At present the Statistical Sampler tracks errors separately
      * It would make sense to move the error count here, but this would
      * mean lots of changes.
@@ -1113,4 +1143,19 @@ public class SampleResult implements Serializable {
     public void setAllThreads(int n) {
         this.allThreads = n;
     }
+
+    // Bug 47394
+    /**
+     * Allow custom SampleSenders to drop unwanted assertionResults
+     */
+    public void removeAssertionResults() {
+        this.assertionResults = null;
+    }
+
+    /**
+     * Allow custom SampleSenders to drop unwanted subResults
+     */
+    public void removeSubResults() {
+        this.subResults = null;
+    }
 }
