@@ -90,6 +90,7 @@ public final class CSVSaveService {
     private static final String CSV_LATENCY = "Latency"; // $NON-NLS-1$
     private static final String CSV_ENCODING = "Encoding"; // $NON-NLS-1$
     private static final String CSV_HOSTNAME = "Hostname"; // $NON-NLS-1$
+    private static final String CSV_IDLETIME = "IdleTime"; // $NON-NLS-1$
 
     // Used to enclose variable name labels, to distinguish from any of the above labels
     private static final String VARIABLE_NAME_QUOTE_CHAR = "\"";  // $NON-NLS-1$
@@ -102,6 +103,8 @@ public final class CSVSaveService {
     private static final String DEFAULT_DATE_FORMAT_STRING = "MM/dd/yy HH:mm:ss"; // $NON-NLS-1$
     private static final DateFormat DEFAULT_DATE_FORMAT = new SimpleDateFormat(DEFAULT_DATE_FORMAT_STRING);
 
+    private static final String LINE_SEP = System.getProperty("line.separator"); // $NON-NLS-1$
+
     /**
      * Private constructor to prevent instantiation.
      */
@@ -157,31 +160,6 @@ public final class CSVSaveService {
         }
     }
 
-    /**
-     * Make a SampleResult given a delimited string.
-     *
-     * @param inputLine - line from CSV file
-     * @param saveConfig - configuration
-     * @param lineNumber - line number for error reporting
-     * @return SampleResult
-     *
-     * @deprecated Does not handle quoted strings; use {@link #processSamples(String, Visualizer, ResultCollector)} instead
-     *
-     * @throws JMeterError
-     */
-    public static SampleEvent makeResultFromDelimitedString(
-            final String inputLine,
-            final SampleSaveConfiguration saveConfig, // may be updated
-            final long lineNumber) {
-        /*
-         * Bug 40772: replaced StringTokenizer with String.split(), as the
-         * former does not return empty tokens.
-         */
-        // The \Q prefix is needed to ensure that meta-characters (e.g. ".") work.
-        String parts[]=inputLine.split("\\Q"+saveConfig.getDelimiter());// $NON-NLS-1$
-        return makeResultFromDelimitedString(parts, saveConfig, lineNumber);
-    }
-
     /**
      * Make a SampleResult given a set of tokens
      * @param parts tokens parsed from the input
@@ -334,6 +312,12 @@ public final class CSVSaveService {
                 hostname = parts[i++];
             }
 
+            if (saveConfig.saveIdleTime()) {
+                field = CSV_IDLETIME;
+                text = parts[i++];
+                result.setIdleTime(Long.parseLong(text));
+            }
+
             if (i + saveConfig.getVarCount() < parts.length){
                 log.warn("Line: "+lineNumber+". Found "+parts.length+" fields, expected "+i+". Extra fields have been ignored.");
             }
@@ -366,7 +350,7 @@ public final class CSVSaveService {
      * @return the field names as a string
      */
     public static String printableFieldNamesToString(SampleSaveConfiguration saveConfig) {
-        StringBuffer text = new StringBuffer();
+        StringBuilder text = new StringBuilder();
         String delim = saveConfig.getDelimiter();
 
         if (saveConfig.saveTimestamp()) {
@@ -458,6 +442,11 @@ public final class CSVSaveService {
             text.append(delim);
         }
 
+        if (saveConfig.saveIdleTime()) {
+            text.append(CSV_IDLETIME);
+            text.append(delim);
+        }
+
         for (int i = 0; i < SampleEvent.getVarCount(); i++){
             text.append(VARIABLE_NAME_QUOTE_CHAR);
             text.append(SampleEvent.getVarName(i));
@@ -505,6 +494,7 @@ public final class CSVSaveService {
             headerLabelMethods.put(CSV_SAMPLE_COUNT, new Functor("setSampleCount"));
             headerLabelMethods.put(CSV_ERROR_COUNT, new Functor("setSampleCount"));
             headerLabelMethods.put(CSV_HOSTNAME, new Functor("setHostname"));
+            headerLabelMethods.put(CSV_IDLETIME, new Functor("setIdleTime"));
     }
 
     /**
@@ -608,7 +598,7 @@ public final class CSVSaveService {
      * @param writer output file
      * @throws IOException
      */
-    public static void saveCSVStats(Vector data, FileWriter writer) throws IOException {
+    public static void saveCSVStats(Vector<?> data, FileWriter writer) throws IOException {
         saveCSVStats(data, writer, null);
     }
 
@@ -622,9 +612,8 @@ public final class CSVSaveService {
      * @param headers header names (if non-null)
      * @throws IOException
      */
-    public static void saveCSVStats(Vector data, FileWriter writer, String headers[]) throws IOException {
+    public static void saveCSVStats(Vector<?> data, FileWriter writer, String headers[]) throws IOException {
         final char DELIM = ',';
-        final String LINE_SEP = System.getProperty("line.separator"); // $NON-NLS-1$
         final char SPECIALS[] = new char[] {DELIM, QUOTING_CHAR};
         if (headers != null){
             for (int i=0; i < headers.length; i++){
@@ -636,7 +625,7 @@ public final class CSVSaveService {
             writer.write(LINE_SEP);
         }
         for (int idx=0; idx < data.size(); idx++) {
-            Vector row = (Vector)data.elementAt(idx);
+            Vector<?> row = (Vector<?>)data.elementAt(idx);
             for (int idy=0; idy < row.size(); idy++) {
                 if (idy > 0) {
                     writer.write(DELIM);
@@ -649,7 +638,7 @@ public final class CSVSaveService {
     }
 
     /**
-     * Method saves aggregate statistics as CSV from a table model.
+     * Method saves aggregate statistics (with header names) as CSV from a table model.
      * Same as {@link #saveCSVStats(Vector, FileWriter, String[])} except
      * that there is no need to create a Vector containing the data.
      *
@@ -658,18 +647,33 @@ public final class CSVSaveService {
      * @throws IOException
      */
     public static void saveCSVStats(DefaultTableModel model, FileWriter writer) throws IOException {
+        saveCSVStats(model, writer, true);
+    }
+
+    /**
+     * Method saves aggregate statistics as CSV from a table model.
+     * Same as {@link #saveCSVStats(Vector, FileWriter, String[])} except
+     * that there is no need to create a Vector containing the data.
+     *
+     * @param model table model containing the data
+     * @param writer output file
+     * @param saveHeaders whether or not to save headers
+     * @throws IOException
+     */
+    public static void saveCSVStats(DefaultTableModel model, FileWriter writer, boolean saveHeaders) throws IOException {
         final char DELIM = ',';
-        final String LINE_SEP = System.getProperty("line.separator"); // $NON-NLS-1$
         final char SPECIALS[] = new char[] {DELIM, QUOTING_CHAR};
         final int columns = model.getColumnCount();
         final int rows = model.getRowCount();
-        for (int i=0; i < columns; i++){
-            if (i>0) {
-                writer.write(DELIM);
+        if (saveHeaders){
+            for (int i=0; i < columns; i++){
+                if (i>0) {
+                    writer.write(DELIM);
+                }
+                writer.write(quoteDelimiters(model.getColumnName(i),SPECIALS));
             }
-            writer.write(quoteDelimiters(model.getColumnName(i),SPECIALS));
+            writer.write(LINE_SEP);
         }
-        writer.write(LINE_SEP);
         for (int row=0; row < rows; row++) {
             for (int column=0; column < columns; column++) {
                 if (column > 0) {
@@ -712,7 +716,7 @@ public final class CSVSaveService {
          * - quotes any strings that require it
          */
         final class StringQuoter{
-            final StringBuffer sb = new StringBuffer();
+            final StringBuilder sb = new StringBuilder();
             private final char[] specials;
             private boolean addDelim;
             public StringQuoter(char delim) {
@@ -751,6 +755,7 @@ public final class CSVSaveService {
                 sb.append(b);
             }
 
+            @Override
             public String toString(){
                 return sb.toString();
             }
@@ -898,7 +903,7 @@ public final class CSVSaveService {
         if (StringUtils.containsNone(input, specialChars)) {
             return input;
         }
-        StringBuffer buffer = new StringBuffer(input.length() + 10);
+        StringBuilder buffer = new StringBuilder(input.length() + 10);
         final char quote = specialChars[1];
         buffer.append(quote);
         for (int i = 0; i < input.length(); i++) {
@@ -931,7 +936,7 @@ public final class CSVSaveService {
     public static String[] csvReadFile(BufferedReader infile, char delim) throws IOException {
         int ch;
         int state = INITIAL;
-        List list = new ArrayList();
+        List<String> list = new ArrayList<String>();
         ByteArrayOutputStream baos = new ByteArrayOutputStream(200);
         boolean push = false;
         while(-1 != (ch=infile.read())){
@@ -1005,7 +1010,7 @@ public final class CSVSaveService {
                 list.add(baos.toString());
             }
         }
-        return (String[]) list.toArray(new String[list.size()]);
+        return list.toArray(new String[list.size()]);
     }
 
     private static boolean isDelimOrEOL(char delim, int ch) {
