@@ -30,6 +30,7 @@ import java.lang.reflect.InvocationTargetException;
 import java.util.Iterator;
 import java.util.Map;
 import java.util.Properties;
+import java.util.Map.Entry;
 
 import java.nio.charset.Charset;
 
@@ -73,16 +74,20 @@ public class SaveService {
         }
 
         // Override wrapMapper in order to insert the Wrapper in the chain
+        @Override
         protected MapperWrapper wrapMapper(MapperWrapper next) {
             // Provide our own aliasing using strings rather than classes
             return new MapperWrapper(next){
             // Translate alias to classname and then delegate to wrapped class
-            public Class realClass(String alias) {
+            @Override
+            public Class<?> realClass(String alias) {
                 String fullName = aliasToClass(alias);
                 return super.realClass(fullName == null ? alias : fullName);
             }
             // Translate to alias and then delegate to wrapped class
-            public String serializedClass(Class type) {
+            @Override
+            public String serializedClass(@SuppressWarnings("rawtypes") // superclass does not use types 
+                    Class type) {
                 if (type == null) {
                     return super.serializedClass(null); // was type, but that caused FindBugs warning
                 }
@@ -98,7 +103,7 @@ public class SaveService {
     static {
         JTLSAVER.setMode(XStream.NO_REFERENCES); // This is needed to stop XStream keeping copies of each class
     }
-    
+
     // The XML header, with placeholder for encoding, since that is controlled by property
     private static final String XML_HEADER = "<?xml version=\"1.0\" encoding=\"<ph>\"?>"; // $NON-NLS-1$
 
@@ -114,8 +119,6 @@ public class SaveService {
     private static final String FILE_FORMAT_TESTLOG = "file_format.testlog"; // $NON-NLS-1$
 
     // Define file format versions
-    private static final String VERSION_2_0 = "2.0";  // $NON-NLS-1$
-    //NOT USED private static final String VERSION_2_1 = "2.1";  // $NON-NLS-1$
     private static final String VERSION_2_2 = "2.2";  // $NON-NLS-1$
 
     // Default to overall format, and then to version 2.2
@@ -127,13 +130,23 @@ public class SaveService {
         = JMeterUtils.getPropDefault(FILE_FORMAT_TESTLOG
         , JMeterUtils.getPropDefault(FILE_FORMAT, VERSION_2_2));
 
-    private static final boolean IS_TESTPLAN_FORMAT_20
-        = VERSION_2_0.equals(TESTPLAN_FORMAT);
+    private static boolean validateFormat(String format){
+        if ("2.2".equals(format)) return true;
+        if ("2.1".equals(format)) return true;
+        return false;
+    }
 
-    private static final boolean IS_TESTLOG_FORMAT_20
-    = VERSION_2_0.equals(TESTLOG_FORMAT);
+    static{
+        if (!validateFormat(TESTPLAN_FORMAT)){
+            log.error("Invalid test plan format: "+TESTPLAN_FORMAT);
+        }
+        if (!validateFormat(TESTLOG_FORMAT)){
+            log.error("Invalid test log format: "+TESTLOG_FORMAT);
+        }
+    }
 
-    private static final boolean IS_TESTPLAN_FORMAT_22
+    /** New XStream format - more compressed class names */
+    public static final boolean IS_TESTPLAN_FORMAT_22
         = VERSION_2_2.equals(TESTPLAN_FORMAT);
 
     // Holds the mappings from the saveservice properties file
@@ -153,7 +166,7 @@ public class SaveService {
 
     // Internal information only
     private static String fileVersion = ""; // read from properties file// $NON-NLS-1$
-    private static final String FILEVERSION = "697317"; // Expected value $NON-NLS-1$
+//    private static final String FILEVERSION = "958773"; // Expected value $NON-NLS-1$
     private static String fileEncoding = ""; // read from properties file// $NON-NLS-1$
 
     static {
@@ -188,9 +201,9 @@ public class SaveService {
         try {
             Properties nameMap = loadProperties();
             // now create the aliases
-            Iterator it = nameMap.entrySet().iterator();
+            Iterator<Entry<Object, Object>> it = nameMap.entrySet().iterator();
             while (it.hasNext()) {
-                Map.Entry me = (Map.Entry) it.next();
+                Map.Entry<Object, Object> me = it.next();
                 String key = (String) me.getKey();
                 String val = (String) me.getValue();
                 if (!key.startsWith("_")) {
@@ -247,7 +260,7 @@ public class SaveService {
      * @param key
      * @param jmxsaver
      * @param useMapper
-     *  
+     *
      * @throws InstantiationException
      * @throws IllegalAccessException
      * @throws InvocationTargetException
@@ -263,7 +276,7 @@ public class SaveService {
                     new Class[] { Mapper.class }).newInstance(
                             new Object[] { jmxsaver.getMapper() }));
         } else {
-            jmxsaver.registerConverter((Converter) Class.forName(key).newInstance());            
+            jmxsaver.registerConverter((Converter) Class.forName(key).newInstance());
         }
     }
 
@@ -373,7 +386,7 @@ public class SaveService {
     }
 
     static boolean checkFileVersion(){
-        return SaveService.FILEVERSION.equals(SaveService.fileVersion);
+        return true; // SaveService.FILEVERSION.equals(SaveService.fileVersion);
     }
 
     static boolean checkVersions() {
@@ -405,9 +418,9 @@ public class SaveService {
         if (!PROPVERSION.equalsIgnoreCase(propertiesVersion)) {
             log.warn("Bad _version - expected " + PROPVERSION + ", found " + propertiesVersion + ".");
         }
-        if (!FILEVERSION.equalsIgnoreCase(fileVersion)) {
-            log.warn("Bad _file_version - expected " + FILEVERSION + ", found " + fileVersion +".");
-        }
+//        if (!FILEVERSION.equalsIgnoreCase(fileVersion)) {
+//            log.warn("Bad _file_version - expected " + FILEVERSION + ", found " + fileVersion +".");
+//        }
         if (versionsOK) {
             log.info("All converter versions present and correct");
         }
@@ -416,7 +429,7 @@ public class SaveService {
 
     /**
      * Read results from JTL file.
-     *  
+     *
      * @param reader of the file
      * @param resultCollectorHelper helper class to enable TestResultWrapperConverter to deliver the samples
      * @throws Exception
@@ -450,14 +463,13 @@ public class SaveService {
             wrapper = (ScriptWrapper) JMXSAVER.fromXML(inputStreamReader);
             inputStreamReader.close();
             if (wrapper == null){
-                log.error("Problem loading new style: see above.");
+                log.error("Problem loading XML: see above.");
                 return null;
             }
             return wrapper.testPlan;
         } catch (CannotResolveClassException e) {
-            log.warn("Problem loading new style: " + e.getLocalizedMessage());
-            reader.reset();
-            return OldSaveService.loadSubTree(reader);
+            log.warn("Problem loading XML: " + e.getLocalizedMessage());
+            return null;
         } catch (NoClassDefFoundError e) {
             log.error("Missing class "+e);
             return null;
@@ -530,20 +542,6 @@ public class SaveService {
         }
     }
 
-    public static boolean isSaveTestPlanFormat20() {
-        return IS_TESTPLAN_FORMAT_20;
-    }
-
-    public static boolean isSaveTestLogFormat20() {
-        return IS_TESTLOG_FORMAT_20;
-    }
-
-    // New test format - more compressed class names
-    public static boolean isSaveTestPlanFormat22() {
-        return IS_TESTPLAN_FORMAT_22;
-    }
-
-
 //  Normal output
 //  ---- Debugging information ----
 //  required-type       : org.apache.jorphan.collections.ListedHashTree
