@@ -78,7 +78,7 @@ public class JMeterThread implements Runnable, Interruptible {
 
     private final JMeterVariables threadVars;
 
-    private final Collection testListeners;
+    private final Collection<TestListener> testListeners;
 
     private final ListenerNotifier notifier;
 
@@ -102,7 +102,7 @@ public class JMeterThread implements Runnable, Interruptible {
     // based on this scheduler is enabled or disabled
 
     // Gives access to parent thread threadGroup
-    private ThreadGroup threadGroup;
+    private AbstractThreadGroup threadGroup;
 
     private StandardJMeterEngine engine = null; // For access to stop methods.
 
@@ -116,7 +116,7 @@ public class JMeterThread implements Runnable, Interruptible {
     private volatile boolean onErrorStopTestNow;
 
     private volatile boolean onErrorStopThread;
-    
+
     private volatile Sampler currentSampler;
 
     public JMeterThread(HashTree test, JMeterThreadMonitor monitor, ListenerNotifier note) {
@@ -125,7 +125,7 @@ public class JMeterThread implements Runnable, Interruptible {
         testTree = test;
         compiler = new TestCompiler(testTree, threadVars);
         controller = (Controller) testTree.getArray()[0];
-        SearchByClass threadListenerSearcher = new SearchByClass(TestListener.class);
+        SearchByClass<TestListener> threadListenerSearcher = new SearchByClass<TestListener>(TestListener.class);
         test.traverse(threadListenerSearcher);
         testListeners = threadListenerSearcher.getSearchResults();
         notifier = note;
@@ -268,6 +268,7 @@ public class JMeterThread implements Runnable, Interruptible {
             log.info("Thread finished: " + threadName);
             threadFinished();
             monitor.threadFinished(this); // Tell the engine we are done
+            JMeterContextService.removeContext(); // Remove the ThreadLocal entry
         }
     }
 
@@ -276,9 +277,10 @@ public class JMeterThread implements Runnable, Interruptible {
      *
      * @param current sampler
      * @param parent sampler
-     * @param threadContext 
+     * @param threadContext
      * @return SampleResult if a transaction was processed
      */
+    @SuppressWarnings("deprecation") // OK to call TestBeanHelper.prepare()
     private SampleResult process_sampler(Sampler current, Sampler parent, JMeterContext threadContext) {
         SampleResult transactionResult = null;
         try {
@@ -356,7 +358,7 @@ public class JMeterThread implements Runnable, Interruptible {
                     runPostProcessors(pack.getPostProcessors());
                     checkAssertions(pack.getAssertions(), result, threadContext);
                     // Do not send subsamples to listeners which receive the transaction sample
-                    List sampleListeners = getSampleListeners(pack, transactionPack, transactionSampler);
+                    List<SampleListener> sampleListeners = getSampleListeners(pack, transactionPack, transactionSampler);
                     notifyListeners(sampleListeners, result);
                     compiler.done(pack);
                     // Add the result as subsample of transaction if we are in a transaction
@@ -407,17 +409,17 @@ public class JMeterThread implements Runnable, Interruptible {
      * @param transactionSampler
      * @return the listeners who should receive the sample result
      */
-    private List getSampleListeners(SamplePackage samplePack, SamplePackage transactionPack, TransactionSampler transactionSampler) {
-        List sampleListeners = samplePack.getSampleListeners();
+    private List<SampleListener> getSampleListeners(SamplePackage samplePack, SamplePackage transactionPack, TransactionSampler transactionSampler) {
+        List<SampleListener> sampleListeners = samplePack.getSampleListeners();
         // Do not send subsamples to listeners which receive the transaction sample
         if(transactionSampler != null) {
-            ArrayList onlySubSamplerListeners = new ArrayList();
-            List transListeners = transactionPack.getSampleListeners();
-            for(Iterator i = sampleListeners.iterator(); i.hasNext();) {
-                SampleListener listener = (SampleListener)i.next();
+            ArrayList<SampleListener> onlySubSamplerListeners = new ArrayList<SampleListener>();
+            List<SampleListener> transListeners = transactionPack.getSampleListeners();
+            for(Iterator<SampleListener> i = sampleListeners.iterator(); i.hasNext();) {
+                SampleListener listener = i.next();
                 // Check if this instance is present in transaction listener list
                 boolean found = false;
-                for(Iterator j = transListeners.iterator(); j.hasNext();) {
+                for(Iterator<SampleListener> j = transListeners.iterator(); j.hasNext();) {
                     // Check for the same instance
                     if(j.next() == listener) {
                         found = true;
@@ -434,7 +436,7 @@ public class JMeterThread implements Runnable, Interruptible {
     }
 
     /**
-     * @param threadContext 
+     * @param threadContext
      *
      */
     private void initRun(JMeterContext threadContext) {
@@ -531,7 +533,7 @@ public class JMeterThread implements Runnable, Interruptible {
             log.warn("Interrupting: " + threadName + " sampler: " +samp.getName());
             try {
                 boolean found = ((Interruptible)samp).interrupt();
-                if (!found) { 
+                if (!found) {
                     log.warn("No operation pending");
                 }
                 return found;
@@ -565,15 +567,16 @@ public class JMeterThread implements Runnable, Interruptible {
         log.info("Stop Thread detected by thread: " + threadName);
     }
 
-    private void checkAssertions(List assertions, SampleResult parent, JMeterContext threadContext) {
-        Iterator iter = assertions.iterator();
+    @SuppressWarnings("deprecation") // OK to call TestBeanHelper.prepare()
+    private void checkAssertions(List<Assertion> assertions, SampleResult parent, JMeterContext threadContext) {
+        Iterator<Assertion> iter = assertions.iterator();
         while (iter.hasNext()) {
-            Assertion assertion = (Assertion) iter.next();
+            Assertion assertion = iter.next();
             TestBeanHelper.prepare((TestElement) assertion);
             if (assertion instanceof AbstractScopedAssertion){
                 AbstractScopedAssertion scopedAssertion = (AbstractScopedAssertion) assertion;
                 String scope = scopedAssertion.fetchScope();
-                if (scopedAssertion.isScopeParent(scope) || scopedAssertion.isScopeAll(scope)){
+                if (scopedAssertion.isScopeParent(scope) || scopedAssertion.isScopeAll(scope) || scopedAssertion.isScopeVariable(scope)){
                     processAssertion(parent, assertion);
                 }
                 if (scopedAssertion.isScopeChildren(scope) || scopedAssertion.isScopeAll(scope)){
@@ -601,34 +604,51 @@ public class JMeterThread implements Runnable, Interruptible {
     }
 
     private void processAssertion(SampleResult result, Assertion assertion) {
-        AssertionResult assertionResult = assertion.getResult(result);
+        AssertionResult assertionResult;
+        try {
+            assertionResult = assertion.getResult(result);
+        } catch (ThreadDeath e) {
+            throw e;
+        } catch (Error e) {
+            log.error("Error processing Assertion ",e);
+            assertionResult = new AssertionResult("Assertion failed! See log file.");
+            assertionResult.setError(true);
+            assertionResult.setFailureMessage(e.toString());
+        } catch (Exception e) {
+            log.error("Exception processing Assertion ",e);
+            assertionResult = new AssertionResult("Assertion failed! See log file.");
+            assertionResult.setError(true);
+            assertionResult.setFailureMessage(e.toString());
+        }
         result.setSuccessful(result.isSuccessful() && !(assertionResult.isError() || assertionResult.isFailure()));
         result.addAssertionResult(assertionResult);
     }
 
-    private void runPostProcessors(List extractors) {
-        ListIterator iter;
+    @SuppressWarnings("deprecation") // OK to call TestBeanHelper.prepare()
+    private void runPostProcessors(List<PostProcessor> extractors) {
+        ListIterator<PostProcessor> iter;
         if (reversePostProcessors) {// Original (rather odd) behaviour
             iter = extractors.listIterator(extractors.size());// start at the end
             while (iter.hasPrevious()) {
-                PostProcessor ex = (PostProcessor) iter.previous();
+                PostProcessor ex = iter.previous();
                 TestBeanHelper.prepare((TestElement) ex);
                 ex.process();
             }
         } else {
             iter = extractors.listIterator(); // start at the beginning
             while (iter.hasNext()) {
-                PostProcessor ex = (PostProcessor) iter.next();
+                PostProcessor ex = iter.next();
                 TestBeanHelper.prepare((TestElement) ex);
                 ex.process();
             }
         }
     }
 
-    private void runPreProcessors(List preProcessors) {
-        Iterator iter = preProcessors.iterator();
+    @SuppressWarnings("deprecation") // OK to call TestBeanHelper.prepare()
+    private void runPreProcessors(List<PreProcessor> preProcessors) {
+        Iterator<PreProcessor> iter = preProcessors.iterator();
         while (iter.hasNext()) {
-            PreProcessor ex = (PreProcessor) iter.next();
+            PreProcessor ex = iter.next();
             if (log.isDebugEnabled()) {
                 log.debug("Running preprocessor: " + ((AbstractTestElement) ex).getName());
             }
@@ -637,11 +657,12 @@ public class JMeterThread implements Runnable, Interruptible {
         }
     }
 
-    private void delay(List timers) {
+    @SuppressWarnings("deprecation") // OK to call TestBeanHelper.prepare()
+    private void delay(List<Timer> timers) {
         long sum = 0;
-        Iterator iter = timers.iterator();
+        Iterator<Timer> iter = timers.iterator();
         while (iter.hasNext()) {
-            Timer timer = (Timer) iter.next();
+            Timer timer = iter.next();
             TestBeanHelper.prepare((TestElement) timer);
             sum += timer.delay();
         }
@@ -656,9 +677,9 @@ public class JMeterThread implements Runnable, Interruptible {
 
     private void notifyTestListeners() {
         threadVars.incIteration();
-        Iterator iter = testListeners.iterator();
+        Iterator<TestListener> iter = testListeners.iterator();
         while (iter.hasNext()) {
-            TestListener listener = (TestListener) iter.next();
+            TestListener listener = iter.next();
             if (listener instanceof TestElement) {
                 listener.testIterationStart(new LoopIterationEvent(controller, threadVars.getIteration()));
                 ((TestElement) listener).recoverRunningVersion();
@@ -668,7 +689,7 @@ public class JMeterThread implements Runnable, Interruptible {
         }
     }
 
-    private void notifyListeners(List listeners, SampleResult result) {
+    private void notifyListeners(List<SampleListener> listeners, SampleResult result) {
         SampleEvent event = new SampleEvent(result, threadGroup.getName(), threadVars);
         notifier.notifyListeners(event, listeners);
 
@@ -711,10 +732,8 @@ public class JMeterThread implements Runnable, Interruptible {
     }
 
     private class IterationListener implements LoopIterationListener {
-        /*
-         * (non-Javadoc)
-         *
-         * @see LoopIterationListener#iterationStart(LoopIterationEvent)
+        /**
+         * {@inheritDoc}
          */
         public void iterationStart(LoopIterationEvent iterEvent) {
             notifyTestListeners();
@@ -760,7 +779,7 @@ public class JMeterThread implements Runnable, Interruptible {
         onErrorStopThread = b;
     }
 
-    public void setThreadGroup(ThreadGroup group) {
+    public void setThreadGroup(AbstractThreadGroup group) {
         this.threadGroup = group;
     }
 
