@@ -33,6 +33,7 @@ import java.net.UnknownHostException;
 import java.util.Enumeration;
 import java.util.Hashtable;
 import java.util.Iterator;
+import java.util.List;
 import java.util.Locale;
 import java.util.MissingResourceException;
 import java.util.Properties;
@@ -50,6 +51,7 @@ import javax.swing.JOptionPane;
 import org.apache.commons.io.IOUtils;
 import org.apache.jmeter.gui.GuiPackage;
 import org.apache.jorphan.logging.LoggingManager;
+import org.apache.jorphan.reflect.ClassFinder;
 import org.apache.jorphan.test.UnitTestManager;
 import org.apache.jorphan.util.JOrphanUtils;
 import org.apache.log.Logger;
@@ -76,7 +78,7 @@ public class JMeterUtils implements UnitTestManager {
 
     private static volatile Properties appProperties;
 
-    private static final Vector localeChangeListeners = new Vector();
+    private static final Vector<LocaleChangeListener> localeChangeListeners = new Vector<LocaleChangeListener>();
 
     private static volatile Locale locale;
 
@@ -90,11 +92,12 @@ public class JMeterUtils implements UnitTestManager {
     private static String localHostName = null;
     //@GuardedBy("this")
     private static String localHostFullName = null;
-    
+
     private static volatile boolean ignoreResorces = false; // Special flag for use in debugging resources
 
-    private static final ThreadLocal localMatcher = new ThreadLocal() {
-        protected Object initialValue() {
+    private static final ThreadLocal<Perl5Matcher> localMatcher = new ThreadLocal<Perl5Matcher>() {
+        @Override
+        protected Perl5Matcher initialValue() {
             return new Perl5Matcher();
         }
     };
@@ -106,7 +109,7 @@ public class JMeterUtils implements UnitTestManager {
      * Gets Perl5Matcher for this thread.
      */
     public static Perl5Matcher getMatcher() {
-        return (Perl5Matcher) localMatcher.get();
+        return localMatcher.get();
     }
 
     /**
@@ -264,6 +267,28 @@ public class JMeterUtils implements UnitTestManager {
         getProperties(file);
     }
 
+    /**
+     * Convenience method for
+     * {@link ClassFinder#findClassesThatExtend(String[], Class[], boolean)}
+     * with the option to include inner classes in the search set to false
+     * and the path list is derived from JMeterUtils.getSearchPaths().
+     *
+     * @param superClass - single class to search for
+     * @return List of Strings containing discovered class names.
+     */
+    public static List<String> findClassesThatExtend(Class<?> superClass)
+        throws IOException {
+        return ClassFinder.findClassesThatExtend(getSearchPaths(), new Class[]{superClass}, false);
+    }
+
+    /**
+     * Generate a list of paths to search.
+     * The output array always starts with
+     * JMETER_HOME/lib/ext
+     * and is followed by any paths obtained from the "search_paths" JMeter property.
+     * 
+     * @return array of path strings
+     */
     public static String[] getSearchPaths() {
         String p = JMeterUtils.getPropDefault("search_paths", null); // $NON-NLS-1$
         String[] result = new String[1];
@@ -366,10 +391,11 @@ public class JMeterUtils implements UnitTestManager {
      */
     private static void notifyLocaleChangeListeners() {
         LocaleChangeEvent event = new LocaleChangeEvent(JMeterUtils.class, locale);
-        Iterator iterator = ((Vector) localeChangeListeners.clone()).iterator();
+        @SuppressWarnings("unchecked") // clone will produce correct type
+        Iterator<LocaleChangeListener> iterator = ((Vector<LocaleChangeListener>) localeChangeListeners.clone()).iterator();
 
         while (iterator.hasNext()) {
-            LocaleChangeListener listener = (LocaleChangeListener) iterator.next();
+            LocaleChangeListener listener = iterator.next();
             listener.localeChanged(event);
         }
     }
@@ -405,6 +431,7 @@ public class JMeterUtils implements UnitTestManager {
      * @deprecated Only intended for use in development; use
      *             getResString(String) normally
      */
+    @Deprecated
     public static String getResString(String key, String defaultValue) {
         return getResStringDefault(key, defaultValue);
     }
@@ -440,7 +467,7 @@ public class JMeterUtils implements UnitTestManager {
      * Get the locale name as a resource.
      * Does not log an error if the resource does not exist.
      * This is needed to support additional locales, as they won't be in existing messages files.
-     * 
+     *
      * @param locale name
      * @return the locale display name as defined in the current Locale or the original string if not present
      */
@@ -513,7 +540,7 @@ public class JMeterUtils implements UnitTestManager {
             String lineEnd = System.getProperty("line.separator"); // $NON-NLS-1$
             fileReader = new BufferedReader(new InputStreamReader(JMeterUtils.class.getClassLoader()
                     .getResourceAsStream(name)));
-            StringBuffer text = new StringBuffer();
+            StringBuilder text = new StringBuilder();
             String line = "NOTNULL"; // $NON-NLS-1$
             while (line != null) {
                 line = fileReader.readLine();
@@ -541,7 +568,7 @@ public class JMeterUtils implements UnitTestManager {
      *            Description of Parameter
      * @return The Timers value
      */
-    public static Vector getTimers(Properties properties) {
+    public static Vector<Object> getTimers(Properties properties) {
         return instantiate(getVector(properties, "timer."), // $NON-NLS-1$
                 "org.apache.jmeter.timers.Timer"); // $NON-NLS-1$
     }
@@ -553,7 +580,7 @@ public class JMeterUtils implements UnitTestManager {
      *            Description of Parameter
      * @return The Visualizers value
      */
-    public static Vector getVisualizers(Properties properties) {
+    public static Vector<Object> getVisualizers(Properties properties) {
         return instantiate(getVector(properties, "visualizer."), // $NON-NLS-1$
                 "org.apache.jmeter.visualizers.Visualizer"); // $NON-NLS-1$
     }
@@ -566,10 +593,10 @@ public class JMeterUtils implements UnitTestManager {
      * @return The Controllers value
      */
     // TODO - does not appear to be called directly
-    public static Vector getControllers(Properties properties) {
+    public static Vector<Object> getControllers(Properties properties) {
         String name = "controller."; // $NON-NLS-1$
-        Vector v = new Vector();
-        Enumeration names = properties.keys();
+        Vector<Object> v = new Vector<Object>();
+        Enumeration<?> names = properties.keys();
         while (names.hasMoreElements()) {
             String prop = (String) names.nextElement();
             if (prop.startsWith(name)) {
@@ -591,7 +618,7 @@ public class JMeterUtils implements UnitTestManager {
      * @return The TestSamples value
      */
     public static String[] getTestSamples(Properties properties, String name) {
-        return (String[]) getVector(properties, name + ".testsample").toArray(new String[0]); // $NON-NLS-1$
+        return getVector(properties, name + ".testsample").toArray(new String[0]); // $NON-NLS-1$
     }
 
     /**
@@ -618,10 +645,9 @@ public class JMeterUtils implements UnitTestManager {
      * Creates the vector of alias strings.
      *
      * @param properties
-     *            Description of Parameter
      * @return The Alias value
      */
-    public static Hashtable getAlias(Properties properties) {
+    public static Hashtable<String, String> getAlias(Properties properties) {
         return getHashtable(properties, "alias."); // $NON-NLS-1$
     }
 
@@ -635,9 +661,9 @@ public class JMeterUtils implements UnitTestManager {
      *            Description of Parameter
      * @return The Vector value
      */
-    public static Vector getVector(Properties properties, String name) {
-        Vector v = new Vector();
-        Enumeration names = properties.keys();
+    public static Vector<String> getVector(Properties properties, String name) {
+        Vector<String> v = new Vector<String>();
+        Enumeration<?> names = properties.keys();
         while (names.hasMoreElements()) {
             String prop = (String) names.nextElement();
             if (prop.startsWith(name)) {
@@ -651,19 +677,18 @@ public class JMeterUtils implements UnitTestManager {
      * Creates a table of strings for all the properties that start with a
      * common prefix.
      *
-     * @param properties
-     *            Description of Parameter
-     * @param name
-     *            Description of Parameter
-     * @return The Hashtable value
+     * @param properties input to search
+     * @param prefix to match against properties
+     * @return a Hashtable where the keys are the original keys with the prefix removed
      */
-    public static Hashtable getHashtable(Properties properties, String name) {
-        Hashtable t = new Hashtable();
-        Enumeration names = properties.keys();
+    public static Hashtable<String, String> getHashtable(Properties properties, String prefix) {
+        Hashtable<String, String> t = new Hashtable<String, String>();
+        Enumeration<?> names = properties.keys();
+        final int length = prefix.length();
         while (names.hasMoreElements()) {
             String prop = (String) names.nextElement();
-            if (prop.startsWith(name)) {
-                t.put(prop.substring(name.length()), properties.getProperty(prop));
+            if (prop.startsWith(prefix)) {
+                t.put(prop.substring(length), properties.getProperty(prop));
             }
         }
         return t;
@@ -786,8 +811,9 @@ public class JMeterUtils implements UnitTestManager {
     /**
      * Sets the selection of the JComboBox to the Object 'name' from the list in
      * namVec.
+     * NOTUSED?
      */
-    public static void selJComboBoxItem(Properties properties, JComboBox combo, Vector namVec, String name) {
+    public static void selJComboBoxItem(Properties properties, JComboBox combo, Vector<?> namVec, String name) {
         int idx = namVec.indexOf(name);
         combo.setSelectedIndex(idx);
         // Redisplay.
@@ -814,9 +840,9 @@ public class JMeterUtils implements UnitTestManager {
         }
 
         try {
-            Class c = Class.forName(impls);
+            Class<?> c = Class.forName(impls);
             try {
-                Class o = Class.forName(className);
+                Class<?> o = Class.forName(className);
                 Object res = o.newInstance();
                 if (c.isInstance(res)) {
                     return res;
@@ -846,13 +872,13 @@ public class JMeterUtils implements UnitTestManager {
      *            Description of Parameter
      * @return Description of the Returned Value
      */
-    public static Vector instantiate(Vector v, String className) {
-        Vector i = new Vector();
+    public static Vector<Object> instantiate(Vector<String> v, String className) {
+        Vector<Object> i = new Vector<Object>();
         try {
-            Class c = Class.forName(className);
-            Enumeration elements = v.elements();
+            Class<?> c = Class.forName(className);
+            Enumeration<String> elements = v.elements();
             while (elements.hasMoreElements()) {
-                String name = (String) elements.nextElement();
+                String name = elements.nextElement();
                 try {
                     Object o = Class.forName(name).newInstance();
                     if (c.isInstance(o)) {
@@ -884,8 +910,8 @@ public class JMeterUtils implements UnitTestManager {
      * @return Description of the Returned Value
      */
     //TODO move to JOrphanUtils ?
-    public static Vector tokenize(String string, String separator) {
-        Vector v = new Vector();
+    public static Vector<String> tokenize(String string, String separator) {
+        Vector<String> v = new Vector<String>();
         StringTokenizer s = new StringTokenizer(string, separator);
         while (s.hasMoreTokens()) {
             v.addElement(s.nextToken());
@@ -1010,7 +1036,7 @@ public class JMeterUtils implements UnitTestManager {
      */
     //TODO - move to JOrphanUtils?
     public static String unsplit(Object[] splittee, Object splitChar) {
-        StringBuffer retVal = new StringBuffer();
+        StringBuilder retVal = new StringBuilder();
         int count = -1;
         while (++count < splittee.length) {
             if (splittee[count] != null) {
@@ -1040,7 +1066,7 @@ public class JMeterUtils implements UnitTestManager {
      */
     //TODO - move to JOrphanUtils?
     public static String unsplit(Object[] splittee, Object splitChar, String def) {
-        StringBuffer retVal = new StringBuffer();
+        StringBuilder retVal = new StringBuilder();
         int count = -1;
         while (++count < splittee.length) {
             if (splittee[count] != null) {
@@ -1126,9 +1152,9 @@ public class JMeterUtils implements UnitTestManager {
     }
 
     /**
-     * Returns the cached result from calling 
+     * Returns the cached result from calling
      * InetAddress.getLocalHost().getHostAddress()
-     * 
+     *
      * @return String representation of local IP address
      */
     public static synchronized String getLocalHostIP(){
@@ -1139,9 +1165,9 @@ public class JMeterUtils implements UnitTestManager {
     }
 
     /**
-     * Returns the cached result from calling 
+     * Returns the cached result from calling
      * InetAddress.getLocalHost().getHostName()
-     * 
+     *
      * @return local host name
      */
     public static synchronized String getLocalHostName(){
@@ -1152,9 +1178,9 @@ public class JMeterUtils implements UnitTestManager {
     }
 
     /**
-     * Returns the cached result from calling 
+     * Returns the cached result from calling
      * InetAddress.getLocalHost().getCanonicalHostName()
-     * 
+     *
      * @return local host name in canonical form
      */
     public static synchronized String getLocalHostFullName(){
