@@ -26,7 +26,6 @@ import java.util.Arrays;
 import java.util.Collection;
 import java.util.HashMap;
 import java.util.Iterator;
-import java.util.LinkedList;
 import java.util.Map;
 import java.util.Set;
 
@@ -50,27 +49,29 @@ import java.util.Set;
  */
 public class HashTree implements Serializable, Map, Cloneable {
 
+    private static final long serialVersionUID = 240L;
+
     // GetLoggerForClass() uses ClassContext, which
     // causes a Security violation in RemoteJMeterImpl
     // so we use getLoggerFor() instead
 //    private static final Logger log = LoggingManager.getLoggerFor(HashTree.class.getName());
-    
+
     // Used for the RuntimeException to short-circuit the traversal
     private static final String FOUND = "found"; // $NON-NLS-1$
 
-    protected final Map data;
+    protected final Map<Object, HashTree> data;
 
     /**
      * Creates an empty new HashTree.
      */
     public HashTree() {
-        data = new HashMap();
+        data = new HashMap<Object, HashTree>();
     }
 
     /**
      * Allow subclasses to provide their own Map.
      */
-    protected HashTree(Map _map) {
+    protected HashTree(Map<Object, HashTree> _map) {
         data = _map;
     }
 
@@ -80,7 +81,7 @@ public class HashTree implements Serializable, Map, Cloneable {
      * @param key
      */
     public HashTree(Object key) {
-        data = new HashMap();
+        data = new HashMap<Object, HashTree>();
         data.put(key, new HashTree());
     }
 
@@ -105,7 +106,7 @@ public class HashTree implements Serializable, Map, Cloneable {
      *
      * @see java.util.Map#entrySet()
      */
-    public Set entrySet() {
+    public Set<?> entrySet() {
         return data.entrySet();
     }
 
@@ -151,7 +152,7 @@ public class HashTree implements Serializable, Map, Cloneable {
      *
      * @see java.util.Map#values()
      */
-    public Collection values() {
+    public Collection<HashTree> values() {
         return data.values();
     }
 
@@ -176,7 +177,7 @@ public class HashTree implements Serializable, Map, Cloneable {
      * @param newTree
      */
     public void add(HashTree newTree) {
-        Iterator iter = newTree.list().iterator();
+        Iterator<?> iter = newTree.list().iterator();
         while (iter.hasNext()) {
             Object item = iter.next();
             add(item);
@@ -191,9 +192,9 @@ public class HashTree implements Serializable, Map, Cloneable {
      * @param keys
      *            a collection of objects to be added to the created HashTree.
      */
-    public HashTree(Collection keys) {
-        data = new HashMap();
-        Iterator it = keys.iterator();
+    public HashTree(Collection<?> keys) {
+        data = new HashMap<Object, HashTree>();
+        Iterator<?> it = keys.iterator();
         while (it.hasNext()) {
             data.put(it.next(), new HashTree());
         }
@@ -204,7 +205,7 @@ public class HashTree implements Serializable, Map, Cloneable {
      * top-level nodes in the tree.
      */
     public HashTree(Object[] keys) {
-        data = new HashMap();
+        data = new HashMap<Object, HashTree>();
         for (int x = 0; x < keys.length; x++) {
             data.put(keys[x], new HashTree());
         }
@@ -286,7 +287,7 @@ public class HashTree implements Serializable, Map, Cloneable {
      *            Collection of objects to be added as keys in the secondary
      *            node
      */
-    public void set(Object key, Collection values) {
+    public void set(Object key, Collection<?> values) {
         data.put(key, createNewTree(values));
     }
 
@@ -324,7 +325,7 @@ public class HashTree implements Serializable, Map, Cloneable {
      * @param values
      *            Collection of values to be added as keys to bottom-most node
      */
-    public void set(Object[] treePath, Collection values) {
+    public void set(Object[] treePath, Collection<?> values) {
         if (treePath != null) {
             set(Arrays.asList(treePath), values);
         }
@@ -344,7 +345,7 @@ public class HashTree implements Serializable, Map, Cloneable {
      * @param values
      *            array of values to be added as keys to bottom-most node
      */
-    public void set(Collection treePath, Object[] values) {
+    public void set(Collection<?> treePath, Object[] values) {
         HashTree tree = addTreePath(treePath);
         tree.set(Arrays.asList(values));
     }
@@ -356,7 +357,7 @@ public class HashTree implements Serializable, Map, Cloneable {
      * @param values
      *            Collection of objects to set as nodes.
      */
-    public void set(Collection values) {
+    public void set(Collection<?> values) {
         clear();
         this.add(values);
     }
@@ -375,7 +376,7 @@ public class HashTree implements Serializable, Map, Cloneable {
      * @param values
      *            collection of values to be added as keys to bottom-most node
      */
-    public void set(Collection treePath, Collection values) {
+    public void set(Collection<?> treePath, Collection<?> values) {
         HashTree tree = addTreePath(treePath);
         tree.set(values);
     }
@@ -413,8 +414,8 @@ public class HashTree implements Serializable, Map, Cloneable {
      * @param keys
      *            Collection of Keys to be added to HashTree.
      */
-    public void add(Collection keys) {
-        Iterator it = keys.iterator();
+    public void add(Collection<?> keys) {
+        Iterator<?> it = keys.iterator();
         while (it.hasNext()) {
             add(it.next());
         }
@@ -459,7 +460,7 @@ public class HashTree implements Serializable, Map, Cloneable {
      *            Collection of objects to be added as keys in the secondary
      *            node
      */
-    public void add(Object key, Collection values) {
+    public void add(Object key, Collection<?> values) {
         add(key);
         getTree(key).add(values);
     }
@@ -494,7 +495,7 @@ public class HashTree implements Serializable, Map, Cloneable {
      * @param values
      *            collection of values to be added as keys to bottom-most node
      */
-    public void add(Object[] treePath, Collection values) {
+    public void add(Object[] treePath, Collection<?> values) {
         if (treePath != null) {
             add(Arrays.asList(treePath), values);
         }
@@ -516,7 +517,7 @@ public class HashTree implements Serializable, Map, Cloneable {
      * @param values
      *            array of values to be added as keys to bottom-most node
      */
-    public void add(Collection treePath, Object[] values) {
+    public void add(Collection<?> treePath, Object[] values) {
         HashTree tree = addTreePath(treePath);
         tree.add(Arrays.asList(values));
     }
@@ -533,7 +534,7 @@ public class HashTree implements Serializable, Map, Cloneable {
      * @param value
      *            Object to add as a node to bottom-most node
      */
-    public HashTree add(Collection treePath, Object value) {
+    public HashTree add(Collection<?> treePath, Object value) {
         HashTree tree = addTreePath(treePath);
         return tree.add(value);
     }
@@ -550,14 +551,14 @@ public class HashTree implements Serializable, Map, Cloneable {
      * @param values
      *            Collection of values to be added as keys to bottom-most node
      */
-    public void add(Collection treePath, Collection values) {
+    public void add(Collection<?> treePath, Collection<?> values) {
         HashTree tree = addTreePath(treePath);
         tree.add(values);
     }
 
-    protected HashTree addTreePath(Collection treePath) {
+    protected HashTree addTreePath(Collection<?> treePath) {
         HashTree tree = this;
-        Iterator iter = treePath.iterator();
+        Iterator<?> iter = treePath.iterator();
         while (iter.hasNext()) {
             Object temp = iter.next();
             tree.add(temp);
@@ -573,7 +574,7 @@ public class HashTree implements Serializable, Map, Cloneable {
      *            Key used to find appropriate HashTree()
      */
     public HashTree getTree(Object key) {
-        return (HashTree) data.get(key);
+        return data.get(key);
     }
 
     /**
@@ -606,6 +607,7 @@ public class HashTree implements Serializable, Map, Cloneable {
      * contents of the tree are not cloned).
      *
      */
+    @Override
     public Object clone() {
         HashTree newTree = new HashTree();
         cloneTree(newTree);
@@ -613,7 +615,7 @@ public class HashTree implements Serializable, Map, Cloneable {
     }
 
     protected void cloneTree(HashTree newTree) {
-        Iterator iter = list().iterator();
+        Iterator<?> iter = list().iterator();
         while (iter.hasNext()) {
             Object key = iter.next();
             newTree.set(key, (HashTree) getTree(key).clone());
@@ -658,7 +660,7 @@ public class HashTree implements Serializable, Map, Cloneable {
      *
      * @return HashTree
      */
-    protected HashTree createNewTree(Collection values) {
+    protected HashTree createNewTree(Collection<?> values) {
         return new HashTree(values);
     }
 
@@ -670,7 +672,7 @@ public class HashTree implements Serializable, Map, Cloneable {
      *            Collection of keys
      * @return HashTree at the end of the recursion
      */
-    public HashTree getTree(Collection treePath) {
+    public HashTree getTree(Collection<?> treePath) {
         return getTreePath(treePath);
     }
 
@@ -696,11 +698,11 @@ public class HashTree implements Serializable, Map, Cloneable {
      * @return Set of all keys in found HashTree.
      */
     public Collection list(Object key) {
-        HashTree temp = (HashTree) data.get(key);
+        HashTree temp = data.get(key);
         if (temp != null) {
             return temp.list();
         }
-        return new LinkedList();
+        return new HashTree().list();
     }
 
     /**
@@ -741,12 +743,12 @@ public class HashTree implements Serializable, Map, Cloneable {
      *            List of keys used to recurse into HashTree structure
      * @return Set of all keys found in end HashTree
      */
-    public Collection list(Collection treePath) {
+    public Collection list(Collection<?> treePath) {
         HashTree tree = getTreePath(treePath);
         if (tree != null) {
             return tree.list();
         }
-        return new LinkedList();
+        return new HashTree().list();
     }
 
     /**
@@ -817,14 +819,14 @@ public class HashTree implements Serializable, Map, Cloneable {
      *            list of keys used to recurse into HashTree structure
      * @return array of all keys found in end HashTree
      */
-    public Object[] getArray(Collection treePath) {
+    public Object[] getArray(Collection<?> treePath) {
         HashTree tree = getTreePath(treePath);
         return (tree != null) ? tree.getArray() : null;
     }
 
-    protected HashTree getTreePath(Collection treePath) {
+    protected HashTree getTreePath(Collection<?> treePath) {
         HashTree tree = this;
-        Iterator iter = treePath.iterator();
+        Iterator<?> iter = treePath.iterator();
         while (iter.hasNext()) {
             if (tree == null) {
                 return null;
@@ -840,6 +842,7 @@ public class HashTree implements Serializable, Map, Cloneable {
      *
      * @see java.lang.Object#hashCode()
      */
+    @Override
     public int hashCode() {
         return data.hashCode() * 7;
     }
@@ -853,6 +856,7 @@ public class HashTree implements Serializable, Map, Cloneable {
      *            Object to be compared against
      * @see java.lang.Object#equals(Object)
      */
+    @Override
     public boolean equals(Object o) {
         if (!(o instanceof HashTree)) {
             return false;
@@ -960,7 +964,7 @@ public class HashTree implements Serializable, Map, Cloneable {
      * @see HashTreeTraverser
      */
     public void traverse(HashTreeTraverser visitor) {
-        Iterator iter = list().iterator();
+        Iterator<?> iter = list().iterator();
         while (iter.hasNext()) {
             Object item = iter.next();
             visitor.addNode(item, getTree(item));
@@ -977,7 +981,7 @@ public class HashTree implements Serializable, Map, Cloneable {
         if (list().size() == 0) {
             visitor.processPath();
         } else {
-            Iterator iter = list().iterator();
+            Iterator<?> iter = list().iterator();
             while (iter.hasNext()) {
                 Object item = iter.next();
                 final HashTree treeItem = getTree(item);
@@ -990,9 +994,10 @@ public class HashTree implements Serializable, Map, Cloneable {
 
     /**
      * Generate a printable representation of the tree.
-     * 
+     *
      * @return a representation of the tree
      */
+    @Override
     public String toString() {
         ConvertToString converter = new ConvertToString();
         try {
@@ -1038,9 +1043,9 @@ public class HashTree implements Serializable, Map, Cloneable {
     }
 
     private static class ConvertToString implements HashTreeTraverser {
-        private final StringBuffer string = new StringBuffer(getClass().getName() + "{");
+        private final StringBuilder string = new StringBuilder(getClass().getName() + "{");
 
-        private final StringBuffer spaces = new StringBuffer();
+        private final StringBuilder spaces = new StringBuilder();
 
         private int depth = 0;
 
@@ -1058,6 +1063,7 @@ public class HashTree implements Serializable, Map, Cloneable {
         public void processPath() {
         }
 
+        @Override
         public String toString() {
             string.append("\n}");
             return string.toString();
