@@ -20,10 +20,11 @@ package org.apache.jorphan.gui;
 
 import java.util.List;
 
+import javax.swing.JComponent;
 import javax.swing.event.ChangeListener;
 
 /**
- * @version $Revision: 674365 $
+ * @version $Revision: 804518 $
  */
 public interface JLabeledField {
     public String getText();
@@ -34,5 +35,5 @@ public interface JLabeledField {
 
     public void addChangeListener(ChangeListener pChangeListener);
 
-    public List getComponentList();
+    public List<JComponent> getComponentList();
 }
\ No newline at end of file
