@@ -23,23 +23,14 @@ package org.apache.jorphan.math;
 
 import java.util.Comparator;
 
-public class NumberComparator implements Comparator {
+public class NumberComparator implements Comparator<Number[]> {
 
-    /**
-     *
-     */
     public NumberComparator() {
         super();
     }
 
-    /*
-     * (non-Javadoc)
-     *
-     * @see java.util.Comparator#compare(java.lang.Object, java.lang.Object)
-     */
-    public int compare(Object val1, Object val2) {
-        Number[] n1 = (Number[]) val1;
-        Number[] n2 = (Number[]) val2;
+    /** {@inheritDoc} */
+    public int compare(Number[] n1, Number[] n2) {
         if (n1[0].longValue() < n2[0].longValue()) {
             return -1;
         } else if (n1[0].longValue() == n2[0].longValue()) {
