@@ -21,6 +21,8 @@ package org.apache.jorphan.reflect;
 import java.io.File;
 import java.io.FilenameFilter;
 import java.io.IOException;
+import java.lang.annotation.Annotation;
+import java.lang.reflect.Method;
 import java.lang.reflect.Modifier;
 import java.util.ArrayList;
 import java.util.Enumeration;
@@ -30,6 +32,7 @@ import java.util.List;
 import java.util.Set;
 import java.util.StringTokenizer;
 import java.util.TreeSet;
+import java.util.zip.ZipEntry;
 import java.util.zip.ZipFile;
 
 import org.apache.jorphan.logging.LoggingManager;
@@ -57,10 +60,10 @@ public final class ClassFinder {
      *
      *
      */
-    private static class FilterTreeSet extends TreeSet{
-        private static final long serialVersionUID = 233L;
+    private static class FilterTreeSet extends TreeSet<String>{
+        private static final long serialVersionUID = 234L;
 
-        private final Class[] parents; // parent classes to check
+        private final Class<?>[] parents; // parent classes to check
         private final boolean inner; // are inner classes OK?
 
         // hack to reduce the need to load every class in non-GUI mode, which only needs functions
@@ -71,7 +74,7 @@ public final class ClassFinder {
         private final transient ClassLoader contextClassLoader
             = Thread.currentThread().getContextClassLoader(); // Potentially expensive; do it once
 
-        FilterTreeSet(Class []parents, boolean inner, String contains, String notContains){
+        FilterTreeSet(Class<?> []parents, boolean inner, String contains, String notContains){
             super();
             this.parents=parents;
             this.inner=inner;
@@ -83,16 +86,16 @@ public final class ClassFinder {
          * Override the superclass so we only add classnames that
          * meet the criteria.
          *
-         * @param o - classname (must be a String)
+         * @param s - classname (must be a String)
          * @return true if it is a new entry
          *
          * @see java.util.TreeSet#add(java.lang.Object)
          */
-        public boolean add(Object o){
-            if (contains(o)) {
+        @Override
+        public boolean add(String s){
+            if (contains(s)) {
                 return false;// No need to check it again
             }
-            String s = (String) o;// we only expect Strings
             if (contains!=null && s.indexOf(contains) == -1){
                 return false; // It does not contain a required string
             }
@@ -108,21 +111,57 @@ public final class ClassFinder {
         }
     }
 
+    private static class AnnoFilterTreeSet extends TreeSet<String>{
+        private static final long serialVersionUID = 240L;
+
+        private final boolean inner; // are inner classes OK?
+
+        private final Class<? extends Annotation>[] annotations; // annotation classes to check
+        private final transient ClassLoader contextClassLoader
+            = Thread.currentThread().getContextClassLoader(); // Potentially expensive; do it once
+        AnnoFilterTreeSet(Class<? extends Annotation> []annotations, boolean inner){
+            super();
+            this.annotations = annotations;
+            this.inner=inner;
+        }
+        /**
+         * Override the superclass so we only add classnames that
+         * meet the criteria.
+         *
+         * @param s - classname (must be a String)
+         * @return true if it is a new entry
+         *
+         * @see java.util.TreeSet#add(java.lang.Object)
+         */
+        @Override
+        public boolean add(String s){
+            if (contains(s)) {
+                return false;// No need to check it again
+            }
+            if ((s.indexOf("$") == -1) || inner) { // $NON-NLS-1$
+                if (hasAnnotationOnMethod(annotations,s, contextClassLoader)) {
+                    return super.add(s);
+                }
+            }
+            return false;
+        }
+    }
+
     /**
      * Convenience method for
-     * <code>findClassesThatExtend(Class[], boolean)</code>
+     * {@link #findClassesThatExtend(String[], Class[], boolean)}
      * with the option to include inner classes in the search set to false.
      *
-     * @return List containing discovered classes.
+     * @return List of Strings containing discovered class names.
      */
-    public static List findClassesThatExtend(String[] paths, Class[] superClasses)
+    public static List<String> findClassesThatExtend(String[] paths, Class<?>[] superClasses)
         throws IOException {
         return findClassesThatExtend(paths, superClasses, false);
     }
 
     // For each directory in the search path, add all the jars found there
     private static String[] addJarsInPath(String[] paths) {
-        Set fullList = new HashSet();
+        Set<String> fullList = new HashSet<String>();
         for (int i = 0; i < paths.length; i++) {
             final String path = paths[i];
             fullList.add(path); // Keep the unexpanded path
@@ -141,7 +180,7 @@ public final class ClassFinder {
                 }
             }
         }
-        return (String[]) fullList.toArray(new String[0]);
+        return fullList.toArray(new String[0]);
     }
 
     /**
@@ -152,8 +191,8 @@ public final class ClassFinder {
      *
      * @return List containing discovered classes
      */
-    public static List findClassesThatExtend(String[] strPathsOrJars,
-            final Class[] superClasses, final boolean innerClasses)
+    public static List<String> findClassesThatExtend(String[] strPathsOrJars,
+            final Class<?>[] superClasses, final boolean innerClasses)
             throws IOException  {
         return findClassesThatExtend(strPathsOrJars,superClasses,innerClasses,null,null);
     }
@@ -168,14 +207,60 @@ public final class ClassFinder {
      *
      * @return List containing discovered classes
      */
-    public static List findClassesThatExtend(String[] strPathsOrJars,
-            final Class[] superClasses, final boolean innerClasses,
+    public static List<String> findClassesThatExtend(String[] strPathsOrJars,
+            final Class<?>[] superClasses, final boolean innerClasses,
             String contains, String notContains)
             throws IOException  {
+        return findClassesThatExtend(strPathsOrJars, superClasses, innerClasses, contains, notContains, false);
+    }
 
+    /**
+     * Find classes in the provided path(s)/jar(s) that extend the class(es).
+     * @param strPathsOrJars - pathnames or jarfiles to search for classes
+     * @param annotations - required annotations
+     * @param innerClasses - should we include inner classes?
+     *
+     * @return List containing discovered classes
+     */
+    public static List<String> findAnnotatedClasses(String[] strPathsOrJars,
+            final Class<? extends Annotation>[] annotations, final boolean innerClasses)
+            throws IOException  {
+        return findClassesThatExtend(strPathsOrJars, annotations, innerClasses, null, null, true);
+    }
+
+    /**
+     * Find classes in the provided path(s)/jar(s) that extend the class(es).
+     * Inner classes are not searched.
+     *
+     * @param strPathsOrJars - pathnames or jarfiles to search for classes
+     * @param annotations - required annotations
+     *
+     * @return List containing discovered classes
+     */
+    public static List<String> findAnnotatedClasses(String[] strPathsOrJars,
+            final Class<? extends Annotation>[] annotations)
+            throws IOException  {
+        return findClassesThatExtend(strPathsOrJars, annotations, false, null, null, true);
+    }
+
+    /**
+     * Find classes in the provided path(s)/jar(s) that extend the class(es).
+     * @param strPathsOrJars - pathnames or jarfiles to search for classes
+     * @param classNames - required parent class(es) or annotations
+     * @param innerClasses - should we include inner classes?
+     * @param contains - classname should contain this string
+     * @param notContains - classname should not contain this string
+     * @param annotations - true if classnames are annotations
+     *
+     * @return List containing discovered classes
+     */
+    private static List<String> findClassesThatExtend(String[] strPathsOrJars,
+                final Class<?>[] classNames, final boolean innerClasses,
+                String contains, String notContains, boolean annotations)
+                throws IOException  {
         if (log.isDebugEnabled()) {
-            for (int i = 0; i < superClasses.length ; i++){
-                log.debug("superclass: "+superClasses[i].getName());
+            for (int i = 0; i < classNames.length ; i++){
+                log.debug("superclass: "+classNames[i].getName());
             }
         }
 
@@ -189,20 +274,26 @@ public final class ClassFinder {
         }
 
         // Now eliminate any classpath entries that do not "match" the search
-        List listPaths = getClasspathMatches(strPathsOrJars);
+        List<String> listPaths = getClasspathMatches(strPathsOrJars);
         if (log.isDebugEnabled()) {
-            Iterator tIter = listPaths.iterator();
+            Iterator<String> tIter = listPaths.iterator();
             while (tIter.hasNext()) {
                 log.debug("listPaths : " + tIter.next());
             }
         }
 
-        Set listClasses = new FilterTreeSet(superClasses, innerClasses, contains, notContains);
+        @SuppressWarnings("unchecked") // Should only be called with classes that extend annotations
+        final Class<? extends Annotation>[] annoclassNames = (Class<? extends Annotation>[]) classNames;
+        Set<String> listClasses =
+            annotations ?
+                new AnnoFilterTreeSet(annoclassNames, innerClasses)
+                :
+                new FilterTreeSet(classNames, innerClasses, contains, notContains);
         // first get all the classes
         findClassesInPaths(listPaths, listClasses);
         if (log.isDebugEnabled()) {
             log.debug("listClasses.size()="+listClasses.size());
-            Iterator tIter = listClasses.iterator();
+            Iterator<String> tIter = listClasses.iterator();
             while (tIter.hasNext()) {
                 log.debug("listClasses : " + tIter.next());
             }
@@ -218,13 +309,13 @@ public final class ClassFinder {
 //            }
 //        }
 
-        return new ArrayList(listClasses);//subClassList);
+        return new ArrayList<String>(listClasses);//subClassList);
     }
 
     /*
      * Returns the classpath entries that match the search list of jars and paths
      */
-    private static List getClasspathMatches(String[] strPathsOrJars) {
+    private static List<String> getClasspathMatches(String[] strPathsOrJars) {
         final String javaClassPath = System.getProperty("java.class.path"); // $NON-NLS-1$
         StringTokenizer stPaths =
             new StringTokenizer(javaClassPath,
@@ -237,7 +328,7 @@ public final class ClassFinder {
         }
 
         // find all jar files or paths that end with strPathOrJar
-        ArrayList listPaths = new ArrayList();
+        ArrayList<String> listPaths = new ArrayList<String>();
         String strPath = null;
         while (stPaths.hasMoreTokens()) {
             strPath = fixPathEntry(stPaths.nextToken());
@@ -370,11 +461,11 @@ public final class ClassFinder {
      * @param contextClassLoader the classloader to use
      * @return
      */
-    private static boolean isChildOf(Class [] parentClasses, String strClassName,
+    private static boolean isChildOf(Class<?> [] parentClasses, String strClassName,
             ClassLoader contextClassLoader){
             // might throw an exception, assume this is ignorable
             try {
-                Class c = Class.forName(strClassName, false, contextClassLoader);
+                Class<?> c = Class.forName(strClassName, false, contextClassLoader);
 
                 if (!c.isInterface() && !Modifier.isAbstract(c.getModifiers())) {
                     for (int i=0; i< parentClasses.length; i++) {
@@ -383,12 +474,36 @@ public final class ClassFinder {
                         }
                     }
                 }
-            } catch (Throwable ignored) {
+            } catch (UnsupportedClassVersionError ignored) {
+                log.debug(ignored.getLocalizedMessage());
+            } catch (NoClassDefFoundError ignored) {
+                log.debug(ignored.getLocalizedMessage());
+            } catch (ClassNotFoundException ignored) {
                 log.debug(ignored.getLocalizedMessage());
             }
         return false;
     }
 
+    private static boolean hasAnnotationOnMethod(Class<? extends Annotation>[] annotations, String classInQuestion,
+        ClassLoader contextClassLoader ){
+        try{
+            Class<?> c = Class.forName(classInQuestion, false, contextClassLoader);
+            for(Method method : c.getMethods()) {
+                for(int i = 0;i<annotations.length;i++) {
+                    Class<? extends Annotation> annotation = annotations[i];
+                    if(method.isAnnotationPresent(annotation)) {
+                        return true;
+                    }
+                }
+            }
+        } catch (NoClassDefFoundError ignored) {
+            log.debug(ignored.getLocalizedMessage());
+        } catch (ClassNotFoundException ignored) {
+            log.debug(ignored.getLocalizedMessage());
+        }
+        return false;
+    }
+
 
     /*
      * Converts a class file from the text stored in a Jar file to a version
@@ -406,7 +521,7 @@ public final class ClassFinder {
         return strClassName;
     }
 
-    private static void findClassesInOnePath(String strPath, Set listClasses) throws IOException {
+    private static void findClassesInOnePath(String strPath, Set<String> listClasses) throws IOException {
         File file = new File(strPath);
         if (file.isDirectory()) {
             findClassesInPathsDir(strPath, file, listClasses);
@@ -414,15 +529,15 @@ public final class ClassFinder {
             ZipFile zipFile = null;
             try {
                 zipFile = new ZipFile(file);
-                Enumeration entries = zipFile.entries();
+                Enumeration<? extends ZipEntry> entries = zipFile.entries();
                 while (entries.hasMoreElements()) {
                     String strEntry = entries.nextElement().toString();
                     if (strEntry.endsWith(DOT_CLASS)) {
                         listClasses.add(fixClassName(strEntry));
-                        }
                     }
+                }
             } catch (IOException e) {
-                log.warn("Can not open the jar " + strPath + " " + e.getLocalizedMessage());
+                log.warn("Can not open the jar " + strPath + " " + e.getLocalizedMessage(),e);
             }
             finally {
                 if(zipFile != null) {
@@ -432,14 +547,14 @@ public final class ClassFinder {
         }
     }
 
-    private static void findClassesInPaths(List listPaths, Set listClasses) throws IOException {
-        Iterator iterPaths = listPaths.iterator();
+    private static void findClassesInPaths(List<String> listPaths, Set<String> listClasses) throws IOException {
+        Iterator<String> iterPaths = listPaths.iterator();
         while (iterPaths.hasNext()) {
-            findClassesInOnePath((String) iterPaths.next(), listClasses);
+            findClassesInOnePath(iterPaths.next(), listClasses);
         }
     }
 
-    private static void findClassesInPathsDir(String strPathElement, File dir, Set listClasses) throws IOException {
+    private static void findClassesInPathsDir(String strPathElement, File dir, Set<String> listClasses) throws IOException {
         String[] list = dir.list();
         for (int i = 0; i < list.length; i++) {
             File file = new File(dir, list[i]);
