@@ -17,6 +17,7 @@
  */
 package org.apache.jmeter.protocol.java.sampler;
 
+import java.lang.annotation.Annotation;
 import java.lang.reflect.Constructor;
 import java.lang.reflect.InvocationTargetException;
 import java.lang.reflect.Method;
@@ -24,7 +25,6 @@ import java.lang.reflect.Modifier;
 import java.util.Enumeration;
 
 import junit.framework.AssertionFailedError;
-import junit.framework.ComparisonFailure;
 import junit.framework.Protectable;
 import junit.framework.TestCase;
 import junit.framework.TestFailure;
@@ -33,8 +33,13 @@ import junit.framework.TestResult;
 import org.apache.jmeter.samplers.AbstractSampler;
 import org.apache.jmeter.samplers.Entry;
 import org.apache.jmeter.samplers.SampleResult;
+import org.apache.jmeter.testelement.ThreadListener;
 import org.apache.jorphan.logging.LoggingManager;
 import org.apache.log.Logger;
+import org.junit.After;
+import org.junit.Before;
+import org.junit.Test;
+import org.junit.Test.None;
 
 /**
  *
@@ -43,57 +48,70 @@ import org.apache.log.Logger;
  * constructor first. If the test class does not declare a string
  * constructor, the sampler will try empty constructor.
  */
-public class JUnitSampler extends AbstractSampler {
+public class JUnitSampler extends AbstractSampler implements ThreadListener {
 
     private static final Logger log = LoggingManager.getLoggerForClass();
 
-    private static final long serialVersionUID = 2331L; // Remember to change this when the class changes ...
+    private static final long serialVersionUID = 240L; // Remember to change this when the class changes ...
 
-    /**
-     * Property key representing the classname of the JavaSamplerClient to
-     * user.
-     */
-    public static final String CLASSNAME = "junitSampler.classname";
-    public static final String CONSTRUCTORSTRING = "junitsampler.constructorstring";
-    public static final String METHOD = "junitsampler.method";
-    public static final String ERROR = "junitsampler.error";
-    public static final String ERRORCODE = "junitsampler.error.code";
-    public static final String FAILURE = "junitsampler.failure";
-    public static final String FAILURECODE = "junitsampler.failure.code";
-    public static final String SUCCESS = "junitsampler.success";
-    public static final String SUCCESSCODE = "junitsampler.success.code";
-    public static final String FILTER = "junitsampler.pkg.filter";
-    public static final String DOSETUP = "junitsampler.exec.setup";
-    public static final String APPEND_ERROR = "junitsampler.append.error";
-    public static final String APPEND_EXCEPTION = "junitsampler.append.exception";
+    //++ JMX file attributes - do not change
+    private static final String CLASSNAME = "junitSampler.classname";
+    private static final String CONSTRUCTORSTRING = "junitsampler.constructorstring";
+    private static final String METHOD = "junitsampler.method";
+    private static final String ERROR = "junitsampler.error";
+    private static final String ERRORCODE = "junitsampler.error.code";
+    private static final String FAILURE = "junitsampler.failure";
+    private static final String FAILURECODE = "junitsampler.failure.code";
+    private static final String SUCCESS = "junitsampler.success";
+    private static final String SUCCESSCODE = "junitsampler.success.code";
+    private static final String FILTER = "junitsampler.pkg.filter";
+    private static final String DOSETUP = "junitsampler.exec.setup";
+    private static final String APPEND_ERROR = "junitsampler.append.error";
+    private static final String APPEND_EXCEPTION = "junitsampler.append.exception";
+    private static final String JUNIT4 = "junitsampler.junit4";
+    //-- JMX file attributes - do not change
 
-    public static final String SETUP = "setUp";
-    public static final String TEARDOWN = "tearDown";
-    public static final String RUNTEST = "run";
+    private static final String SETUP = "setUp";
+    private static final String TEARDOWN = "tearDown";
 
-    /// the Method objects for setUp and tearDown methods
-    private transient Method SETUP_METHOD = null;
-    private transient Method TDOWN_METHOD = null;
-    private boolean checkStartUpTearDown = false;
+    // the Method objects for setUp (@Before) and tearDown (@After) methods
+    // Will be null if not provided or not required
+    private transient Method setUpMethod;
+    private transient Method tearDownMethod;
 
-    private transient TestCase TEST_INSTANCE = null;
+    // The TestCase to run
+    private transient TestCase testCase;
+    // The test object, i.e. the instance of the class containing the test method
+    // This is the same as testCase for JUnit3 tests
+    // but different for JUnit4 tests which use a wrapper
+    private transient Object testObject;
+
+    // The method name to be invoked
+    private transient String methodName;
+    // The name of the class containing the method
+    private transient String className;
+    // The wrapper used to invoke the method
+    private transient Protectable protectable;
 
     public JUnitSampler(){
     }
 
     /**
      * Method tries to get the setUp and tearDown method for the class
-     * @param tc
+     * @param testObject
      */
-    private void initMethodObjects(TestCase tc){
-        if (!this.checkStartUpTearDown && !getDoNotSetUpTearDown()) {
-            if (SETUP_METHOD == null) {
-                SETUP_METHOD = getMethod(tc, SETUP);
-            }
-            if (TDOWN_METHOD == null) {
-                TDOWN_METHOD = getMethod(tc, TEARDOWN);
-            }
-            this.checkStartUpTearDown = true;
+    private void initMethodObjects(Object testObject){
+        setUpMethod = null;
+        tearDownMethod = null;
+        if (!getDoNotSetUpTearDown()) {
+            setUpMethod = getJunit4() ?
+                getMethodWithAnnotation(testObject, Before.class)
+                :
+                getMethod(testObject, SETUP);
+            tearDownMethod = getJunit4() ?
+                getMethodWithAnnotation(testObject, After.class)
+                :
+                getMethod(testObject, TEARDOWN);
         }
     }
 
@@ -290,6 +308,11 @@ public class JUnitSampler extends AbstractSampler {
         return getPropertyAsBoolean(APPEND_ERROR,false);
     }
 
+    /**
+     * Set whether to append errors or not.
+     *
+     * @param error the setting to apply
+     */
     public void setAppendError(boolean error) {
         setProperty(APPEND_ERROR,String.valueOf(error));
     }
@@ -303,139 +326,135 @@ public class JUnitSampler extends AbstractSampler {
         return getPropertyAsBoolean(APPEND_EXCEPTION,false);
     }
 
+    /**
+     * Set whether to append exceptions or not.
+     *
+     * @param exc the setting to apply.
+     */
     public void setAppendException(boolean exc) {
         setProperty(APPEND_EXCEPTION,String.valueOf(exc));
     }
 
-    /* (non-Javadoc)
-     * @see org.apache.jmeter.samplers.Sampler#sample(org.apache.jmeter.samplers.Entry)
+    /**
+     * Check if JUnit4 (annotations) are to be used instead of
+     * the JUnit3 style (TestClass and specific method names)
+     *
+     * @return true if JUnit4 (annotations) are to be used.
+     * Default is false.
      */
+    public boolean getJunit4() {
+        return getPropertyAsBoolean(JUNIT4, false);
+    }
+
+    /**
+     * Set whether to use JUnit4 style or not.
+     * @param junit4 true if JUnit4 style is to be used.
+     */
+    public void setJunit4(boolean junit4) {
+        setProperty(JUNIT4, junit4, false);
+    }
+
+    /** {@inheritDoc} */
     public SampleResult sample(Entry entry) {
         SampleResult sresult = new SampleResult();
-        String rlabel = getConstructorString();
-        if (rlabel.length()== 0) {
-            rlabel = JUnitSampler.class.getName();
-        }
         sresult.setSampleLabel(getName());// Bug 41522 - don't use rlabel here
-        final String methodName = getMethod();
-        final String className = getClassname();
         sresult.setSamplerData(className + "." + methodName);
-        // check to see if the test class is null. if it is, we create
-        // a new instance. this should only happen at the start of a
-        // test run
-        if (this.TEST_INSTANCE == null) {
-            this.TEST_INSTANCE = (TestCase)getClassInstance(className,rlabel);
-        }
-        if (this.TEST_INSTANCE != null){
-            initMethodObjects(this.TEST_INSTANCE);
+        sresult.setDataType(SampleResult.TEXT);
+        // Assume success
+        sresult.setSuccessful(true);
+        sresult.setResponseMessage(getSuccess());
+        sresult.setResponseCode(getSuccessCode());
+        if (this.testCase != null){
             // create a new TestResult
             TestResult tr = new TestResult();
-            this.TEST_INSTANCE.setName(methodName);
+            final TestCase theClazz = this.testCase;
             try {
-
-                if (!getDoNotSetUpTearDown() && SETUP_METHOD != null){
-                    try {
-                        SETUP_METHOD.invoke(this.TEST_INSTANCE,new Class[0]);
-                    } catch (InvocationTargetException e) {
-                        tr.addFailure(this.TEST_INSTANCE,
-                                new AssertionFailedError(e.getMessage()));
-                    } catch (IllegalAccessException e) {
-                        tr.addFailure(this.TEST_INSTANCE,
-                                new AssertionFailedError(e.getMessage()));
-                    } catch (IllegalArgumentException e) {
-                        tr.addFailure(this.TEST_INSTANCE,
-                                new AssertionFailedError(e.getMessage()));
-                    }
+                if (setUpMethod != null){
+                    setUpMethod.invoke(this.testObject,new Object[0]);
                 }
-                final Method m = getMethod(this.TEST_INSTANCE,methodName);
-                final TestCase theClazz = this.TEST_INSTANCE;
-                tr.startTest(this.TEST_INSTANCE);
                 sresult.sampleStart();
+                tr.startTest(this.testCase);
                 // Do not use TestCase.run(TestResult) method, since it will
                 // call setUp and tearDown. Doing that will result in calling
                 // the setUp and tearDown method twice and the elapsed time
                 // will include setup and teardown.
-                Protectable p = new Protectable() {
-                    public void protect() throws Throwable {
-                        m.invoke(theClazz,new Class[0]);
-                    }
-                };
-                tr.runProtected(theClazz, p);
-                tr.endTest(this.TEST_INSTANCE);
+                tr.runProtected(theClazz, protectable);
+                tr.endTest(this.testCase);
                 sresult.sampleEnd();
-
-                if (!getDoNotSetUpTearDown() && TDOWN_METHOD != null){
-                    TDOWN_METHOD.invoke(TEST_INSTANCE,new Class[0]);
+                if (tearDownMethod != null){
+                    tearDownMethod.invoke(testObject,new Object[0]);
                 }
             } catch (InvocationTargetException e) {
-                // log.warn(e.getMessage());
-                sresult.setResponseCode(getErrorCode());
-                sresult.setResponseMessage(getError());
-                sresult.setResponseData(e.getMessage().getBytes());
-                sresult.setSuccessful(false);
+                Throwable cause = e.getCause();
+                if (cause instanceof AssertionFailedError){
+                    tr.addFailure(theClazz, (AssertionFailedError) cause);
+                } else if (cause instanceof AssertionError) {
+                    // Convert JUnit4 failure to Junit3 style
+                    AssertionFailedError afe = new AssertionFailedError(cause.toString());
+                    // copy the original stack trace
+                    afe.setStackTrace(cause.getStackTrace());
+                    tr.addFailure(theClazz, afe);
+                } else if (cause != null) {
+                    tr.addError(theClazz, cause);
+                } else {
+                    tr.addError(theClazz, e);
+                }
             } catch (IllegalAccessException e) {
-                // log.warn(e.getMessage());
-                sresult.setResponseCode(getErrorCode());
-                sresult.setResponseMessage(getError());
-                sresult.setResponseData(e.getMessage().getBytes());
-                sresult.setSuccessful(false);
-            } catch (ComparisonFailure e) {
-                sresult.setResponseCode(getErrorCode());
-                sresult.setResponseMessage(getError());
-                sresult.setResponseData(e.getMessage().getBytes());
-                sresult.setSuccessful(false);
+                tr.addError(theClazz, e);
             } catch (IllegalArgumentException e) {
-                sresult.setResponseCode(getErrorCode());
-                sresult.setResponseMessage(getError());
-                sresult.setResponseData(e.getMessage().getBytes());
-                sresult.setSuccessful(false);
-            } catch (Exception e) {
-                sresult.setResponseCode(getErrorCode());
-                sresult.setResponseMessage(getError());
-                sresult.setResponseData(e.getMessage().getBytes());
-                sresult.setSuccessful(false);
-            } catch (Throwable e) {
-                sresult.setResponseCode(getErrorCode());
-                sresult.setResponseMessage(getError());
-                sresult.setResponseData(e.getMessage().getBytes());
-                sresult.setSuccessful(false);
+                tr.addError(theClazz, e);
             }
             if ( !tr.wasSuccessful() ){
                 sresult.setSuccessful(false);
-                StringBuffer buf = new StringBuffer();
-                buf.append( getFailure() );
-                Enumeration en = tr.errors();
-                while (en.hasMoreElements()){
-                    Object item = en.nextElement();
-                    if (getAppendError() && item instanceof TestFailure) {
-                        buf.append( "Trace -- ");
-                        buf.append( ((TestFailure)item).trace() );
+                StringBuilder buf = new StringBuilder();
+                StringBuilder buftrace = new StringBuilder();
+                Enumeration<TestFailure> en;
+                if (getAppendError()) {
+                    en = tr.failures();
+                    if (en.hasMoreElements()){
+                        sresult.setResponseCode(getFailureCode());
+                        buf.append( getFailure() );
+                        buf.append("\n");
+                    }
+                    while (en.hasMoreElements()){
+                        TestFailure item = en.nextElement();
                         buf.append( "Failure -- ");
-                        buf.append( ((TestFailure)item).toString() );
-                    } else if (getAppendException() && item instanceof Throwable) {
-                        buf.append( ((Throwable)item).getMessage() );
+                        buf.append( item.toString() );
+                        buf.append("\n");
+                        buftrace.append( "Failure -- ");
+                        buftrace.append( item.toString() );
+                        buftrace.append("\n");
+                        buftrace.append( "Trace -- ");
+                        buftrace.append( item.trace() );
+                    }
+                    en = tr.errors();
+                    if (en.hasMoreElements()){
+                        sresult.setResponseCode(getErrorCode());
+                        buf.append( getError() );
+                        buf.append("\n");
+                    }
+                    while (en.hasMoreElements()){
+                        TestFailure item = en.nextElement();
+                        buf.append( "Error -- ");
+                        buf.append( item.toString() );
+                        buf.append("\n");
+                        buftrace.append( "Error -- ");
+                        buftrace.append( item.toString() );
+                        buftrace.append("\n");
+                        buftrace.append( "Trace -- ");
+                        buftrace.append( item.trace() );
                     }
                 }
                 sresult.setResponseMessage(buf.toString());
-                sresult.setRequestHeaders(buf.toString());
-                sresult.setResponseCode(getFailureCode());
-            } else {
-                // this means there's no failures
-                sresult.setSuccessful(true);
-                sresult.setResponseMessage(getSuccess());
-                sresult.setResponseCode(getSuccessCode());
-                sresult.setResponseData("Not Applicable".getBytes());
+                sresult.setResponseData(buftrace.toString(), null);
             }
         } else {
             // we should log a warning, but allow the test to keep running
             sresult.setSuccessful(false);
             // this should be externalized to the properties
             sresult.setResponseMessage("failed to create an instance of the class");
+            sresult.setResponseCode(getErrorCode());
         }
-        sresult.setBytes(0);
-        sresult.setContentType("text");
-        sresult.setDataType("Not Applicable");
-        sresult.setRequestHeaders("Not Applicable");
         return sresult;
     }
 
@@ -444,12 +463,12 @@ public class JUnitSampler extends AbstractSampler {
      * class, it returns null and logs all the exceptions at
      * warning level.
      */
-    public static Object getClassInstance(String className, String label){
+    private static Object getClassInstance(String className, String label){
         Object testclass = null;
         if (className != null){
-            Constructor con = null;
-            Constructor strCon = null;
-            Class theclazz = null;
+            Constructor<?> con = null;
+            Constructor<?> strCon = null;
+            Class<?> theclazz = null;
             Object[] strParams = null;
             Object[] params = null;
             try
@@ -510,15 +529,14 @@ public class JUnitSampler extends AbstractSampler {
     }
 
     /**
-     *
-     * @param clazz
-     * @param method
+     * Get a method.
+     * @param clazz the classname (may be null)
+     * @param method the method name (may be null)
      * @return the method or null if an error occurred
+     * (or either parameter is null)
      */
-    public Method getMethod(Object clazz, String method){
+    private Method getMethod(Object clazz, String method){
         if (clazz != null && method != null){
-            // log.info("class " + clazz.getClass().getName() +
-            //        " method name is " + method);
             try {
                 return clazz.getClass().getMethod(method,new Class[0]);
             } catch (NoSuchMethodException e) {
@@ -528,17 +546,131 @@ public class JUnitSampler extends AbstractSampler {
         return null;
     }
 
-    public Method getRunTestMethod(Object clazz){
-        if (clazz != null){
-            // log.info("class " + clazz.getClass().getName() +
-            //        " method name is " + RUNTEST);
-            try {
-                Class[] param = {TestResult.class};
-                return clazz.getClass().getMethod(RUNTEST,param);
-            } catch (NoSuchMethodException e) {
-                log.warn(e.getMessage());
+    private Method getMethodWithAnnotation(Object clazz, Class<? extends Annotation> annotation) {
+        if(null != clazz && null != annotation) {
+            for(Method m : clazz.getClass().getMethods()) {
+                if(m.isAnnotationPresent(annotation)) {
+                    return m;
+                }
             }
         }
         return null;
     }
+
+    /*
+     * Wrapper to convert a JUnit4 class into a TestCase
+     *
+     *  TODO - work out how to convert JUnit4 assertions so they are treated as failures rather than errors
+     */
+    private class AnnotatedTestCase extends TestCase {
+        private final Method method;
+        private final Class<? extends Throwable> expectedException;
+        private final long timeout;
+        public AnnotatedTestCase(Method method, Class<? extends Throwable> expectedException2, long timeout) {
+            this.method = method;
+            this.expectedException = expectedException2;
+            this.timeout = timeout;
+        }
+
+        @Override
+        protected void runTest() throws Throwable {
+            try {
+                long start = System.currentTimeMillis();
+                method.invoke(testObject, (Object[])null);
+                if (expectedException != None.class) {
+                    throw new AssertionFailedError(
+                    "No error was generated for a test case which specifies an error.");
+                }
+                if (timeout > 0){
+                    long elapsed = System.currentTimeMillis() - start;
+                    if (elapsed > timeout) {
+                        throw new AssertionFailedError("Test took longer than the specified timeout.");
+                    }
+                }
+            } catch (InvocationTargetException e) {
+                Throwable thrown = e.getCause();
+                if (thrown == null) { // probably should not happen
+                    throw e;
+                }
+                if (expectedException == None.class){
+                    // Convert JUnit4 AssertionError failures to JUnit3 style so
+                    // will be treated as failure rather than error.
+                    if (thrown instanceof AssertionError && !(thrown instanceof AssertionFailedError)){
+                        AssertionFailedError afe = new AssertionFailedError(thrown.toString());
+                        // copy the original stack trace
+                        afe.setStackTrace(thrown.getStackTrace());
+                        throw afe;
+                    }
+                    throw thrown;
+                }
+                if (!expectedException.isAssignableFrom(thrown.getClass())){
+                    throw new AssertionFailedError("The wrong exception was thrown from the test case");
+                }
+            }
+        }
+    }
+
+    public void threadFinished() {
+    }
+
+    /**
+     * Set up all variables that don't change between samples.
+     */
+    public void threadStarted() {
+        testObject = null;
+        testCase = null;
+        methodName = getMethod();
+        className = getClassname();
+        final Method m;
+        protectable = null;
+        String rlabel = getConstructorString();
+        if (rlabel.length()== 0) {
+            rlabel = JUnitSampler.class.getName();
+        }
+        this.testObject = getClassInstance(className,rlabel);
+        if (this.testObject != null){
+            initMethodObjects(this.testObject);
+            m = getMethod(this.testObject,methodName);
+            if (getJunit4()){
+                Class<? extends Throwable> expectedException = None.class;
+                long timeout = 0;
+                Test annotation = m.getAnnotation(Test.class);
+                if(null != annotation) {
+                    expectedException = annotation.expected();
+                    timeout = annotation.timeout();
+                }
+                final AnnotatedTestCase at = new AnnotatedTestCase(m, expectedException, timeout);
+                testCase = at;
+                protectable = new Protectable() {
+                    public void protect() throws Throwable {
+                        at.runTest();
+                    }
+                };
+            } else {
+                this.testCase = (TestCase) this.testObject;
+                final Object theClazz = this.testObject; // Must be final to create instance
+                protectable = new Protectable() {
+                    public void protect() throws Throwable {
+                        try {
+                            m.invoke(theClazz,new Object[0]);
+                        } catch (InvocationTargetException e) {
+                            /*
+                             * Calling a method via reflection results in wrapping any
+                             * Exceptions in ITE; unwrap these here so runProtected can
+                             * allocate them correctly.
+                             */
+                            Throwable t = e.getCause();
+                            if (t != null) {
+                                throw t;
+                            }
+                            throw e;
+                        }
+                    }
+                };
+            }
+            if (this.testCase != null){
+                this.testCase.setName(methodName);
+            }
+        }
+    }
 }
