@@ -21,8 +21,8 @@ public interface Status {
 
     void setJvm(Jvm vm);
 
-    java.util.List getConnector();
-    
+    java.util.List<Connector> getConnector();
+
     void addConnector(Connector conn);
 
     void setConnectorPrefix(String prefix);
