@@ -79,7 +79,7 @@ public class CookieManager extends ConfigTestElement implements TestListener, Se
 
     private static final String COOKIE_NAME_PREFIX =
         JMeterUtils.getPropDefault("CookieManager.name.prefix", "COOKIE_").trim();// $NON-NLS-1$ $NON-NLS-2$
-    
+
     private static final boolean SAVE_COOKIES =
         JMeterUtils.getPropDefault("CookieManager.save.cookies", false);// $NON-NLS-1$
 
@@ -97,6 +97,8 @@ public class CookieManager extends ConfigTestElement implements TestListener, Se
     }
 
     // ensure that the initial cookies are copied to the per-thread instances
+    /** {@inheritDoc} */
+    @Override
     public Object clone(){
         CookieManager clone = (CookieManager) super.clone();
         clone.initialCookies = initialCookies;
@@ -213,7 +215,7 @@ public class CookieManager extends ConfigTestElement implements TestListener, Se
     }
 
     private String cookieToString(Cookie c){
-        StringBuffer sb=new StringBuffer(80);
+        StringBuilder sb=new StringBuilder(80);
         sb.append(c.getDomain());
         //flag - if all machines within a given domain can access the variable.
         //(from http://www.cookiecentral.com/faq/ 3.5)
@@ -226,10 +228,14 @@ public class CookieManager extends ConfigTestElement implements TestListener, Se
         return sb.toString();
     }
 
+    /** {@inheritDoc} */
+    @Override
     public void recoverRunningVersion() {
         // do nothing, the cookie manager has to accept changes.
     }
 
+    /** {@inheritDoc} */
+    @Override
     public void setRunningVersion(boolean running) {
         // do nothing, the cookie manager has to accept changes.
     }
@@ -260,6 +266,8 @@ public class CookieManager extends ConfigTestElement implements TestListener, Se
         }
     }
 
+    /** {@inheritDoc} */
+    @Override
     public void clear(){
         super.clear();
         clearCookies(); // ensure data is set up OK initially
@@ -270,7 +278,7 @@ public class CookieManager extends ConfigTestElement implements TestListener, Se
      */
     private void clearCookies() {
         log.debug("Clear all cookies from store");
-        setProperty(new CollectionProperty(COOKIES, new ArrayList()));
+        setProperty(new CollectionProperty(COOKIES, new ArrayList<Object>()));
     }
 
     /**
@@ -460,6 +468,7 @@ public class CookieManager extends ConfigTestElement implements TestListener, Se
         }
     }
 
+    /** {@inheritDoc} */
     public void testStarted() {
         initialCookies = getCookies();
         cookieSpec = CookiePolicy.getCookieSpec(getPolicy());
@@ -468,16 +477,20 @@ public class CookieManager extends ConfigTestElement implements TestListener, Se
         }
     }
 
+    /** {@inheritDoc} */
     public void testEnded() {
     }
 
+    /** {@inheritDoc} */
     public void testStarted(String host) {
         testStarted();
     }
 
+    /** {@inheritDoc} */
     public void testEnded(String host) {
     }
 
+    /** {@inheritDoc} */
     public void testIterationStart(LoopIterationEvent event) {
         if (getClearEachIteration()) {
             log.debug("Initialise cookies from pre-defined list");
