@@ -24,7 +24,6 @@ import java.io.IOException;
 import java.io.InputStream;
 import java.io.UnsupportedEncodingException;
 import java.net.MalformedURLException;
-import java.net.ProtocolException;
 import java.net.URL;
 import java.util.HashMap;
 import java.util.HashSet;
@@ -73,12 +72,12 @@ public class HttpRequestHdr {
                                    ".binary"); // $NON-NLS-1$
 
     /** Which content-types will be treated as binary (exact match) */
-    private static final Set binaryContentTypes = new HashSet();
+    private static final Set<String> binaryContentTypes = new HashSet<String>();
 
     /** Where to store the temporary binary files */
     private static final String binaryDirectory =
         JMeterUtils.getPropDefault("proxy.binary.directory",// $NON-NLS-1$
-                JMeterUtils.getProperty("user.dir")); // $NON-NLS-1$ proxy.binary.filetype=binary
+                System.getProperty("user.dir")); // $NON-NLS-1$ proxy.binary.filetype=binary
 
     static {
         String binaries = JMeterUtils.getPropDefault("proxy.binary.types", // $NON-NLS-1$
@@ -92,10 +91,13 @@ public class HttpRequestHdr {
     }
 
     /**
-     * Http Request method. Such as get or post.
+     * Http Request method, uppercased, e.g. GET or POST.
      */
     private String method = ""; // $NON-NLS-1$
 
+    /** CONNECT url. */
+    private String paramHttps = ""; // $NON-NLS-1$
+
     /**
      * The requested url. The universal resource locator that hopefully uniquely
      * describes the object or service the client is requesting.
@@ -109,7 +111,7 @@ public class HttpRequestHdr {
 
     private byte[] rawPostData;
 
-    private final Map headers = new HashMap();
+    private final Map<String, Header> headers = new HashMap<String, Header>();
 
     private final HTTPSamplerBase sampler;
 
@@ -182,7 +184,7 @@ public class HttpRequestHdr {
         rawPostData = line.toByteArray();
 
         if (log.isDebugEnabled()){
-            log.debug("rawPostData in default JRE encoding: " + new String(rawPostData));
+            log.debug("rawPostData in default JRE encoding: " + new String(rawPostData)); // TODO - charset?
             log.debug("Request: " + clientRequest.toString());
         }
         return clientRequest.toByteArray();
@@ -192,15 +194,9 @@ public class HttpRequestHdr {
         if (log.isDebugEnabled()) {
             log.debug("browser request: " + firstLine);
         }
-        if (!CharUtils.isAsciiAlphanumeric(firstLine.charAt(0))) {
-            throw new IllegalArgumentException("Unrecognised header line (probably used HTTPS)");
-        }
         StringTokenizer tz = new StringTokenizer(firstLine);
         method = getToken(tz).toUpperCase(java.util.Locale.ENGLISH);
         url = getToken(tz);
-        if (url.toLowerCase(java.util.Locale.ENGLISH).startsWith(HTTPConstants.PROTOCOL_HTTPS)) {
-            throw new IllegalArgumentException("Cannot handle https URLS: " + url);
-        }
         version = getToken(tz);
         if (log.isDebugEnabled()) {
             log.debug("parser input:  " + firstLine);
@@ -208,9 +204,14 @@ public class HttpRequestHdr {
             log.debug("parsed url:    " + url);
             log.debug("parsed version:" + version);
         }
-        if ("CONNECT".equalsIgnoreCase(method)){
-            throw new IllegalArgumentException("Cannot handle CONNECT - probably used HTTPS");
+        // SSL connection
+        if (getMethod().startsWith(HTTPConstants.CONNECT)) {
+            paramHttps = url;
         }
+        if (url.startsWith("/")) {
+            url = HTTPS + "://" + paramHttps + url; // $NON-NLS-1$
+        }
+        log.debug("First Line: " + url);
     }
 
     /*
@@ -233,11 +234,13 @@ public class HttpRequestHdr {
 
     private HeaderManager createHeaderManager() {
         HeaderManager manager = new HeaderManager();
-        Iterator keys = headers.keySet().iterator();
+        Iterator<String> keys = headers.keySet().iterator();
         while (keys.hasNext()) {
-            String key = (String) keys.next();
-            if (!key.equals(PROXY_CONNECTION) && !key.equals(CONTENT_LENGTH)) {
-                manager.add((Header) headers.get(key));
+            String key = keys.next();
+            if (!key.equals(PROXY_CONNECTION)
+             && !key.equals(CONTENT_LENGTH)
+             && !key.equalsIgnoreCase(HTTPConstants.HEADER_CONNECTION)) {
+                manager.add(headers.get(key));
             }
         }
         manager.setName(JMeterUtils.getResString("header_manager_title")); // $NON-NLS-1$
@@ -253,7 +256,7 @@ public class HttpRequestHdr {
         return headerManager;
     }
 
-    public HTTPSamplerBase getSampler(Map pageEncodings, Map formEncodings)
+    public HTTPSamplerBase getSampler(Map<String, String> pageEncodings, Map<String, String> formEncodings)
             throws MalformedURLException, IOException {
         // Damn! A whole new GUI just to instantiate a test element?
         // Isn't there a beter way?
@@ -282,21 +285,8 @@ public class HttpRequestHdr {
         return sampler;
     }
 
-    /**
-     *
-     * @return the sampler
-     * @throws MalformedURLException
-     * @throws IOException
-     * @throws ProtocolException
-     * @deprecated use the getSampler(HashMap pageEncodings, HashMap formEncodings) instead, since
-     * that properly handles the encodings of the page
-     */
-    public HTTPSamplerBase getSampler() throws MalformedURLException, IOException, ProtocolException {
-        return getSampler(null, null);
-    }
-
     private String getContentType() {
-        Header contentTypeHeader = (Header) headers.get(CONTENT_TYPE);
+        Header contentTypeHeader = headers.get(CONTENT_TYPE);
         if (contentTypeHeader != null) {
             return contentTypeHeader.getValue();
         }
@@ -319,7 +309,7 @@ public class HttpRequestHdr {
         return null;
     }
 
-    private void populateSampler(Map pageEncodings, Map formEncodings)
+    private void populateSampler(Map<String, String> pageEncodings, Map<String, String> formEncodings)
             throws MalformedURLException, UnsupportedEncodingException {
         sampler.setDomain(serverName());
         if (log.isDebugEnabled()) {
@@ -369,13 +359,13 @@ public class HttpRequestHdr {
             // Check if we know the encoding of the page
             if (pageEncodings != null) {
                 synchronized (pageEncodings) {
-                    contentEncoding = (String) pageEncodings.get(urlWithoutQuery);
+                    contentEncoding = pageEncodings.get(urlWithoutQuery);
                 }
             }
             // Check if we know the encoding of the form
             if (formEncodings != null) {
                 synchronized (formEncodings) {
-                    String formEncoding = (String) formEncodings.get(urlWithoutQuery);
+                    String formEncoding = formEncodings.get(urlWithoutQuery);
                     // Form encoding has priority over page encoding
                     if (formEncoding != null) {
                         contentEncoding = formEncoding;
@@ -414,7 +404,7 @@ public class HttpRequestHdr {
         if (log.isDebugEnabled()) {
             log.debug("Proxy: setting path: " + sampler.getPath());
         }
-        if (numberRequests) {
+        if (!HTTPConstants.CONNECT.equals(getMethod()) && numberRequests) {
             requestNumber++;
             sampler.setName(requestNumber + " " + sampler.getPath());
         } else {
@@ -429,7 +419,7 @@ public class HttpRequestHdr {
         // If it was a HTTP GET request, then all parameters in the URL
         // has been handled by the sampler.setPath above, so we just need
         // to do parse the rest of the request if it is not a GET request
-        if(!HTTPConstants.GET.equals(method)) {
+        if((!HTTPConstants.CONNECT.equals(getMethod())) && (!HTTPConstants.GET.equals(method))) {
             // Check if it was a multipart http post request
             final String contentType = getContentType();
             MultipartUrlConfig urlConfig = getMultipartConfig(contentType);
@@ -568,6 +558,15 @@ public class HttpRequestHdr {
         return url;
     }
 
+    /**
+     * Returns the method string extracted from the first line of the client request.
+     *
+     * @return the method (will always be upper case)
+     */
+    public String getMethod(){
+        return method;
+    }
+
     /**
      * Returns the next token in a string.
      *
@@ -590,7 +589,7 @@ public class HttpRequestHdr {
 //     * @return The remainder
 //     */
 //    private String getRemainder(StringTokenizer tk) {
-//        StringBuffer strBuff = new StringBuffer();
+//        StringBuilder strBuff = new StringBuilder();
 //        if (tk.hasMoreTokens()) {
 //            strBuff.append(tk.nextToken());
 //        }
