@@ -33,7 +33,9 @@ import org.apache.jmeter.config.Arguments;
 import org.apache.jmeter.config.ConfigElement;
 import org.apache.jmeter.config.ConfigTestElement;
 import org.apache.jmeter.control.GenericController;
+import org.apache.jmeter.control.TransactionController;
 import org.apache.jmeter.control.gui.LogicControllerGui;
+import org.apache.jmeter.control.gui.TransactionControllerGui;
 import org.apache.jmeter.engine.util.ValueReplacer;
 import org.apache.jmeter.exceptions.IllegalUserActionException;
 import org.apache.jmeter.functions.InvalidVariableException;
@@ -57,7 +59,7 @@ import org.apache.jmeter.testelement.property.IntegerProperty;
 import org.apache.jmeter.testelement.property.JMeterProperty;
 import org.apache.jmeter.testelement.property.PropertyIterator;
 import org.apache.jmeter.testelement.property.StringProperty;
-import org.apache.jmeter.threads.ThreadGroup;
+import org.apache.jmeter.threads.AbstractThreadGroup;
 import org.apache.jmeter.timers.Timer;
 import org.apache.jmeter.util.JMeterUtils;
 import org.apache.jorphan.logging.LoggingManager;
@@ -75,10 +77,13 @@ public class ProxyControl extends GenericController {
 
     private static final Logger log = LoggingManager.getLoggerForClass();
 
-    private static final long serialVersionUID = 233L;
+    private static final long serialVersionUID = 240L;
 
     private static final String ASSERTION_GUI = AssertionGui.class.getName();
 
+
+    private static final String TRANSACTION_CONTROLLER_GUI = TransactionControllerGui.class.getName();
+
     private static final String LOGIC_CONTROLLER_GUI = LogicControllerGui.class.getName();
 
     private static final String HEADER_PANEL = HeaderPanel.class.getName();
@@ -125,13 +130,13 @@ public class ProxyControl extends GenericController {
     private static final String CONTENT_TYPE_INCLUDE = "ProxyControlGui.content_type_include"; // $NON-NLS-1$
     //- JMX file attributes
 
-    public static final int GROUPING_NO_GROUPS = 0;
-
-    public static final int GROUPING_ADD_SEPARATORS = 1;
-
-    public static final int GROUPING_IN_CONTROLLERS = 2;
-
-    public static final int GROUPING_STORE_FIRST_ONLY = 3;
+    // Must agree with the order of entries in the drop-down
+    // created in ProxyControlGui.createGroupingPanel()
+    //private static final int GROUPING_NO_GROUPS = 0;
+    private static final int GROUPING_ADD_SEPARATORS = 1;
+    private static final int GROUPING_IN_SIMPLE_CONTROLLERS = 2;
+    private static final int GROUPING_STORE_FIRST_ONLY = 3;
+    private static final int GROUPING_IN_TRANSACTION_CONTROLLERS = 4;
 
     // Must agree with the order of entries in the drop-down
     // created in ProxyControlGui.createHTTPSamplerPanel()
@@ -167,8 +172,8 @@ public class ProxyControl extends GenericController {
 
     public ProxyControl() {
         setPort(DEFAULT_PORT);
-        setExcludeList(new HashSet());
-        setIncludeList(new HashSet());
+        setExcludeList(new HashSet<String>());
+        setIncludeList(new HashSet<String>());
         setCaptureHttpHeaders(true); // maintain original behaviour
     }
 
@@ -221,12 +226,12 @@ public class ProxyControl extends GenericController {
         setProperty(new BooleanProperty(SAMPLER_DOWNLOAD_IMAGES, b));
     }
 
-    public void setIncludeList(Collection list) {
-        setProperty(new CollectionProperty(INCLUDE_LIST, new HashSet(list)));
+    public void setIncludeList(Collection<String> list) {
+        setProperty(new CollectionProperty(INCLUDE_LIST, new HashSet<String>(list)));
     }
 
-    public void setExcludeList(Collection list) {
-        setProperty(new CollectionProperty(EXCLUDE_LIST, new HashSet(list)));
+    public void setExcludeList(Collection<String> list) {
+        setProperty(new CollectionProperty(EXCLUDE_LIST, new HashSet<String>(list)));
     }
 
     /**
@@ -317,11 +322,6 @@ public class ProxyControl extends GenericController {
         return getPropertyAsString(CONTENT_TYPE_INCLUDE);
     }
 
-
-    public Class getGuiClass() {
-        return org.apache.jmeter.protocol.http.proxy.gui.ProxyControlGui.class;
-    }
-
     public void addConfigElement(ConfigElement config) {
     }
 
@@ -383,8 +383,10 @@ public class ProxyControl extends GenericController {
     public synchronized void deliverSampler(HTTPSamplerBase sampler, TestElement[] subConfigs, SampleResult result) {
         if (filterContentType(result) && filterUrl(sampler)) {
             JMeterTreeNode myTarget = findTargetControllerNode();
-            Collection defaultConfigurations = findApplicableElements(myTarget, ConfigTestElement.class, false);
-            Collection userDefinedVariables = findApplicableElements(myTarget, Arguments.class, true);
+            @SuppressWarnings("unchecked") // OK, because find only returns correct element types
+            Collection<ConfigTestElement> defaultConfigurations = (Collection<ConfigTestElement>) findApplicableElements(myTarget, ConfigTestElement.class, false);
+            @SuppressWarnings("unchecked") // OK, because find only returns correct element types
+            Collection<Arguments> userDefinedVariables = (Collection<Arguments>) findApplicableElements(myTarget, Arguments.class, true);
 
             removeValuesFromSampler(sampler, defaultConfigurations);
             replaceValues(sampler, subConfigs, userDefinedVariables);
@@ -549,6 +551,23 @@ public class ProxyControl extends GenericController {
         model.addComponent(sc, node);
     }
 
+    /**
+     * Helper method to add a Transaction Controller to contain the samplers.
+     *
+     * @param model
+     *            Test component tree model
+     * @param node
+     *            Node in the tree where we will add the Controller
+     * @param name
+     *            A name for the Controller
+     */
+    private void addTransactionController(JMeterTreeModel model, JMeterTreeNode node, String name)
+            throws IllegalUserActionException {
+        TransactionController sc = new TransactionController();
+        sc.setProperty(TestElement.GUI_CLASS, TRANSACTION_CONTROLLER_GUI);
+        sc.setName(name);
+        model.addComponent(sc, node);
+    }
     /**
      * Helpler method to replicate any timers found within the Proxy Controller
      * into the provided sampler, while replacing any occurences of string _T_
@@ -566,9 +585,9 @@ public class ProxyControl extends GenericController {
         variables.addParameter("T", Long.toString(deltaT)); // $NON-NLS-1$
         ValueReplacer replacer = new ValueReplacer(variables);
         JMeterTreeNode mySelf = model.getNodeOf(this);
-        Enumeration children = mySelf.children();
+        Enumeration<JMeterTreeNode> children = mySelf.children();
         while (children.hasMoreElements()) {
-            JMeterTreeNode templateNode = (JMeterTreeNode) children.nextElement();
+            JMeterTreeNode templateNode = children.nextElement();
             if (templateNode.isEnabled()) {
                 TestElement template = templateNode.getTestElement();
                 if (template instanceof Timer) {
@@ -601,12 +620,12 @@ public class ProxyControl extends GenericController {
      * @return the first node of the given type in the test component tree, or
      *         <code>null</code> if none was found.
      */
-    private JMeterTreeNode findFirstNodeOfType(Class type) {
+    private JMeterTreeNode findFirstNodeOfType(Class<?> type) {
         JMeterTreeModel treeModel = GuiPackage.getInstance().getTreeModel();
-        List nodes = treeModel.getNodesOfType(type);
-        Iterator iter = nodes.iterator();
+        List<JMeterTreeNode> nodes = treeModel.getNodesOfType(type);
+        Iterator<JMeterTreeNode> iter = nodes.iterator();
         while (iter.hasNext()) {
-            JMeterTreeNode node = (JMeterTreeNode) iter.next();
+            JMeterTreeNode node = iter.next();
             if (node.isEnabled()) {
                 return node;
             }
@@ -619,7 +638,7 @@ public class ProxyControl extends GenericController {
      * <ul>
      * <li>The controller specified by the <code>target</code> property.
      * <li>If none was specified, the first RecordingController in the tree.
-     * <li>If none is found, the first ThreadGroup in the tree.
+     * <li>If none is found, the first AbstractThreadGroup in the tree.
      * <li>If none is found, the Workspace.
      * </ul>
      *
@@ -635,7 +654,7 @@ public class ProxyControl extends GenericController {
         if (myTarget != null) {
             return myTarget;
         }
-        myTarget = findFirstNodeOfType(ThreadGroup.class);
+        myTarget = findFirstNodeOfType(AbstractThreadGroup.class);
         if (myTarget != null) {
             return myTarget;
         }
@@ -668,12 +687,13 @@ public class ProxyControl extends GenericController {
      *
      * @return a collection of applicable objects of the given class.
      */
-    private Collection findApplicableElements(JMeterTreeNode myTarget, Class myClass, boolean ascending) {
+    // TODO - could be converted to generic class?
+    private Collection<?> findApplicableElements(JMeterTreeNode myTarget, Class<? extends TestElement> myClass, boolean ascending) {
         JMeterTreeModel treeModel = GuiPackage.getInstance().getTreeModel();
-        LinkedList elements = new LinkedList();
+        LinkedList<TestElement> elements = new LinkedList<TestElement>();
 
         // Look for elements directly within the HTTP proxy:
-        Enumeration kids = treeModel.getNodeOf(this).children();
+        Enumeration<?> kids = treeModel.getNodeOf(this).children();
         while (kids.hasMoreElements()) {
             JMeterTreeNode subNode = (JMeterTreeNode) kids.nextElement();
             if (subNode.isEnabled()) {
@@ -735,9 +755,12 @@ public class ProxyControl extends GenericController {
                 if (!myTarget.isLeaf() && groupingMode == GROUPING_ADD_SEPARATORS) {
                     addDivider(treeModel, myTarget);
                 }
-                if (groupingMode == GROUPING_IN_CONTROLLERS) {
+                if (groupingMode == GROUPING_IN_SIMPLE_CONTROLLERS) {
                     addSimpleController(treeModel, myTarget, sampler.getName());
                 }
+                if (groupingMode == GROUPING_IN_TRANSACTION_CONTROLLERS) {
+                    addTransactionController(treeModel, myTarget, sampler.getName());
+                }
                 firstInBatch = true;// Remember this was first in its batch
             }
             if (lastTime == 0) {
@@ -756,7 +779,8 @@ public class ProxyControl extends GenericController {
                 sampler.setImageParser(true);
             }
 
-            if (groupingMode == GROUPING_IN_CONTROLLERS) {
+            if (groupingMode == GROUPING_IN_SIMPLE_CONTROLLERS ||
+                groupingMode == GROUPING_IN_TRANSACTION_CONTROLLERS) {
                 // Find the last controller in the target to store the
                 // sampler there:
                 for (int i = myTarget.getChildCount() - 1; i >= 0; i--) {
@@ -799,7 +823,7 @@ public class ProxyControl extends GenericController {
      * @param configurations
      *            ConfigTestElements in descending priority.
      */
-    private void removeValuesFromSampler(HTTPSamplerBase sampler, Collection configurations) {
+    private void removeValuesFromSampler(HTTPSamplerBase sampler, Collection<ConfigTestElement> configurations) {
         for (PropertyIterator props = sampler.propertyIterator(); props.hasNext();) {
             JMeterProperty prop = props.next();
             String name = prop.getName();
@@ -811,8 +835,8 @@ public class ProxyControl extends GenericController {
                 continue; // go on with next property.
             }
 
-            for (Iterator configs = configurations.iterator(); configs.hasNext();) {
-                ConfigTestElement config = (ConfigTestElement) configs.next();
+            for (Iterator<ConfigTestElement> configs = configurations.iterator(); configs.hasNext();) {
+                ConfigTestElement config = configs.next();
 
                 String configValue = config.getPropertyAsString(name);
 
@@ -830,7 +854,7 @@ public class ProxyControl extends GenericController {
     }
 
     private String generateMatchUrl(HTTPSamplerBase sampler) {
-        StringBuffer buf = new StringBuffer(sampler.getDomain());
+        StringBuilder buf = new StringBuilder(sampler.getDomain());
         buf.append(':'); // $NON-NLS-1$
         buf.append(sampler.getPort());
         buf.append(sampler.getPath());
@@ -870,12 +894,12 @@ public class ProxyControl extends GenericController {
      *            Collection of Arguments to use to do the replacement, ordered
      *            by ascending priority.
      */
-    private void replaceValues(TestElement sampler, TestElement[] configs, Collection variables) {
+    private void replaceValues(TestElement sampler, TestElement[] configs, Collection<Arguments> variables) {
         // Build the replacer from all the variables in the collection:
         ValueReplacer replacer = new ValueReplacer();
-        for (Iterator vars = variables.iterator(); vars.hasNext();) {
-            final Map map = ((Arguments) vars.next()).getArgumentsAsMap();
-            for (Iterator vals = map.values().iterator(); vals.hasNext();){
+        for (Iterator<Arguments> vars = variables.iterator(); vars.hasNext();) {
+            final Map<String, String> map = vars.next().getArgumentsAsMap();
+            for (Iterator<String> vals = map.values().iterator(); vals.hasNext();){
                final Object next = vals.next();
                if ("".equals(next)) {// Drop any empty values (Bug 45199)
                    vals.remove();
@@ -908,9 +932,9 @@ public class ProxyControl extends GenericController {
     private void notifySampleListeners(SampleEvent event) {
         JMeterTreeModel treeModel = GuiPackage.getInstance().getTreeModel();
         JMeterTreeNode myNode = treeModel.getNodeOf(this);
-        Enumeration kids = myNode.children();
+        Enumeration<JMeterTreeNode> kids = myNode.children();
         while (kids.hasMoreElements()) {
-            JMeterTreeNode subNode = (JMeterTreeNode) kids.nextElement();
+            JMeterTreeNode subNode = kids.nextElement();
             if (subNode.isEnabled()) {
                 TestElement testElement = subNode.getTestElement();
                 if (testElement instanceof SampleListener) {
@@ -927,9 +951,9 @@ public class ProxyControl extends GenericController {
     private void notifyTestListenersOfStart() {
         JMeterTreeModel treeModel = GuiPackage.getInstance().getTreeModel();
         JMeterTreeNode myNode = treeModel.getNodeOf(this);
-        Enumeration kids = myNode.children();
+        Enumeration<JMeterTreeNode> kids = myNode.children();
         while (kids.hasMoreElements()) {
-            JMeterTreeNode subNode = (JMeterTreeNode) kids.nextElement();
+            JMeterTreeNode subNode = kids.nextElement();
             if (subNode.isEnabled()) {
                 TestElement testElement = subNode.getTestElement();
                 if (testElement instanceof TestListener) {
@@ -946,9 +970,9 @@ public class ProxyControl extends GenericController {
     private void notifyTestListenersOfEnd() {
         JMeterTreeModel treeModel = GuiPackage.getInstance().getTreeModel();
         JMeterTreeNode myNode = treeModel.getNodeOf(this);
-        Enumeration kids = myNode.children();
+        Enumeration<JMeterTreeNode> kids = myNode.children();
         while (kids.hasMoreElements()) {
-            JMeterTreeNode subNode = (JMeterTreeNode) kids.nextElement();
+            JMeterTreeNode subNode = kids.nextElement();
             if (subNode.isEnabled()) {
                 TestElement testElement = subNode.getTestElement();
                 if (testElement instanceof TestListener) {
@@ -958,6 +982,7 @@ public class ProxyControl extends GenericController {
         }
     }
 
+    @Override
     public boolean canRemove() {
         return null == server;
     }
