@@ -130,6 +130,9 @@ public class ProxyControlGui extends LogicControllerGui implements JMeterGUIComp
      */
     private JCheckBox httpsSpoof;
 
+    /*
+     * Only spoof the URLs that match (optional)
+     */
     private JTextField httpsMatch;
 
     /**
@@ -170,6 +173,7 @@ public class ProxyControlGui extends LogicControllerGui implements JMeterGUIComp
 
     private static final String RESTART = "restart"; // $NON-NLS-1$
 
+    // This is applied to fields that should cause a restart when changed
     private static final String ENABLE_RESTART = "enable_restart"; // $NON-NLS-1$
 
     private static final String ADD_INCLUDE = "add_include"; // $NON-NLS-1$
@@ -195,6 +199,8 @@ public class ProxyControlGui extends LogicControllerGui implements JMeterGUIComp
         init();
     }
 
+    /** {@inheritDoc} */
+    @Override
     public TestElement createTestElement() {
         model = makeProxyControl();
         log.debug("creating/configuring model = " + model);
@@ -207,11 +213,8 @@ public class ProxyControlGui extends LogicControllerGui implements JMeterGUIComp
         return local;
     }
 
-    /**
-     * Modifies a given TestElement to mirror the data in the gui components.
-     *
-     * @see org.apache.jmeter.gui.JMeterGUIComponent#modifyTestElement(TestElement)
-     */
+    /** {@inheritDoc} */
+    @Override
     public void modifyTestElement(TestElement el) {
         if (excludeTable.isEditing()) {// Bug 42948
             excludeTable.getCellEditor().stopCellEditing();
@@ -248,32 +251,38 @@ public class ProxyControlGui extends LogicControllerGui implements JMeterGUIComp
     }
 
     protected void setIncludeListInProxyControl(ProxyControl element) {
-        List includeList = getDataList(includeModel, INCLUDE_COL);
+        List<String> includeList = getDataList(includeModel, INCLUDE_COL);
         element.setIncludeList(includeList);
     }
 
     protected void setExcludeListInProxyControl(ProxyControl element) {
-        List excludeList = getDataList(excludeModel, EXCLUDE_COL);
+        List<String> excludeList = getDataList(excludeModel, EXCLUDE_COL);
         element.setExcludeList(excludeList);
     }
 
-    private List getDataList(PowerTableModel p_model, String colName) {
+    private List<String> getDataList(PowerTableModel p_model, String colName) {
         String[] dataArray = p_model.getData().getColumn(colName);
-        List list = new LinkedList();
+        List<String> list = new LinkedList<String>();
         for (int i = 0; i < dataArray.length; i++) {
             list.add(dataArray[i]);
         }
         return list;
     }
 
+    /** {@inheritDoc} */
+    @Override
     public String getLabelResource() {
         return "proxy_title"; // $NON-NLS-1$
     }
 
-    public Collection getMenuCategories() {
+    /** {@inheritDoc} */
+    @Override
+    public Collection<String> getMenuCategories() {
         return Arrays.asList(new String[] { MenuFactory.NON_TEST_ELEMENTS });
     }
 
+    /** {@inheritDoc} */
+    @Override
     public void configure(TestElement element) {
         log.debug("Configuring gui with " + element);
         super.configure(element);
@@ -290,6 +299,7 @@ public class ProxyControlGui extends LogicControllerGui implements JMeterGUIComp
         regexMatch.setSelected(model.getRegexMatch());
         httpsSpoof.setSelected(model.getHttpsSpoof());
         httpsMatch.setText(model.getHttpsSpoofMatch());
+        httpsMatch.setEnabled(httpsSpoof.isSelected()); // Only valid if Spoof is selected
         contentTypeInclude.setText(model.getContentTypeInclude());
         contentTypeExclude.setText(model.getContentTypeExclude());
 
@@ -314,17 +324,13 @@ public class ProxyControlGui extends LogicControllerGui implements JMeterGUIComp
      * activated whenever the Proxy is selected in the Test Plan
      * Also handles samplerTypeName
      */
+    /** {@inheritDoc} */
     public void itemStateChanged(ItemEvent e) {
         // System.err.println(e.paramString());
         enableRestart();
     }
 
-    /***************************************************************************
-     * !ToDo (Method description)
-     *
-     * @param action
-     *            !ToDo (Parameter description)
-     **************************************************************************/
+    /** {@inheritDoc} */
     public void actionPerformed(ActionEvent action) {
         String command = action.getActionCommand();
 
@@ -343,6 +349,7 @@ public class ProxyControlGui extends LogicControllerGui implements JMeterGUIComp
             startProxy();
         } else if (command.equals(ENABLE_RESTART)){
             enableRestart();
+            httpsMatch.setEnabled(httpsSpoof.isSelected()); // Only valid if Spoof is selected
         } else if (command.equals(ADD_EXCLUDE)) {
             excludeModel.addNewRow();
             excludeModel.fireTableDataChanged();
@@ -402,30 +409,15 @@ public class ProxyControlGui extends LogicControllerGui implements JMeterGUIComp
         }
     }
 
-    /***************************************************************************
-     * !ToDo (Method description)
-     *
-     * @param e
-     *            !ToDo (Parameter description)
-     **************************************************************************/
+    /** {@inheritDoc} */
     public void keyPressed(KeyEvent e) {
     }
 
-    /***************************************************************************
-     * !ToDo (Method description)
-     *
-     * @param e
-     *            !ToDo (Parameter description)
-     **************************************************************************/
+    /** {@inheritDoc} */
     public void keyTyped(KeyEvent e) {
     }
 
-    /***************************************************************************
-     * !ToDo (Method description)
-     *
-     * @param e
-     *            !ToDo (Parameter description)
-     **************************************************************************/
+    /** {@inheritDoc} */
     public void keyReleased(KeyEvent e) {
         String fieldName = e.getComponent().getName();
 
@@ -515,6 +507,7 @@ public class ProxyControlGui extends LogicControllerGui implements JMeterGUIComp
         httpsMatch = new JTextField(40);
         httpsMatch.addKeyListener(this);
         httpsMatch.setName(ENABLE_RESTART);
+        httpsMatch.setEnabled(false); // Only valid if Spoof is selected
 
         JLabel matchlabel = new JLabel(JMeterUtils.getResString("proxy_httpsspoofing_match")); // $NON-NLS-1$
         matchlabel.setLabelFor(httpsMatch);
@@ -635,6 +628,7 @@ public class ProxyControlGui extends LogicControllerGui implements JMeterGUIComp
         m.addElement(JMeterUtils.getResString("grouping_add_separators")); // $NON-NLS-1$
         m.addElement(JMeterUtils.getResString("grouping_in_controllers")); // $NON-NLS-1$
         m.addElement(JMeterUtils.getResString("grouping_store_first_only")); // $NON-NLS-1$
+        m.addElement(JMeterUtils.getResString("grouping_in_transaction_controllers")); // $NON-NLS-1$
         groupingMode = new JComboBox(m);
         groupingMode.setSelectedIndex(0);
         groupingMode.addItemListener(this);
@@ -762,7 +756,7 @@ public class ProxyControlGui extends LogicControllerGui implements JMeterGUIComp
         String seperator = " > ";
         if (node != null) {
             for (int i = 0; i < node.getChildCount(); i++) {
-                StringBuffer name = new StringBuffer();
+                StringBuilder name = new StringBuilder();
                 JMeterTreeNode cur = (JMeterTreeNode) node.getChildAt(i);
                 TestElement te = cur.getTestElement();
                 /*
@@ -780,7 +774,7 @@ public class ProxyControlGui extends LogicControllerGui implements JMeterGUIComp
                     name.append(cur.getName());
                     TreeNodeWrapper tnw = new TreeNodeWrapper(cur, name.toString());
                     targetNodesModel.addElement(tnw);
-                    name = new StringBuffer();
+                    name = new StringBuilder();
                     name.append(cur.getName());
                     name.append(seperator);
                     buildNodesModel(cur, name.toString(), level + 1);
@@ -796,16 +790,13 @@ public class ProxyControlGui extends LogicControllerGui implements JMeterGUIComp
 
     private String spaces(int level) {
         int multi = 4;
-        StringBuffer spaces = new StringBuffer(level * multi);
+        StringBuilder spaces = new StringBuilder(level * multi);
         for (int i = 0; i < level * multi; i++) {
             spaces.append(" "); // $NON-NLS-1$
         }
         return spaces.toString();
     }
 
-    public void setNode(JMeterTreeNode node) {
-        getNamePanel().setNode(node);
-    }
 }
 
 class TreeNodeWrapper {
@@ -822,6 +813,8 @@ class TreeNodeWrapper {
         return tn;
     }
 
+    /** {@inheritDoc} */
+    @Override
     public String toString() {
         return label;
     }
