@@ -21,10 +21,10 @@ import java.io.FileNotFoundException;
 import java.io.IOException;
 import java.io.InputStream;
 
-import java.lang.reflect.InvocationTargetException;
-import java.lang.reflect.Method;
 import java.net.BindException;
 import java.net.HttpURLConnection;
+import java.net.InetSocketAddress;
+import java.net.Proxy;
 import java.net.URL;
 import java.net.URLConnection;
 
@@ -59,6 +59,9 @@ import org.apache.log.Logger;
  *
  */
 public class HTTPSampler extends HTTPSamplerBase implements Interruptible {
+    private static final boolean OBEY_CONTENT_LENGTH =
+        JMeterUtils.getPropDefault("httpsampler.obey_contentlength", false); // $NON-NLS-1$
+
     private static final long serialVersionUID = 233L;
 
     private static final Logger log = LoggingManager.getLoggerForClass();
@@ -67,49 +70,9 @@ public class HTTPSampler extends HTTPSamplerBase implements Interruptible {
         JMeterUtils.getPropDefault("http.java.sampler.retries" // $NON-NLS-1$
                 ,10); // Maximum connection retries
 
-    // TODO - these can be removed when JMeter moves up to Java 1.5+
-    private static final Method setReadTimeout;
-    
-    private static final Method setConnectTimeout;
-    
     static {
         log.info("Maximum connection retries = "+MAX_CONN_RETRIES); // $NON-NLS-1$
         // Temporary copies, so can set the final ones
-        Method _setConnectTimeout = null, _setReadTimeout = null;
-        Class clazz = URLConnection.class;
-        try {
-            _setConnectTimeout = clazz.getMethod("setConnectTimeout", //$NON-NLS-1$
-                    new Class[] { Integer.TYPE });
-            _setReadTimeout = clazz.getMethod("setReadTimeout", //$NON-NLS-1$
-                    new Class[] { Integer.TYPE });
-        } catch (SecurityException e) {
-            log.warn("Error trying to find timeout methods: "+e);
-        } catch (NoSuchMethodException e) {
-        } finally {
-            if (_setReadTimeout != null) {
-                log.info("Connection and read timeouts are available on this JVM");
-            } else {
-                log.info("Connection and read timeouts are *not* available on this JVM");
-            }
-            setReadTimeout = _setReadTimeout;
-            setConnectTimeout = _setConnectTimeout;
-        }
-    }
-
-    private static void setTimeout(Method method, URLConnection conn ,int timeout){
-        if (method == null){
-            log.warn("Connect/Read Timeout is not supported on this JVM");
-            return;
-        }
-        try {
-            method.invoke(conn, new Integer[]{new Integer(timeout)});
-        } catch (IllegalArgumentException e1) {
-            log.warn("Failed to set timeout: "+e1);
-        } catch (IllegalAccessException e1) {
-            log.warn("Failed to set timeout: "+e1);
-        } catch (InvocationTargetException e1) {
-            log.warn("Failed to set timeout: "+e1);
-        }        
     }
 
     private static final byte[] NULL_BA = new byte[0];// can share these
@@ -190,20 +153,35 @@ public class HTTPSampler extends HTTPSamplerBase implements Interruptible {
             }
         }
 
-        HttpURLConnection conn = (HttpURLConnection) u.openConnection();
+        final HttpURLConnection conn;
+        final String proxyHost = getProxyHost();
+        final int proxyPort = getProxyPortInt();
+        if (proxyHost.length() > 0 && proxyPort > 0){
+            Proxy proxy = new Proxy(Proxy.Type.HTTP, new InetSocketAddress(proxyHost, proxyPort));
+            //TODO - how to define proxy authentication for a single connection?
+            // It's not clear if this is possible
+//            String user = getProxyUser();
+//            if (user.length() > 0){
+//                Authenticator auth = new ProxyAuthenticator(user, getProxyPass());
+//            }
+            conn = (HttpURLConnection) u.openConnection(proxy);
+        } else {
+            conn = (HttpURLConnection) u.openConnection();
+        }
+
         // Update follow redirects setting just for this connection
         conn.setInstanceFollowRedirects(getAutoRedirects());
 
         int cto = getConnectTimeout();
         if (cto > 0){
-            setTimeout(setConnectTimeout, conn, cto);
+            conn.setConnectTimeout(cto);
         }
-        
+
         int rto = getResponseTimeout();
         if (rto > 0){
-            setTimeout(setReadTimeout, conn, rto);
+            conn.setReadTimeout(rto);
         }
-        
+
         if (PROTOCOL_HTTPS.equalsIgnoreCase(u.getProtocol())) {
             try {
                 if (null != sslmgr){
@@ -237,8 +215,6 @@ public class HTTPSampler extends HTTPSamplerBase implements Interruptible {
         }
 
         if (res != null) {
-            res.setURL(u);
-            res.setHTTPMethod(method);
             res.setRequestHeaders(getConnectionHeaders(conn));
             res.setCookies(cookies);
         }
@@ -260,10 +236,10 @@ public class HTTPSampler extends HTTPSamplerBase implements Interruptible {
 
         final int contentLength = conn.getContentLength();
         if ((contentLength == 0)
-            && JMeterUtils.getPropDefault("httpsampler.obey_contentlength", // $NON-NLS-1$
-            false)) {
+            && OBEY_CONTENT_LENGTH) {
             log.info("Content-Length: 0, not reading http-body");
             res.setResponseHeaders(getResponseHeaders(conn));
+            res.latencyEnd();
             return NULL_BA;
         }
 
@@ -290,6 +266,7 @@ public class HTTPSampler extends HTTPSamplerBase implements Interruptible {
             if (errorStream == null) {
                 log.info("Error Response Code: "+conn.getResponseCode()+", Server sent no Errorpage");
                 res.setResponseHeaders(getResponseHeaders(conn));
+                res.latencyEnd();
                 return NULL_BA;
             }
 
@@ -319,7 +296,7 @@ public class HTTPSampler extends HTTPSamplerBase implements Interruptible {
      * @return string containing the headers, one per line
      */
     protected String getResponseHeaders(HttpURLConnection conn) {
-        StringBuffer headerBuf = new StringBuffer();
+        StringBuilder headerBuf = new StringBuilder();
         headerBuf.append(conn.getHeaderField(0));// Leave header as is
         // headerBuf.append(conn.getHeaderField(0).substring(0, 8));
         // headerBuf.append(" ");
@@ -405,19 +382,19 @@ public class HTTPSampler extends HTTPSamplerBase implements Interruptible {
      */
     private String getConnectionHeaders(HttpURLConnection conn) {
         // Get all the request properties, which are the headers set on the connection
-        StringBuffer hdrs = new StringBuffer(100);
-        Map requestHeaders = conn.getRequestProperties();
-        Set headerFields = requestHeaders.entrySet();
-        for(Iterator i = headerFields.iterator(); i.hasNext();) {
-            Map.Entry entry = (Map.Entry)i.next();
-            String headerKey=(String) entry.getKey();
+        StringBuilder hdrs = new StringBuilder(100);
+        Map<String, List<String>> requestHeaders = conn.getRequestProperties();
+        Set<Map.Entry<String, List<String>>> headerFields = requestHeaders.entrySet();
+        for(Iterator<Map.Entry<String, List<String>>> i = headerFields.iterator(); i.hasNext();) {
+            Map.Entry<String, List<String>> entry = i.next();
+            String headerKey=entry.getKey();
             // Exclude the COOKIE header, since cookie is reported separately in the sample
             if(!HEADER_COOKIE.equalsIgnoreCase(headerKey)) {
-                List values = (List) entry.getValue();// value is a List of Strings
+                List<String> values = entry.getValue();// value is a List of Strings
                 for (int j=0;j<values.size();j++){
                     hdrs.append(headerKey);
                     hdrs.append(": "); // $NON-NLS-1$
-                    hdrs.append((String) values.get(j));
+                    hdrs.append(values.get(j));
                     hdrs.append("\n"); // $NON-NLS-1$
                 }
             }
@@ -466,6 +443,7 @@ public class HTTPSampler extends HTTPSamplerBase implements Interruptible {
      *            prevent infinite recursion.
      * @return results of the sampling
      */
+    @Override
     protected HTTPSampleResult sample(URL url, String method, boolean areFollowingRedirect, int frameDepth) {
         HttpURLConnection conn = null;
 
@@ -476,7 +454,22 @@ public class HTTPSampler extends HTTPSamplerBase implements Interruptible {
         res.setMonitor(isMonitor());
 
         res.setSampleLabel(urlStr);
+        res.setURL(url);
+        res.setHTTPMethod(method);
+
         res.sampleStart(); // Count the retries as well in the time
+
+        // Check cache for an entry with an Expires header in the future
+        final CacheManager cacheManager = getCacheManager();
+        if (cacheManager != null && GET.equalsIgnoreCase(method)) {
+           if (cacheManager.inCache(url)) {
+               res.sampleEnd();
+               res.setResponseNoContent();
+               res.setSuccessful(true);
+               return res;
+           }
+        }
+
         try {
             // Sampling proper - establish the connection and read the response:
             // Repeatedly try to connect:
@@ -496,6 +489,7 @@ public class HTTPSampler extends HTTPSamplerBase implements Interruptible {
                     }
                     log.debug("Bind exception, try again");
                     if (conn!=null) {
+                        savedConn = null; // we don't want interrupt to try disconnection again
                         conn.disconnect();
                     }
                     this.setUseKeepAlive(false);
@@ -579,7 +573,6 @@ public class HTTPSampler extends HTTPSamplerBase implements Interruptible {
             saveConnectionCookies(conn, url, getCookieManager());
 
             // Save cache information
-            final CacheManager cacheManager = getCacheManager();
             if (cacheManager != null){
                 cacheManager.saveDetails(conn, res);
             }
@@ -592,16 +585,18 @@ public class HTTPSampler extends HTTPSamplerBase implements Interruptible {
             res.sampleEnd();
             // We don't want to continue using this connection, even if KeepAlive is set
             if (conn != null) { // May not exist
+                savedConn = null; // we don't want interrupt to try disconnection again
                 conn.disconnect();
             }
+            savedConn = null; // we don't want interrupt to try disconnection again
             conn=null; // Don't process again
             return errorResult(e, res);
         } finally {
             // calling disconnect doesn't close the connection immediately,
             // but indicates we're through with it. The JVM should close
             // it when necessary.
+            savedConn = null; // we don't want interrupt to try disconnection again
             disconnect(conn); // Disconnect unless using KeepAlive
-            savedConn = null;
         }
     }
 
