@@ -60,6 +60,7 @@ import org.apache.commons.httpclient.methods.TraceMethod;
 import org.apache.commons.httpclient.methods.multipart.FilePart;
 import org.apache.commons.httpclient.methods.multipart.MultipartRequestEntity;
 import org.apache.commons.httpclient.methods.multipart.Part;
+import org.apache.commons.httpclient.methods.multipart.PartBase;
 import org.apache.commons.httpclient.methods.multipart.StringPart;
 import org.apache.commons.httpclient.params.DefaultHttpParams;
 import org.apache.commons.httpclient.params.HttpMethodParams;
@@ -94,31 +95,35 @@ public class HTTPSampler2 extends HTTPSamplerBase implements Interruptible {
 
     private static final Logger log = LoggingManager.getLoggerForClass();
 
+    private static final long serialVersionUID = 240L;
+
     private static final String HTTP_AUTHENTICATION_PREEMPTIVE = "http.authentication.preemptive"; // $NON-NLS-1$
 
     private static final boolean canSetPreEmptive; // OK to set pre-emptive auth?
 
-    static final String PROXY_HOST =
+    private static final String PROXY_HOST =
         System.getProperty("http.proxyHost",""); // $NON-NLS-1$
 
     private static final String NONPROXY_HOSTS =
         System.getProperty("http.nonProxyHosts",""); // $NON-NLS-1$
 
-    static final int PROXY_PORT =
+    private static final int PROXY_PORT =
         Integer.parseInt(System.getProperty("http.proxyPort","0")); // $NON-NLS-1$
 
     // Have proxy details been provided?
     private static final boolean PROXY_DEFINED = PROXY_HOST.length() > 0 && PROXY_PORT > 0;
 
-    static final String PROXY_USER =
+    private static final String PROXY_USER =
         JMeterUtils.getPropDefault(JMeter.HTTP_PROXY_USER,""); // $NON-NLS-1$
 
-    static final String PROXY_PASS =
+    private static final String PROXY_PASS =
         JMeterUtils.getPropDefault(JMeter.HTTP_PROXY_PASS,""); // $NON-NLS-1$
 
     private static final String PROXY_DOMAIN =
         JMeterUtils.getPropDefault("http.proxyDomain",""); // $NON-NLS-1$ $NON-NLS-2$
 
+    public static final String IP_SOURCE = "HTTPSampler.ipSource"; // $NON-NLS-1$
+
     static final InetAddress localAddress;
 
     private static final String localHost;
@@ -126,10 +131,11 @@ public class HTTPSampler2 extends HTTPSamplerBase implements Interruptible {
     /*
      * Connection is re-used within the thread if possible
      */
-    static final ThreadLocal httpClients = new ThreadLocal();
+    static final ThreadLocal<Map<HostConfiguration, HttpClient>> httpClients =
+        new ThreadLocal<Map<HostConfiguration, HttpClient>>();
 
-    private static final Set nonProxyHostFull   = new HashSet();// www.apache.org
-    private static final List nonProxyHostSuffix = new ArrayList();// .apache.org
+    private static final Set<String> nonProxyHostFull   = new HashSet<String>();// www.apache.org
+    private static final List<String> nonProxyHostSuffix = new ArrayList<String>();// .apache.org
 
     private static final int nonProxyHostSuffixSize;
 
@@ -141,7 +147,7 @@ public class HTTPSampler2 extends HTTPSamplerBase implements Interruptible {
 
     private static boolean isPartialMatch(String host) {
         for (int i=0;i<nonProxyHostSuffixSize;i++){
-            if (host.endsWith((String)nonProxyHostSuffix.get(i))) {
+            if (host.endsWith(nonProxyHostSuffix.get(i))) {
                 return true;
             }
         }
@@ -258,7 +264,7 @@ public class HTTPSampler2 extends HTTPSamplerBase implements Interruptible {
      */
     private String sendPostData(PostMethod post) throws IOException {
         // Buffer to hold the post body, except file content
-        StringBuffer postedBody = new StringBuffer(1000);
+        StringBuilder postedBody = new StringBuilder(1000);
         HTTPFileArg files[] = getHTTPFiles();
         // Check if we should do a multipart/form-data or an
         // application/x-www-form-urlencoded post request
@@ -271,7 +277,7 @@ public class HTTPSampler2 extends HTTPSamplerBase implements Interruptible {
             }
 
             // We don't know how many entries will be skipped
-            ArrayList partlist = new ArrayList();
+            ArrayList<PartBase> partlist = new ArrayList<PartBase>();
             // Create the parts
             // Add any parameters
             PropertyIterator args = getArguments().iterator();
@@ -296,7 +302,7 @@ public class HTTPSampler2 extends HTTPSamplerBase implements Interruptible {
 
             // Set the multipart for the post
             int partNo = partlist.size();
-            Part[] parts = (Part[])partlist.toArray(new Part[partNo]);
+            Part[] parts = partlist.toArray(new Part[partNo]);
             MultipartRequestEntity multiPart = new MultipartRequestEntity(parts, post.getParams());
             post.setRequestEntity(multiPart);
 
@@ -396,7 +402,7 @@ public class HTTPSampler2 extends HTTPSamplerBase implements Interruptible {
                     }
 
                     // Just append all the parameter values, and use that as the post body
-                    StringBuffer postBody = new StringBuffer();
+                    StringBuilder postBody = new StringBuilder();
                     PropertyIterator args = getArguments().iterator();
                     while (args.hasNext()) {
                         HTTPArgument arg = (HTTPArgument) args.next().getObjectValue();
@@ -534,18 +540,35 @@ public class HTTPSampler2 extends HTTPSamplerBase implements Interruptible {
         // Set up the local address if one exists
         if (localAddress != null){
             hc.setLocalAddress(localAddress);
-        }
-
-        boolean useProxy = PROXY_DEFINED && !isNonProxy(host);
-        if (useProxy) {
-            if (log.isDebugEnabled()){
-                log.debug("Setting proxy: "+PROXY_HOST+":"+PROXY_PORT);
+        } else {
+            final String ipSource = getIpSource();
+            if (ipSource.length() > 0) {// Use special field ip source address (for pseudo 'ip spoofing')
+                InetAddress inetAddr = InetAddress.getByName(ipSource);
+                hc.setLocalAddress(inetAddr);
             }
-            hc.setProxy(PROXY_HOST, PROXY_PORT);
         }
 
-        Map map = (Map) httpClients.get();
-        HttpClient httpClient = (HttpClient) map.get(hc);
+        boolean useStaticProxy = PROXY_DEFINED && !isNonProxy(host);
+        boolean useDynamicProxy = false;
+
+        final String proxyHost = getProxyHost();
+        final int proxyPort = getProxyPortInt();
+        if (proxyHost.length() > 0 && proxyPort > 0){
+            hc.setProxy(proxyHost, proxyPort);
+            useStaticProxy = false; // Dynamic proxy overrules static proxy
+            useDynamicProxy = true;
+        } else {
+            if (useStaticProxy) {
+                if (log.isDebugEnabled()){
+                    log.debug("Setting proxy: "+PROXY_HOST+":"+PROXY_PORT);
+                }
+                hc.setProxy(PROXY_HOST, PROXY_PORT);
+            }
+        }
+
+        Map<HostConfiguration, HttpClient> map = httpClients.get();
+        // N.B. HostConfiguration.equals() includes proxy settings in the compare.
+        HttpClient httpClient = map.get(hc);
 
         if ( httpClient == null )
         {
@@ -555,19 +578,33 @@ public class HTTPSampler2 extends HTTPSamplerBase implements Interruptible {
             }
             httpClient.setHostConfiguration(hc);
             map.put(hc, httpClient);
-            // These items don't change, so only need to be done once
-            if (useProxy) {
+        } else {
+            if (log.isDebugEnabled()) {
+                log.debug("Reusing the HttpClient: @"+System.identityHashCode(httpClient));
+            }
+        }
+
+        // Set up any required Proxy credentials
+        if (useDynamicProxy){
+            String user = getProxyUser();
+            if (user.length() > 0){
+                httpClient.getState().setProxyCredentials(
+                        new AuthScope(proxyHost,proxyPort,null,AuthScope.ANY_SCHEME),
+                        new NTCredentials(user,getProxyPass(),localHost,PROXY_DOMAIN)
+                    );
+            } else {
+                httpClient.getState().clearProxyCredentials();
+            }
+        } else {
+            if (useStaticProxy) {
                 if (PROXY_USER.length() > 0){
                     httpClient.getState().setProxyCredentials(
                         new AuthScope(PROXY_HOST,PROXY_PORT,null,AuthScope.ANY_SCHEME),
                         new NTCredentials(PROXY_USER,PROXY_PASS,localHost,PROXY_DOMAIN)
                     );
                 }
-            }
-
-        } else {
-            if (log.isDebugEnabled()) {
-                log.debug("Reusing the HttpClient: @"+System.identityHashCode(httpClient));
+            } else {
+                httpClient.getState().clearProxyCredentials();
             }
         }
 
@@ -601,7 +638,6 @@ public class HTTPSampler2 extends HTTPSamplerBase implements Interruptible {
         setConnectionAuthorization(httpClient, u, getAuthManager());
 
         if (res != null) {
-            res.setURL(u);
             res.setCookies(cookies);
         }
 
@@ -625,7 +661,7 @@ public class HTTPSampler2 extends HTTPSamplerBase implements Interruptible {
      * @return string containing the headers, one per line
      */
     protected String getResponseHeaders(HttpMethod method) {
-        StringBuffer headerBuf = new StringBuffer();
+        StringBuilder headerBuf = new StringBuilder();
         org.apache.commons.httpclient.Header rh[] = method.getResponseHeaders();
         headerBuf.append(method.getStatusLine());// header[0] is not the status line...
         headerBuf.append("\n"); // $NON-NLS-1$
@@ -709,7 +745,7 @@ public class HTTPSampler2 extends HTTPSamplerBase implements Interruptible {
      */
     protected String getConnectionHeaders(HttpMethod method) {
         // Get all the request headers
-        StringBuffer hdrs = new StringBuffer(100);
+        StringBuilder hdrs = new StringBuilder(100);
         Header[] requestHeaders = method.getRequestHeaders();
         for(int i = 0; i < requestHeaders.length; i++) {
             // Exclude the COOKIE header, since cookie is reported separately in the sample
@@ -798,6 +834,7 @@ public class HTTPSampler2 extends HTTPSamplerBase implements Interruptible {
      *            prevent infinite recursion.
      * @return results of the sampling
      */
+    @Override
     protected HTTPSampleResult sample(URL url, String method, boolean areFollowingRedirect, int frameDepth) {
 
         String urlStr = url.toString();
@@ -812,6 +849,8 @@ public class HTTPSampler2 extends HTTPSamplerBase implements Interruptible {
 
         res.setSampleLabel(urlStr); // May be replaced later
         res.setHTTPMethod(method);
+        res.setURL(url);
+
         res.sampleStart(); // Count the retries as well in the time
         HttpClient client = null;
         InputStream instream = null;
@@ -831,6 +870,15 @@ public class HTTPSampler2 extends HTTPSamplerBase implements Interruptible {
                 httpMethod = new DeleteMethod(urlStr);
             } else if (method.equals(GET)){
                 httpMethod = new GetMethod(urlStr);
+                final CacheManager cacheManager = getCacheManager();
+                if (cacheManager != null && GET.equalsIgnoreCase(method)) {
+                   if (cacheManager.inCache(url)) {
+                       res.sampleEnd();
+                       res.setResponseNoContent();
+                       res.setSuccessful(true);
+                       return res;
+                   }
+                }
             } else {
                 log.error("Unexpected method (converted to GET): "+method);
                 httpMethod = new GetMethod(urlStr);
@@ -840,6 +888,8 @@ public class HTTPSampler2 extends HTTPSamplerBase implements Interruptible {
             setDefaultRequestHeaders(httpMethod);
             // Setup connection
             client = setupConnection(url, httpMethod, res);
+            savedClient = client;
+
             // Handle the various methods
             if (method.equals(POST)) {
                 String postBody = sendPostData((PostMethod)httpMethod);
@@ -849,8 +899,6 @@ public class HTTPSampler2 extends HTTPSamplerBase implements Interruptible {
                 res.setQueryString(putBody);
             }
 
-
-            savedClient = client;
             int statusCode = client.executeMethod(httpMethod);
 
             // Needs to be done after execute to pick up all the headers
@@ -945,7 +993,7 @@ public class HTTPSampler2 extends HTTPSamplerBase implements Interruptible {
      */
     private String sendPutData(PutMethod put) throws IOException {
         // Buffer to hold the put body, except file content
-        StringBuffer putBody = new StringBuffer(1000);
+        StringBuilder putBody = new StringBuilder(1000);
         boolean hasPutBody = false;
 
         // Check if the header manager had a content type header
@@ -981,7 +1029,7 @@ public class HTTPSampler2 extends HTTPSamplerBase implements Interruptible {
             }
 
             // Just append all the parameter values, and use that as the post body
-            StringBuffer putBodyContent = new StringBuffer();
+            StringBuilder putBodyContent = new StringBuilder();
             PropertyIterator args = getArguments().iterator();
             while (args.hasNext()) {
                 HTTPArgument arg = (HTTPArgument) args.next().getObjectValue();
@@ -1048,6 +1096,7 @@ public class HTTPSampler2 extends HTTPSamplerBase implements Interruptible {
             this.hideFileData = hideFileData;
         }
 
+        @Override
         protected void sendData(OutputStream out) throws IOException {
             // Check if we should send only placeholder text for the
             // file content, or the real file content
@@ -1081,24 +1130,26 @@ public class HTTPSampler2 extends HTTPSamplerBase implements Interruptible {
     }
 
 
+    @Override
     public void threadStarted() {
         log.debug("Thread Started");
 
         // Does not need to be synchronised, as all access is from same thread
-        httpClients.set ( new HashMap() );
+        httpClients.set ( new HashMap<HostConfiguration, HttpClient>() );
     }
 
+    @Override
     public void threadFinished() {
         log.debug("Thread Finished");
 
         // Does not need to be synchronised, as all access is from same thread
-        Map map = (Map)httpClients.get();
+        Map<HostConfiguration, HttpClient> map = httpClients.get();
 
         if ( map != null ) {
-            for ( Iterator it = map.entrySet().iterator(); it.hasNext(); )
+            for ( Iterator<Map.Entry<HostConfiguration, HttpClient> > it = map.entrySet().iterator(); it.hasNext(); )
             {
-                Map.Entry entry = (Map.Entry) it.next();
-                HttpClient cl = (HttpClient) entry.getValue();
+                Map.Entry<HostConfiguration, HttpClient> entry = it.next();
+                HttpClient cl = entry.getValue();
                 // Can cause NPE in HttpClient 3.1
                 //((SimpleHttpConnectionManager)cl.getHttpConnectionManager()).shutdown();// Closes the connection
                 // Revert to original method:
@@ -1121,4 +1172,13 @@ public class HTTPSampler2 extends HTTPSamplerBase implements Interruptible {
         }
         return client != null;
     }
+
+
+    public void setIpSource(String value) {
+        setProperty(IP_SOURCE, value, "");
+    }
+
+    public String getIpSource() {
+        return getPropertyAsString(IP_SOURCE,"");
+    }
 }
