@@ -78,6 +78,8 @@ import org.apache.oro.text.regex.Perl5Matcher;
 public abstract class HTTPSamplerBase extends AbstractSampler
     implements TestListener, ThreadListener, HTTPConstantsInterface {
 
+    private static final long serialVersionUID = 240L;
+
     private static final Logger log = LoggingManager.getLoggerForClass();
 
     public static final String ARGUMENTS = "HTTPsampler.Arguments"; // $NON-NLS-1$
@@ -94,6 +96,14 @@ public abstract class HTTPSamplerBase extends AbstractSampler
 
     public static final String PORT = "HTTPSampler.port"; // $NON-NLS-1$
 
+    public static final String PROXYHOST = "HTTPSampler.proxyHost"; // $NON-NLS-1$
+
+    public static final String PROXYPORT = "HTTPSampler.proxyPort"; // $NON-NLS-1$
+
+    public static final String PROXYUSER = "HTTPSampler.proxyUser"; // $NON-NLS-1$
+
+    public static final String PROXYPASS = "HTTPSampler.proxyPass"; // $NON-NLS-1$
+
     public static final String CONNECT_TIMEOUT = "HTTPSampler.connect_timeout"; // $NON-NLS-1$
 
     public static final String RESPONSE_TIMEOUT = "HTTPSampler.response_timeout"; // $NON-NLS-1$
@@ -132,33 +142,14 @@ public abstract class HTTPSamplerBase extends AbstractSampler
         DELETE,
         };
 
-    private static final List METHODLIST = Collections.unmodifiableList(Arrays.asList(METHODS));
+    private static final List<String> METHODLIST = Collections.unmodifiableList(Arrays.asList(METHODS));
 
 
     public static final String USE_KEEPALIVE = "HTTPSampler.use_keepalive"; // $NON-NLS-1$
 
     public static final String DO_MULTIPART_POST = "HTTPSampler.DO_MULTIPART_POST"; // $NON-NLS-1$
 
-    /*
-     * JMeter 2.3.1 and earlier only had fields for one file on the GUI:
-     * - FILE_NAME
-     * - FILE_FIELD
-     * - MIMETYPE
-     * These were stored in their own individual properties.
-     *
-     * Version 2.3.3 introduced a list of files, each with their own path, name and mimetype.
-     *
-     * In order to maintain backwards compatibility of test plans, the 3 original properties
-     * have been retained; additional file entries are stored in an HTTPFileArgs class.
-     * The HTTPFileArgs class is only present if there is more than 1 file; this means that
-     * such test plans are backward compatible.
-     *
-     * The original get methods still work, but have been deprecated;
-     * the user is expected to retrieve the list of files using the getHTTPFiles() method.
-     *
-     * In order to speed up processing, the class maintains a copy of the current
-     * files in the fileList variable.
-     */
+    // @see mergeFileProperties
     // Must be private, as the file list needs special handling
     private final static String FILE_ARGS = "HTTPsampler.Files"; // $NON-NLS-1$
     // MIMETYPE is kept for backward compatibility with old test plans
@@ -207,13 +198,13 @@ public abstract class HTTPSamplerBase extends AbstractSampler
 
 
     // Derive the mapping of content types to parsers
-    private static final Map parsersForType = new HashMap();
+    private static final Map<String, String> parsersForType = new HashMap<String, String>();
     // Not synch, but it is not modified after creation
 
     private static final String RESPONSE_PARSERS= // list of parsers
         JMeterUtils.getProperty("HTTPResponse.parsers");//$NON-NLS-1$
 
-   static{
+    static{
         String []parsers = JOrphanUtils.split(RESPONSE_PARSERS, " " , true);// returns empty array for null
         for (int i=0;i<parsers.length;i++){
             final String parser = parsers[i];
@@ -240,13 +231,14 @@ public abstract class HTTPSamplerBase extends AbstractSampler
         }
     }
 
+    // Bug 49083
+    /** Whether to remove '/pathsegment/..' from redirects; default true */
+    private static boolean REMOVESLASHDOTDOT = JMeterUtils.getPropDefault("httpsampler.redirect.removeslashdotdot", true);
+
     ////////////////////// Variables //////////////////////
 
     private boolean dynamicPath = false;// Set false if spaces are already encoded
 
-    // must be kept in synch with the various file properties
-    private transient HTTPFileArg[] fileList;
-    // The code assumes that this will be accessed from a single thread only
 
     ////////////////////// Code ///////////////////////////
 
@@ -254,88 +246,6 @@ public abstract class HTTPSamplerBase extends AbstractSampler
         setArguments(new Arguments());
     }
 
-    /**
-     * The name parameter to be applied to the file
-     * @deprecated use setHTTPFiles() instead
-     */
-    public void setFileField(String value) {
-        fileList = null; // Force rebuild
-        setFileFieldProperty(value);
-    }
-
-    private void setFileFieldProperty(String value) {
-        if (value == null) {
-            throw new IllegalArgumentException("Value must not be null");
-        }
-        setProperty(FILE_FIELD, value);
-    }
-
-    /**
-     * The name parameter to be applied to the file
-     * @deprecated Use getHTTPFiles() array instead
-     */
-    public String getFileField() {
-        checkCount("getFileField");
-        return getPropertyAsString(FILE_FIELD);
-    }
-
-    /**
-     * The actual name of the file to POST
-     * @deprecated use setHTTPFiles() instead
-     */
-    public void setFilename(String value) {
-        fileList = null; // Force rebuild
-        setFilenameProperty(value);
-    }
-
-    private void setFilenameProperty(String value) {
-        if (value == null) {
-            throw new IllegalArgumentException("Value must not be null");
-        }
-        setProperty(FILE_NAME, value);
-    }
-
-    /**
-     * The actual name of the file to POST
-     * @deprecated Use getHTTPFiles() array instead
-     */
-    public String getFilename() {
-        checkCount("getFilename");
-        return getPropertyAsString(FILE_NAME);
-    }
-
-    /**
-     * Set the files mime type
-     * @deprecated use setHTTPFiles() instead
-     * @param value
-     */
-    public void setMimetype(String value) {
-        fileList = null; // Force rebuild
-        setMimetypeProperty(value);
-    }
-
-    private void setMimetypeProperty(String value) {
-        if (value == null) {
-            throw new IllegalArgumentException("Value must not be null");
-        }
-        setProperty(MIMETYPE, value);
-    }
-
-    /**
-     * @deprecated Use getHTTPFiles() array instead
-     */
-    public String getMimetype() {
-        checkCount("getMimetype");
-        return getPropertyAsString(MIMETYPE);
-    }
-
-    private void checkCount(String method) {
-        if (getHTTPFileCount() > 1) {
-            log.warn(method + "() called with more than 1 file; additional files may be ignored.");
-        }
-
-    }
-
     /**
      * Determine if the file should be sent as the entire Post body,
      * i.e. without any additional wrapping
@@ -603,6 +513,7 @@ public abstract class HTTPSamplerBase extends AbstractSampler
         return getArguments().getArgumentCount() > 0;
     }
 
+    @Override
     public void addTestElement(TestElement el) {
         if (el instanceof CookieManager) {
             setCookieManager((CookieManager) el);
@@ -713,6 +624,22 @@ public abstract class HTTPSamplerBase extends AbstractSampler
         return getPropertyAsInt(RESPONSE_TIMEOUT, 0);
     }
 
+    public String getProxyHost() {
+        return getPropertyAsString(PROXYHOST);
+    }
+
+    public int getProxyPortInt() {
+        return getPropertyAsInt(PROXYPORT, 0);
+    }
+
+    public String getProxyUser() {
+        return getPropertyAsString(PROXYUSER);
+    }
+
+    public String getProxyPass() {
+        return getPropertyAsString(PROXYPASS);
+    }
+
     public void setArguments(Arguments value) {
         setProperty(new TestElementProperty(ARGUMENTS, value));
     }
@@ -808,8 +735,8 @@ public abstract class HTTPSamplerBase extends AbstractSampler
      * @return a sampling result useful to inform the user about the exception.
      */
     protected HTTPSampleResult errorResult(Throwable e, HTTPSampleResult res) {
-        res.setSampleLabel("Error");
-        res.setDataType(HTTPSampleResult.TEXT);
+        res.setSampleLabel("Error: " + res.getSampleLabel());
+        res.setDataType(SampleResult.TEXT);
         ByteArrayOutputStream text = new ByteArrayOutputStream(200);
         e.printStackTrace(new PrintStream(text));
         res.setResponseData(text.toByteArray());
@@ -835,7 +762,7 @@ public abstract class HTTPSamplerBase extends AbstractSampler
      * @throws MalformedURLException
      */
     public URL getUrl() throws MalformedURLException {
-        StringBuffer pathAndQuery = new StringBuffer(100);
+        StringBuilder pathAndQuery = new StringBuilder(100);
         String path = this.getPath();
         // Hack to allow entire URL to be provided in host field
         if (path.startsWith(HTTP_PREFIX)
@@ -893,7 +820,7 @@ public abstract class HTTPSamplerBase extends AbstractSampler
              // We use the encoding which should be used according to the HTTP spec, which is UTF-8
              contentEncoding = EncoderCache.URL_ARGUMENT_ENCODING;
          }
-        StringBuffer buf = new StringBuffer();
+        StringBuilder buf = new StringBuilder();
         PropertyIterator iter = getArguments().iterator();
         boolean first = true;
         while (iter.hasNext()) {
@@ -997,9 +924,10 @@ public abstract class HTTPSamplerBase extends AbstractSampler
         parseArguments(queryString, null);
     }
 
+    @Override
     public String toString() {
         try {
-            StringBuffer stringBuffer = new StringBuffer();
+            StringBuilder stringBuffer = new StringBuilder();
             stringBuffer.append(this.getUrl().toString());
             // Append body if it is a post or put
             if(POST.equals(getMethod()) || PUT.equals(getMethod())) {
@@ -1061,7 +989,7 @@ public abstract class HTTPSamplerBase extends AbstractSampler
         res.sampleStart();
         try {
             byte[] responseData;
-            StringBuffer ctb=new StringBuffer("text/html"); // $NON-NLS-1$
+            StringBuilder ctb=new StringBuilder("text/html"); // $NON-NLS-1$
             fis = new FileInputStream(getPath());
             String contentEncoding = getContentEncoding();
             if (contentEncoding.length() > 0) {
@@ -1110,25 +1038,18 @@ public abstract class HTTPSamplerBase extends AbstractSampler
 
     /**
      * Download the resources of an HTML page.
-     * <p>
-     * If createContainerResult is true, the returned result will contain one
-     * subsample for each request issued, including the original one that was
-     * passed in. It will otherwise look exactly like that original one.
-     * <p>
-     * If createContainerResult is false, one subsample will be added to the
-     * provided result for each requests issued.
-     *
+     * 
      * @param res
      *            result of the initial request - must contain an HTML response
      * @param container
-     *            for storing the results
+     *            for storing the results, if any
      * @param frameDepth
      *            Depth of this target in the frame structure. Used only to
      *            prevent infinite recursion.
-     * @return "Container" result with one subsample per request issued
+     * @return res if no resources exist, otherwise the "Container" result with one subsample per request issued
      */
     protected HTTPSampleResult downloadPageResources(HTTPSampleResult res, HTTPSampleResult container, int frameDepth) {
-        Iterator urls = null;
+        Iterator<URL> urls = null;
         try {
             final byte[] responseData = res.getResponseData();
             if (responseData.length > 0){  // Bug 39205
@@ -1152,10 +1073,10 @@ public abstract class HTTPSamplerBase extends AbstractSampler
         // Iterate through the URLs and download each image:
         if (urls != null && urls.hasNext()) {
             if (container == null) {
-                res = new HTTPSampleResult(res);
-            } else {
-                res = container;
+                container = new HTTPSampleResult(res);
+                container.addRawSubResult(res);
             }
+            res = container;
 
             // Get the URL matcher
             String re=getEmbeddedUrlRE();
@@ -1170,7 +1091,7 @@ public abstract class HTTPSamplerBase extends AbstractSampler
                 }
             }
             while (urls.hasNext()) {
-                Object binURL = urls.next();
+                Object binURL = urls.next(); // See catch clause below
                 try {
                     URL url = (URL) binURL;
                     if (url == null) {
@@ -1195,7 +1116,7 @@ public abstract class HTTPSamplerBase extends AbstractSampler
                         res.addSubResult(binRes);
                         res.setSuccessful(res.isSuccessful() && binRes.isSuccessful());
                     }
-                } catch (ClassCastException e) {
+                } catch (ClassCastException e) { // TODO can this happen?
                     res.addSubResult(errorResult(new Exception(binURL + " is not a correct URI"), res));
                     res.setSuccessful(false);
                     continue;
@@ -1211,7 +1132,7 @@ public abstract class HTTPSamplerBase extends AbstractSampler
      */
     private String getParserClass(HTTPSampleResult res) {
         final String ct = res.getMediaType();
-        return (String)parsersForType.get(ct);
+        return parsersForType.get(ct);
     }
 
     // TODO: make static?
@@ -1219,36 +1140,28 @@ public abstract class HTTPSamplerBase extends AbstractSampler
         return JOrphanUtils.replaceAllChars(path, ' ', "%20"); // $NON-NLS-1$
     }
 
-    /*
-     * (non-Javadoc)
-     *
-     * @see org.apache.jmeter.testelement.TestListener#testEnded()
+    /**
+     * {@inheritDoc}
      */
     public void testEnded() {
         dynamicPath = false;
     }
 
-    /*
-     * (non-Javadoc)
-     *
-     * @see org.apache.jmeter.testelement.TestListener#testEnded(java.lang.String)
+    /**
+     * {@inheritDoc}
      */
     public void testEnded(String host) {
         testEnded();
     }
 
-    /*
-     * (non-Javadoc)
-     *
-     * @see org.apache.jmeter.testelement.TestListener#testIterationStart(org.apache.jmeter.engine.event.LoopIterationEvent)
+    /**
+     * {@inheritDoc}
      */
     public void testIterationStart(LoopIterationEvent event) {
     }
 
-    /*
-     * (non-Javadoc)
-     *
-     * @see org.apache.jmeter.testelement.TestListener#testStarted()
+    /**
+     * {@inheritDoc}
      */
     public void testStarted() {
         JMeterProperty pathP = getProperty(PATH);
@@ -1265,24 +1178,20 @@ public abstract class HTTPSamplerBase extends AbstractSampler
         log.debug("path ending value = " + pathP.getStringValue());
     }
 
-    /*
-     * (non-Javadoc)
-     *
-     * @see org.apache.jmeter.testelement.TestListener#testStarted(java.lang.String)
+    /**
+     * {@inheritDoc}
      */
     public void testStarted(String host) {
         testStarted();
     }
 
-    /*
-     * (non-Javadoc)
-     *
-     * @see java.lang.Object#clone()
+    /**
+     * {@inheritDoc}
      */
+    @Override
     public Object clone() {
         HTTPSamplerBase base = (HTTPSamplerBase) super.clone();
         base.dynamicPath = dynamicPath;
-        base.fileList = null;
         return base;
     }
 
@@ -1303,6 +1212,7 @@ public abstract class HTTPSamplerBase extends AbstractSampler
      */
     protected HTTPSampleResult followRedirects(HTTPSampleResult res, int frameDepth) {
         HTTPSampleResult totalRes = new HTTPSampleResult(res);
+        totalRes.addRawSubResult(res);
         HTTPSampleResult lastRes = res;
 
         int redirect;
@@ -1311,7 +1221,11 @@ public abstract class HTTPSamplerBase extends AbstractSampler
             // Browsers seem to tolerate Location headers with spaces,
             // replacing them automatically with %20. We want to emulate
             // this behaviour.
-            String location = encodeSpaces(lastRes.getRedirectLocation());
+            String location = lastRes.getRedirectLocation(); 
+            if (REMOVESLASHDOTDOT) {
+                location = ConversionUtils.removeSlashDotDot(location);
+            }
+            location = encodeSpaces(location);
             try {
                 lastRes = sample(ConversionUtils.makeRelativeURL(lastRes.getURL(), location), GET, true, frameDepth);
             } catch (MalformedURLException e) {
@@ -1394,17 +1308,13 @@ public abstract class HTTPSamplerBase extends AbstractSampler
             if (frameDepth > MAX_FRAME_DEPTH) {
                 res.addSubResult(errorResult(new Exception("Maximum frame/iframe nesting depth exceeded."), res));
             } else {
-                // If we followed redirects, we already have a container:
-                if(!areFollowingRedirect) {
-                HTTPSampleResult container = (HTTPSampleResult) (areFollowingRedirect ? res.getParent() : res);
-
                 // Only download page resources if we were not redirected.
                 // If we were redirected, the page resources have already been
                 // downloaded for the sample made for the redirected url
                 if(!wasRedirected) {
+                    HTTPSampleResult container = (HTTPSampleResult) (areFollowingRedirect ? res.getParent() : res);
                     res = downloadPageResources(res, container, frameDepth);
                 }
-                }
             }
         }
         return res;
@@ -1421,7 +1331,7 @@ public abstract class HTTPSamplerBase extends AbstractSampler
     }
 
     protected static String encodeBackSlashes(String value) {
-        StringBuffer newValue = new StringBuffer();
+        StringBuilder newValue = new StringBuilder();
         for (int i = 0; i < value.length(); i++) {
             char charAt = value.charAt(i);
             if (charAt == '\\') { // $NON-NLS-1$
@@ -1440,7 +1350,11 @@ public abstract class HTTPSamplerBase extends AbstractSampler
      *   HTTPFileArgs object that stores file list to be uploaded.
      */
     private void setHTTPFileArgs(HTTPFileArgs value) {
-        setProperty(new TestElementProperty(FILE_ARGS, value));
+        if (value.getHTTPFileArgCount() > 0){
+            setProperty(new TestElementProperty(FILE_ARGS, value));
+        } else {
+            removeProperty(FILE_ARGS); // no point saving an empty list
+        }
     }
 
     /*
@@ -1450,15 +1364,6 @@ public abstract class HTTPSamplerBase extends AbstractSampler
         return (HTTPFileArgs) getProperty(FILE_ARGS).getObjectValue();
     }
 
-    /*
-     * Method to clear the additional files list to be uploaded.
-     *
-     *   HTTPFileArgs object that stores file list to be uploaded.
-     */
-    private void clearHTTPFileArgs() {
-        removeProperty(FILE_ARGS);
-    }
-
     /**
      * Get the collection of files as a list.
      * The list is built up from the filename/filefield/mimetype properties,
@@ -1469,34 +1374,8 @@ public abstract class HTTPSamplerBase extends AbstractSampler
      * @return an array of file arguments (never null)
      */
     public HTTPFileArg[] getHTTPFiles() {
-        if (fileList != null){
-            return fileList;
-        }
-        HTTPFileArg[] outFiles;
-        // Check for original data names
-        // Use properties so variables and functions are not resolved too early
-        JMeterProperty fileName = getProperty(FILE_NAME);
-        JMeterProperty paramName = getProperty(FILE_FIELD);
-        JMeterProperty mimeType = getProperty(MIMETYPE);
-        HTTPFileArg file = new HTTPFileArg(fileName, paramName, mimeType);
-        if(file.isNotEmpty()) {
-            // Now deal with any additional file arguments
-            final HTTPFileArgs fileArgs = getHTTPFileArgs();
-            if(fileArgs != null) {
-                outFiles = new HTTPFileArg[1+fileArgs.getHTTPFileArgCount()];
-                outFiles[0] = file; // first file
-                HTTPFileArg[] infiles = fileArgs.asArray();
-                for (int i = 0; i < infiles.length; i++){
-                    outFiles[i+1] = infiles[i];
-                }
-            } else {
-                outFiles = new HTTPFileArg[]{file}; // just one file
-            }
-        } else {
-            outFiles = new HTTPFileArg[]{}; // no files, empty array
-        }
-        fileList = outFiles; // update the list cache
-        return fileList;
+        final HTTPFileArgs fileArgs = getHTTPFileArgs();
+        return fileArgs == null ? new HTTPFileArg[] {} : fileArgs.asArray();
     }
 
     public int getHTTPFileCount(){
@@ -1510,49 +1389,21 @@ public abstract class HTTPSamplerBase extends AbstractSampler
      * @param files list of files to save
      */
     public void setHTTPFiles(HTTPFileArg[] files) {
-        clearHTTPFileArgs();
-        fileList = null; // it will be regenerated by get
-        // First weed out the empty files
-        HTTPFileArg[] nonEmptyFile = {};
-        int filesFound = 0;
+        HTTPFileArgs fileArgs = new HTTPFileArgs();
+        // Weed out the empty files
         if (files.length > 0) {
-            nonEmptyFile = new HTTPFileArg[files.length];
             for(int i=0; i < files.length; i++){
                 HTTPFileArg file = files[i];
                 if (file.isNotEmpty()){
-                    nonEmptyFile[filesFound++] = file;
+                    fileArgs.addHTTPFileArg(file);
                 }
             }
         }
-        // Any files left?
-        if (filesFound > 0){
-            HTTPFileArg file = nonEmptyFile[0];
-            setFilenameProperty(file.getPath());
-            setFileFieldProperty(file.getParamName());
-            setMimetypeProperty(file.getMimeType());
-            if (filesFound > 1){
-                HTTPFileArgs fileArgs = new HTTPFileArgs();
-                boolean empty=true;
-                for(int i=1; i < filesFound; i++){
-                    final HTTPFileArg fileArg = nonEmptyFile[i];
-                    if (fileArg.isNotEmpty()){
-                        fileArgs.addHTTPFileArg(fileArg);
-                        empty=false;
-                    }
-                }
-                if (!empty){
-                    setHTTPFileArgs(fileArgs);
-                }
-            }
-        } else {
-            setFilenameProperty("");
-            setFileFieldProperty("");
-            setMimetypeProperty("");
-        }
+        setHTTPFileArgs(fileArgs);
     }
 
     public static String[] getValidMethodsAsArray(){
-        return (String[]) METHODLIST.toArray(new String[0]);
+        return METHODLIST.toArray(new String[0]);
     }
 
     public static boolean isSecure(String protocol){
@@ -1625,11 +1476,60 @@ public abstract class HTTPSamplerBase extends AbstractSampler
         w.flush();
         if (asMD5 && md != null) {
             byte[] md5Result = md.digest();
-            w.write(JOrphanUtils.baToHexString(md5Result).getBytes());
+            w.write(JOrphanUtils.baToHexBytes(md5Result)); 
             sampleResult.setBytes(totalBytes);
         }
         w.close();
         return w.toByteArray();
     }
+
+    /**
+     * JMeter 2.3.1 and earlier only had fields for one file on the GUI:
+     * - FILE_NAME
+     * - FILE_FIELD
+     * - MIMETYPE
+     * These were stored in their own individual properties.
+     *
+     * Version 2.3.3 introduced a list of files, each with their own path, name and mimetype.
+     *
+     * In order to maintain backwards compatibility of test plans, the 3 original properties
+     * were retained; additional file entries are stored in an HTTPFileArgs class.
+     * The HTTPFileArgs class was only present if there is more than 1 file; this means that
+     * such test plans are backward compatible.
+     *
+     * Versions after 2.3.4 dispense with the original set of 3 properties.
+     * Test plans that use them are converted to use a single HTTPFileArgs list.
+     *
+     * @see HTTPSamplerBaseConverter
+     */
+    void mergeFileProperties() {
+        JMeterProperty fileName = getProperty(FILE_NAME);
+        JMeterProperty paramName = getProperty(FILE_FIELD);
+        JMeterProperty mimeType = getProperty(MIMETYPE);
+        HTTPFileArg oldStyleFile = new HTTPFileArg(fileName, paramName, mimeType);
+
+        HTTPFileArgs fileArgs = getHTTPFileArgs();
+
+        HTTPFileArgs allFileArgs = new HTTPFileArgs();
+        if(oldStyleFile.isNotEmpty()) { // OK, we have an old-style file definition
+            allFileArgs.addHTTPFileArg(oldStyleFile); // save it
+            // Now deal with any additional file arguments
+            if(fileArgs != null) {
+                HTTPFileArg[] infiles = fileArgs.asArray();
+                for (int i = 0; i < infiles.length; i++){
+                    allFileArgs.addHTTPFileArg(infiles[i]);
+                }
+            }
+        } else {
+            if(fileArgs != null) { // for new test plans that don't have FILE/PARAM/MIME properties
+                allFileArgs = fileArgs;
+            }
+        }
+        // Updated the property lists
+        setHTTPFileArgs(allFileArgs);
+        removeProperty(FILE_FIELD);
+        removeProperty(FILE_NAME);
+        removeProperty(MIMETYPE);
+    }
 }
 
