@@ -40,7 +40,7 @@ import org.apache.jmeter.testelement.property.PropertyIterator;
 public class PostWriter {
 
     private static final String DASH_DASH = "--";  // $NON-NLS-1$
-    private static final byte[] DASH_DASH_BYTES = DASH_DASH.getBytes();
+    private static final byte[] DASH_DASH_BYTES = {'-', '-'};
 
     /** The bounday string between multiparts */
     protected final static String BOUNDARY = "---------------------------7d159c1302d0y0"; // $NON-NLS-1$
@@ -82,7 +82,7 @@ public class PostWriter {
      */
     public String sendPostData(URLConnection connection, HTTPSampler sampler) throws IOException {
         // Buffer to hold the post body, except file content
-        StringBuffer postedBody = new StringBuffer(1000);
+        StringBuilder postedBody = new StringBuilder(1000);
 
         HTTPFileArg files[] = sampler.getHTTPFiles();
 
@@ -107,10 +107,10 @@ public class PostWriter {
             for (int i=0; i < files.length; i++) {
                 HTTPFileArg file = files[i];
                 // First write the start multipart file
-                byte[] header = file.getHeader().getBytes(); // TODO what encoding should be used here?
+                byte[] header = file.getHeader().getBytes();  // TODO - charset?
                 out.write(header);
                 // Retrieve the formatted data using the same encoding used to create it
-                postedBody.append(new String(header));
+                postedBody.append(new String(header)); // TODO - charset?
                 // Write the actual file content
                 writeFileToStream(file.getPath(), out);
                 // We just add placeholder text for file content
@@ -122,7 +122,7 @@ public class PostWriter {
                 postedBody.append(new String(fileMultipartEndDivider, ENCODING));
                 if(i + 1 < files.length) {
                     out.write(CRLF);
-                    postedBody.append(new String(CRLF));
+                    postedBody.append(new String(CRLF)); // TODO - charset?
                 }
             }
             // Write end of multipart
@@ -299,7 +299,7 @@ public class PostWriter {
                     }
 
                     // Just append all the parameter values, and use that as the post body
-                    StringBuffer postBodyBuffer = new StringBuffer();
+                    StringBuilder postBodyBuffer = new StringBuilder();
                     PropertyIterator args = sampler.getArguments().iterator();
                     while (args.hasNext()) {
                         HTTPArgument arg = (HTTPArgument) args.next().getObjectValue();
