@@ -77,12 +77,13 @@ public class JDBCSampler extends AbstractSampler implements TestBean {
     private static final String ENCODING = "UTF-8"; // $NON-NLS-1$
 
     // key: name (lowercase) from java.sql.Types; entry: corresponding int value
-    private static final Map mapJdbcNameToInt;
+    private static final Map<String, Integer> mapJdbcNameToInt;
+    // read-only after class init
 
     static {
         // based on e291. Getting the Name of a JDBC Type from javaalmanac.com
         // http://javaalmanac.com/egs/java.sql/JdbcInt2Str.html
-        mapJdbcNameToInt = new HashMap();
+        mapJdbcNameToInt = new HashMap<String, Integer>();
 
         //Get all fields in java.sql.Types and store the corresponding int values
         Field[] fields = java.sql.Types.class.getFields();
@@ -121,16 +122,18 @@ public class JDBCSampler extends AbstractSampler implements TestBean {
     /**
      *  Cache of PreparedStatements stored in a per-connection basis. Each entry of this
      *  cache is another Map mapping the statement string to the actual PreparedStatement.
-     *  The cache has a fixed size of MAX_ENTRIES and it will throw aways all PreparedStatements
+     *  The cache has a fixed size of MAX_ENTRIES and it will throw away all PreparedStatements
      *  from the least recently used connections.
      */
-    private static final Map perConnCache = new LinkedHashMap(MAX_ENTRIES){
+    private static final Map<Connection, Map<String, PreparedStatement>> perConnCache =
+        new LinkedHashMap<Connection, Map<String, PreparedStatement>>(MAX_ENTRIES){
         private static final long serialVersionUID = 1L;
-        protected boolean removeEldestEntry(java.util.Map.Entry arg0) {
+        @Override
+        protected boolean removeEldestEntry(Map.Entry<Connection, Map<String, PreparedStatement>> arg0) {
             if (size() > MAX_ENTRIES) {
                 final Object value = arg0.getValue();
-                if (value instanceof Map) {
-                    closeAllStatements(((Map)value).values());
+                if (value instanceof Map<?,?>) {
+                    closeAllStatements(((Map<?,?>)value).values());
                 }
                 return true;
             }
@@ -335,7 +338,7 @@ public class JDBCSampler extends AbstractSampler implements TestBean {
 
 
     private static int getJdbcType(String jdbcType) throws SQLException {
-        Integer entry = (Integer)mapJdbcNameToInt.get(jdbcType.toLowerCase(java.util.Locale.ENGLISH));
+        Integer entry = mapJdbcNameToInt.get(jdbcType.toLowerCase(java.util.Locale.ENGLISH));
         if (entry == null) {
             try {
                 entry = Integer.decode(jdbcType);
@@ -356,11 +359,14 @@ public class JDBCSampler extends AbstractSampler implements TestBean {
     }
 
     private PreparedStatement getPreparedStatement(Connection conn, boolean callable) throws SQLException {
-        Map preparedStatementMap = (Map) perConnCache.get(conn);
+        Map<String, PreparedStatement> preparedStatementMap = perConnCache.get(conn);
         if (null == preparedStatementMap ) {
             // MRU PreparedStatements cache.
-            preparedStatementMap = new LinkedHashMap(MAX_ENTRIES) {
-                protected boolean removeEldestEntry(java.util.Map.Entry arg0) {
+            preparedStatementMap = new LinkedHashMap<String, PreparedStatement>(MAX_ENTRIES) {
+                private static final long serialVersionUID = 240L;
+
+                @Override
+                protected boolean removeEldestEntry(Map.Entry<String, PreparedStatement> arg0) {
                     final int theSize = size();
                     if (theSize > MAX_ENTRIES) {
                         Object value = arg0.getValue();
@@ -375,7 +381,7 @@ public class JDBCSampler extends AbstractSampler implements TestBean {
             };
             perConnCache.put(conn, preparedStatementMap);
         }
-        PreparedStatement pstmt = (PreparedStatement) preparedStatementMap.get(getQuery());
+        PreparedStatement pstmt = preparedStatementMap.get(getQuery());
         if (null == pstmt) {
             if (callable) {
                 pstmt = conn.prepareCall(getQuery());
@@ -388,8 +394,8 @@ public class JDBCSampler extends AbstractSampler implements TestBean {
         return pstmt;
     }
 
-    private static void closeAllStatements(Collection collection) {
-        Iterator iterator = collection.iterator();
+    private static void closeAllStatements(Collection<?> collection) {
+        Iterator<?> iterator = collection.iterator();
         while (iterator.hasNext()) {
             PreparedStatement pstmt = (PreparedStatement) iterator.next();
             close(pstmt);
@@ -403,13 +409,13 @@ public class JDBCSampler extends AbstractSampler implements TestBean {
      *            ResultSet passed in from a database query
      * @return a Data object
      * @throws java.sql.SQLException
-     * @throws UnsupportedEncodingException 
+     * @throws UnsupportedEncodingException
      */
     private String getStringFromResultSet(ResultSet rs) throws SQLException, UnsupportedEncodingException {
         ResultSetMetaData meta = rs.getMetaData();
 
         StrBuilder sb = new StrBuilder();
-        
+
         int numColumns = meta.getColumnCount();
         for (int i = 1; i <= numColumns; i++) {
             sb.append(meta.getColumnName(i));
@@ -463,7 +469,7 @@ public class JDBCSampler extends AbstractSampler implements TestBean {
                 jmvars.put(varCount, Integer.toString(j)); // save the current count
             }
         }
-        
+
         return sb.toString();
     }
 
@@ -501,6 +507,7 @@ public class JDBCSampler extends AbstractSampler implements TestBean {
         return query;
     }
 
+    @Override
     public String toString() {
         StrBuilder sb = new StrBuilder(80);
         sb.append("["); // $NON-NLS-1$
