@@ -46,6 +46,8 @@ import org.apache.log.Logger;
  * with the existing jmeter menubar.
  */
 public class ReportMenuBar extends JMenuBar implements LocaleChangeListener {
+    private static final long serialVersionUID = 240L;
+
     private static final Logger log = LoggingManager.getLoggerForClass();
 
     private JMenu fileMenu;
@@ -75,7 +77,7 @@ public class ReportMenuBar extends JMenuBar implements LocaleChangeListener {
 
     private JMenuItem remote_start_all;
 
-    private final Collection remote_engine_start;
+    private final Collection<JMenuItem> remote_engine_start;
 
     private JMenuItem run_stop;
 
@@ -85,7 +87,7 @@ public class ReportMenuBar extends JMenuBar implements LocaleChangeListener {
 
     private JMenuItem remote_stop_all;
 
-    private final Collection remote_engine_stop;
+    private final Collection<JMenuItem> remote_engine_stop;
 
     private JMenuItem run_clear;
 
@@ -109,12 +111,12 @@ public class ReportMenuBar extends JMenuBar implements LocaleChangeListener {
 
     private JMenuItem remote_exit_all;
 
-    private final Collection remote_engine_exit;
+    private final Collection<JMenuItem> remote_engine_exit;
 
     public ReportMenuBar() {
-        remote_engine_start = new LinkedList();
-        remote_engine_stop = new LinkedList();
-        remote_engine_exit = new LinkedList();
+        remote_engine_start = new LinkedList<JMenuItem>();
+        remote_engine_stop = new LinkedList<JMenuItem>();
+        remote_engine_exit = new LinkedList<JMenuItem>();
         remoteHosts = JOrphanUtils.split(JMeterUtils.getPropDefault("remote_hosts", ""), ",");
         if (remoteHosts.length == 1 && remoteHosts[0].equals("")) {
             remoteHosts = new String[0];
@@ -409,13 +411,13 @@ public class ReportMenuBar extends JMenuBar implements LocaleChangeListener {
     public void setRunning(boolean running, String host) {
         log.info("setRunning(" + running + "," + host + ")");
 
-        Iterator iter = remote_engine_start.iterator();
-        Iterator iter2 = remote_engine_stop.iterator();
-        Iterator iter3 = remote_engine_exit.iterator();
+        Iterator<JMenuItem> iter = remote_engine_start.iterator();
+        Iterator<JMenuItem> iter2 = remote_engine_stop.iterator();
+        Iterator<JMenuItem> iter3 = remote_engine_exit.iterator();
         while (iter.hasNext() && iter2.hasNext() && iter3.hasNext()) {
-            JMenuItem start = (JMenuItem) iter.next();
-            JMenuItem stop = (JMenuItem) iter2.next();
-            JMenuItem exit = (JMenuItem) iter3.next();
+            JMenuItem start = iter.next();
+            JMenuItem stop = iter2.next();
+            JMenuItem exit = iter3.next();
             if (start.getText().equals(host)) {
                 log.info("Found start host: " + start.getText());
                 start.setEnabled(!running);
@@ -431,6 +433,7 @@ public class ReportMenuBar extends JMenuBar implements LocaleChangeListener {
         }
     }
 
+    @Override
     public void setEnabled(boolean enable) {
         run_start.setEnabled(!enable);
         run_stop.setEnabled(enable);
