@@ -64,11 +64,11 @@ import org.jdom.input.SAXBuilder;
 public class JMeterTest extends JMeterTestCase {
     private static final Logger log = LoggingManager.getLoggerForClass();
 
-    private static Map guiTitles;
+    private static Map<String, Boolean> guiTitles;
 
-    private static Map guiTags;
+    private static Map<String, Boolean> guiTags;
 
-    private static Map funcTitles;
+    private static Map<String, Boolean> funcTitles;
 
     private static Properties nameMap;
     
@@ -162,7 +162,7 @@ public class JMeterTest extends JMeterTestCase {
      * Extract titles from component_reference.xml
      */
     public void createTitleSet() throws Exception {
-        guiTitles = new HashMap(90);
+        guiTitles = new HashMap<String, Boolean>(90);
 
         String compref = "../xdocs/usermanual/component_reference.xml";
         SAXBuilder bldr = new SAXBuilder();
@@ -170,11 +170,13 @@ public class JMeterTest extends JMeterTestCase {
         doc = bldr.build(compref);
         Element root = doc.getRootElement();
         Element body = root.getChild("body");
-        List sections = body.getChildren("section");
+        @SuppressWarnings("unchecked")
+        List<Element> sections = body.getChildren("section");
         for (int i = 0; i < sections.size(); i++) {
-            List components = ((Element) sections.get(i)).getChildren("component");
+            @SuppressWarnings("unchecked")
+            List<Element> components = sections.get(i).getChildren("component");
             for (int j = 0; j < components.size(); j++) {
-                Element comp = (Element) components.get(j);
+                Element comp = components.get(j);
                 String nm=comp.getAttributeValue("name");
                 if (!nm.equals("SSL Manager")){// Not a true GUI component
                     guiTitles.put(nm.replace(' ','_'), Boolean.FALSE);
@@ -190,7 +192,7 @@ public class JMeterTest extends JMeterTestCase {
      * Extract titles from component_reference.xml
      */
     public void createTagSet() throws Exception {
-        guiTags = new HashMap(90);
+        guiTags = new HashMap<String, Boolean>(90);
 
         String compref = "../xdocs/usermanual/component_reference.xml";
         SAXBuilder bldr = new SAXBuilder();
@@ -198,11 +200,13 @@ public class JMeterTest extends JMeterTestCase {
         doc = bldr.build(compref);
         Element root = doc.getRootElement();
         Element body = root.getChild("body");
-        List sections = body.getChildren("section");
+        @SuppressWarnings("unchecked")
+        List<Element> sections = body.getChildren("section");
         for (int i = 0; i < sections.size(); i++) {
-            List components = ((Element) sections.get(i)).getChildren("component");
+            @SuppressWarnings("unchecked")
+            List<Element> components = sections.get(i).getChildren("component");
             for (int j = 0; j < components.size(); j++) {
-                Element comp = (Element) components.get(j);
+                Element comp = components.get(j);
                 guiTags.put(comp.getAttributeValue("tag"), Boolean.FALSE);
             }
         }
@@ -212,7 +216,7 @@ public class JMeterTest extends JMeterTestCase {
      * Extract titles from functions.xml
      */
     public void createFunctionSet() throws Exception {
-        funcTitles = new HashMap(20);
+        funcTitles = new HashMap<String, Boolean>(20);
 
         String compref = "../xdocs/usermanual/functions.xml";
         SAXBuilder bldr = new SAXBuilder();
@@ -221,25 +225,27 @@ public class JMeterTest extends JMeterTestCase {
         Element root = doc.getRootElement();
         Element body = root.getChild("body");
         Element section = body.getChild("section");
-        List sections = section.getChildren("subsection");
+        @SuppressWarnings("unchecked")
+        List<Element> sections = section.getChildren("subsection");
         for (int i = 0; i < sections.size(); i++) {
-            List components = ((Element) sections.get(i)).getChildren("component");
+            @SuppressWarnings("unchecked")
+            List<Element> components = sections.get(i).getChildren("component");
             for (int j = 0; j < components.size(); j++) {
-                Element comp = (Element) components.get(j);
+                Element comp = components.get(j);
                 funcTitles.put(comp.getAttributeValue("name"), Boolean.FALSE);
             }
         }
     }
 
-    private int scanprintMap(Map m, String t) {
-        Set s = m.keySet();
+    private int scanprintMap(Map<String, Boolean> m, String t) {
+        Set<String> s = m.keySet();
         int unseen = 0;
         if (s.size() == 0) {
             return 0;
         }
-        Iterator i = s.iterator();
+        Iterator<String> i = s.iterator();
         while (i.hasNext()) {
-            Object key = i.next();
+            String key = i.next();
             if (!m.get(key).equals(Boolean.TRUE)) {
                 if (unseen == 0)// first time
                 {
@@ -267,7 +273,7 @@ public class JMeterTest extends JMeterTestCase {
      */
     private static Test suiteGUIComponents() throws Exception {
         TestSuite suite = new TestSuite("GuiComponents");
-        Iterator iter = getObjects(JMeterGUIComponent.class).iterator();
+        Iterator<Object> iter = getObjects(JMeterGUIComponent.class).iterator();
         while (iter.hasNext()) {
             JMeterGUIComponent item = (JMeterGUIComponent) iter.next();
             if (item instanceof JMeterTreeNode) {
@@ -295,7 +301,7 @@ public class JMeterTest extends JMeterTestCase {
      */
     private static Test suiteFunctions() throws Exception {
         TestSuite suite = new TestSuite("Functions");
-        Iterator iter = getObjects(Function.class).iterator();
+        Iterator<Object> iter = getObjects(Function.class).iterator();
         while (iter.hasNext()) {
             Object item = iter.next();
             if (item.getClass().equals(CompoundVariable.class)) {
@@ -314,9 +320,9 @@ public class JMeterTest extends JMeterTestCase {
      */
     private static Test suiteBeanComponents() throws Exception {
         TestSuite suite = new TestSuite("BeanComponents");
-        Iterator iter = getObjects(TestBean.class).iterator();
+        Iterator<Object> iter = getObjects(TestBean.class).iterator();
         while (iter.hasNext()) {
-            Class c = iter.next().getClass();
+            Class<? extends Object> c = iter.next().getClass();
             try {
                 JMeterGUIComponent item = new TestBeanGUI(c);
                 // JMeterGUIComponent item = (JMeterGUIComponent) iter.next();
@@ -392,7 +398,7 @@ public class JMeterTest extends JMeterTestCase {
      * Check that function descriptions are OK
      */
     public void runFunction2() throws Exception {
-        Iterator i = funcItem.getArgumentDesc().iterator();
+        Iterator<?> i = funcItem.getArgumentDesc().iterator();
         while (i.hasNext()) {
             Object o = i.next();
             assertTrue("Description must be a String", o instanceof String);
@@ -413,7 +419,7 @@ public class JMeterTest extends JMeterTestCase {
         if (!name.endsWith("TestBeanGUI")) {
             try {
                 String label = guiItem.getLabelResource();
-                assertTrue(label.length() > 0);
+                assertTrue("Label should not be null or empty for "+name,label != null && label.length() > 0);
                 assertFalse("'" + label + "' should be in resource file for " + name, JMeterUtils.getResString(
                         label).startsWith(JMeterUtils.RES_KEY_PFX));
             } catch (UnsupportedOperationException uoe) {
@@ -463,7 +469,7 @@ public class JMeterTest extends JMeterTestCase {
      */
     private static Test suiteSerializableElements() throws Exception {
         TestSuite suite = new TestSuite("SerializableElements");
-        Iterator iter = getObjects(Serializable.class).iterator();
+        Iterator<Object> iter = getObjects(Serializable.class).iterator();
         while (iter.hasNext()) {
             Serializable serObj = (Serializable) iter.next();
             if (serObj.getClass().getName().endsWith("_Stub")) {
@@ -502,7 +508,7 @@ public class JMeterTest extends JMeterTestCase {
      */
     private static Test suiteTestElements() throws Exception {
         TestSuite suite = new TestSuite("TestElements");
-        Iterator iter = getObjects(TestElement.class).iterator();
+        Iterator<Object> iter = getObjects(TestElement.class).iterator();
         while (iter.hasNext()) {
             TestElement item = (TestElement) iter.next();
             TestSuite ts = new TestSuite(item.getClass().getName());
@@ -543,23 +549,23 @@ public class JMeterTest extends JMeterTestCase {
         }
     }
 
-    private static Collection getObjects(Class extendsClass) throws Exception {
+    private static Collection<Object> getObjects(Class<?> extendsClass) throws Exception {
         String exName = extendsClass.getName();
         Object myThis = "";
-        Iterator classes = ClassFinder
+        Iterator<String> classes = ClassFinder
                 .findClassesThatExtend(JMeterUtils.getSearchPaths(), new Class[] { extendsClass }).iterator();
-        List objects = new LinkedList();
+        List<Object> objects = new LinkedList<Object>();
         String n = "";
         boolean caughtError = true;
         Throwable caught = null;
         try {
             while (classes.hasNext()) {
-                n = (String) classes.next();
+                n = classes.next();
                 // TODO - improve this check
                 if (n.endsWith("RemoteJMeterEngineImpl")) {
                     continue; // Don't try to instantiate remote server
                 }
-                Class c = null;
+                Class<?> c = null;
                 try {
                     c = Class.forName(n);
                     try {
