@@ -36,6 +36,7 @@ import junit.framework.TestCase;
 
 import org.apache.jmeter.config.Arguments;
 import org.apache.jmeter.protocol.http.util.HTTPArgument;
+import org.apache.jmeter.protocol.http.util.HTTPFileArg;
 import org.apache.jorphan.logging.LoggingManager;
 import org.apache.log.Logger;
 
@@ -52,7 +53,8 @@ public class PostWriterTest extends TestCase {
     private HTTPSampler sampler;
     private File temporaryFile;
     
-    PostWriter postWriter;
+    private PostWriter postWriter;
+    @Override
     protected void setUp() throws Exception {
         establishConnection();
         sampler = new HTTPSampler();// This must be the original (Java) HTTP sampler
@@ -70,6 +72,7 @@ public class PostWriterTest extends TestCase {
         output.close();
     }
 
+    @Override
     protected void tearDown() throws Exception {
         // delete temporay file
         temporaryFile.delete();
@@ -157,7 +160,7 @@ public class PostWriterTest extends TestCase {
         postWriter.sendPostData(connection, sampler);
         
         checkContentTypeUrlEncoded(connection);
-        byte[] expectedUrl = "title=mytitle&description=mydescription".getBytes();
+        byte[] expectedUrl = "title=mytitle&description=mydescription".getBytes(); // TODO - charset?
         checkContentLength(connection, expectedUrl.length);
         checkArraysHaveSameContent(expectedUrl, connection.getOutputStreamContent());
         expectedUrl = "title=mytitle&description=mydescription".getBytes(UTF_8);
@@ -217,7 +220,8 @@ public class PostWriterTest extends TestCase {
         checkContentLength(connection, TEST_FILE_CONTENT.length);        
         checkArraysHaveSameContent(TEST_FILE_CONTENT, connection.getOutputStreamContent());
         // Check that other encoding is not the current encoding
-        checkArraysHaveDifferentContent(new String(TEST_FILE_CONTENT).getBytes(otherEncoding), connection.getOutputStreamContent());
+        checkArraysHaveDifferentContent(new String(TEST_FILE_CONTENT) // TODO - charset?
+            .getBytes(otherEncoding), connection.getOutputStreamContent());
         
         // If we have both file as body, and form data, then only form data will be sent
         setupFormData(sampler);
@@ -227,7 +231,7 @@ public class PostWriterTest extends TestCase {
         postWriter.sendPostData(connection, sampler);
         
         checkContentTypeUrlEncoded(connection);
-        byte[] expectedUrl = "title=mytitle&description=mydescription".getBytes();
+        byte[] expectedUrl = "title=mytitle&description=mydescription".getBytes(); // TODO - charset?
         checkContentLength(connection, expectedUrl.length);
         checkArraysHaveSameContent(expectedUrl, connection.getOutputStreamContent());
     }
@@ -587,14 +591,8 @@ public class PostWriterTest extends TestCase {
      * @param httpSampler
      */
     private void setupFilepart(HTTPSampler httpSampler, String fileField, File file, String mimeType) {
-        httpSampler.setFileField(fileField);
-        if(file != null) {
-            httpSampler.setFilename(file.getAbsolutePath());
-        }
-        else {
-            httpSampler.setFilename("");
-        }
-        httpSampler.setMimetype(mimeType);
+        HTTPFileArg[] hfa = {new HTTPFileArg(file == null ? "" : file.getAbsolutePath(), fileField, mimeType)};
+        httpSampler.setHTTPFiles(hfa);
     }
 
     /**
@@ -715,7 +713,7 @@ public class PostWriterTest extends TestCase {
             output.write(titleValue.getBytes(contentEncoding));
         }
         else {
-            output.write(titleValue.getBytes());
+            output.write(titleValue.getBytes()); // TODO - charset?
         }
         output.write(CRLF);
         output.write(DASH_DASH);
@@ -737,7 +735,7 @@ public class PostWriterTest extends TestCase {
             output.write(descriptionValue.getBytes(contentEncoding));
         }
         else {
-            output.write(descriptionValue.getBytes());
+            output.write(descriptionValue.getBytes()); // TODO - charset?
         }
         output.write(CRLF);
         output.write(DASH_DASH);
@@ -880,30 +878,36 @@ public class PostWriterTest extends TestCase {
      */
     private static class StubURLConnection extends HttpURLConnection {
         private ByteArrayOutputStream output = new ByteArrayOutputStream();
-        private Map properties = new HashMap();
+        private Map<String, String> properties = new HashMap<String, String>();
         
         public StubURLConnection(String url) throws MalformedURLException {
             super(new URL(url));
         }
 
+        @Override
         public void connect() throws IOException {
         }
         
+        @Override
         public OutputStream getOutputStream() throws IOException {
             return output;
         }
 
+        @Override
         public void disconnect() {
         }
 
+        @Override
         public boolean usingProxy() {
             return false;
         }
 
+        @Override
         public String getRequestProperty(String key) {
-            return (String) properties.get(key);
+            return properties.get(key);
         }
 
+        @Override
         public void setRequestProperty(String key, String value) {
             properties.put(key, value);
         }
