@@ -34,6 +34,7 @@ import org.apache.jmeter.protocol.http.control.HttpMirrorServer;
 import org.apache.jmeter.protocol.http.control.TestHTTPMirrorThread;
 import org.apache.jmeter.protocol.http.util.EncoderCache;
 import org.apache.jmeter.protocol.http.util.HTTPArgument;
+import org.apache.jmeter.protocol.http.util.HTTPFileArg;
 import org.apache.jmeter.testelement.TestPlan;
 import org.apache.jmeter.threads.JMeterContextService;
 import org.apache.jmeter.threads.JMeterVariables;
@@ -74,6 +75,7 @@ public class TestHTTPSamplersAgainstHttpMirrorServer extends JMeterTestCase {
     public static Test suite(){
         TestSetup setup = new TestSetup(new TestSuite(TestHTTPSamplersAgainstHttpMirrorServer.class)){
             private HttpMirrorServer httpServer;
+            @Override
             protected void setUp() throws Exception {
                     httpServer = TestHTTPMirrorThread.startHttpMirror(MIRROR_PORT);
                     // Create the test file content
@@ -88,6 +90,7 @@ public class TestHTTPSamplersAgainstHttpMirrorServer extends JMeterTestCase {
                     output.close();
             }
             
+            @Override
             protected void tearDown() throws Exception {
                     // Shutdown mirror server
                     httpServer.stopServer();
@@ -737,7 +740,7 @@ public class TestHTTPSamplersAgainstHttpMirrorServer extends JMeterTestCase {
         String dataSentToMirrorServer = new String(res.getResponseData(), contentEncoding);
         int posDividerHeadersAndBody = getPositionOfBody(dataSentToMirrorServer);
         String headersSent = null;
-        String bodySent = null;
+        String bodySent = "";
         if(posDividerHeadersAndBody >= 0) {
             headersSent = dataSentToMirrorServer.substring(0, posDividerHeadersAndBody);
             // Skip the blank line with crlf dividing headers and body
@@ -746,7 +749,7 @@ public class TestHTTPSamplersAgainstHttpMirrorServer extends JMeterTestCase {
         else {
             fail("No header and body section found");
         }
-        // Check response headers
+         // Check response headers
         checkHeaderTypeLength(headersSent, "multipart/form-data" + "; boundary=" + boundaryString, expectedPostBody.length);
         // Check post body which was sent to the mirror server, and
         // sent back by the mirror server
@@ -818,7 +821,7 @@ public class TestHTTPSamplersAgainstHttpMirrorServer extends JMeterTestCase {
         String dataSentToMirrorServer = new String(res.getResponseData(), contentEncoding);
         int posDividerHeadersAndBody = getPositionOfBody(dataSentToMirrorServer);
         String headersSent = null;
-        String bodySent = null;
+        String bodySent = "";
         if(posDividerHeadersAndBody >= 0) {
             headersSent = dataSentToMirrorServer.substring(0, posDividerHeadersAndBody);
             // Skip the blank line with crlf dividing headers and body
@@ -851,7 +854,7 @@ public class TestHTTPSamplersAgainstHttpMirrorServer extends JMeterTestCase {
         String dataSentToMirrorServer = new String(res.getResponseData(), EncoderCache.URL_ARGUMENT_ENCODING);
         int posDividerHeadersAndBody = getPositionOfBody(dataSentToMirrorServer);
         String headersSent = null;
-        String bodySent = null;
+        String bodySent = "";
         if(posDividerHeadersAndBody >= 0) {
             headersSent = dataSentToMirrorServer.substring(0, posDividerHeadersAndBody);
             // Skip the blank line with crlf dividing headers and body
@@ -900,7 +903,7 @@ public class TestHTTPSamplersAgainstHttpMirrorServer extends JMeterTestCase {
         String dataSentToMirrorServer = new String(res.getResponseData(), EncoderCache.URL_ARGUMENT_ENCODING);
         int posDividerHeadersAndBody = getPositionOfBody(dataSentToMirrorServer);
         String headersSent = null;
-        String bodySent = null;
+        String bodySent = "";
         if(posDividerHeadersAndBody >= 0) {
             headersSent = dataSentToMirrorServer.substring(0, posDividerHeadersAndBody);
             // Skip the blank line with crlf dividing headers and body
@@ -1102,9 +1105,9 @@ public class TestHTTPSamplersAgainstHttpMirrorServer extends JMeterTestCase {
         // Set the form data
         setupFormData(httpSampler, isEncoded, titleField, titleValue, descriptionField, descriptionValue);
         // Set the file upload data
-        httpSampler.setFileField(fileField);
-        httpSampler.setFilename(fileValue.getAbsolutePath());
-        httpSampler.setMimetype(fileMimeType);      
+        HTTPFileArg[] hfa = {new HTTPFileArg(fileValue == null ? "" : fileValue.getAbsolutePath(), fileField, fileMimeType)};
+        httpSampler.setHTTPFiles(hfa);
+
     }
 
     /**
@@ -1195,7 +1198,7 @@ public class TestHTTPSamplersAgainstHttpMirrorServer extends JMeterTestCase {
             output.write(titleValue.getBytes(contentEncoding));
         }
         else {
-            output.write(titleValue.getBytes());
+            output.write(titleValue.getBytes()); // TODO - charset?
         }
         output.write(CRLF);
         output.write(DASH_DASH);
@@ -1218,7 +1221,7 @@ public class TestHTTPSamplersAgainstHttpMirrorServer extends JMeterTestCase {
             output.write(descriptionValue.getBytes(contentEncoding));
         }
         else {
-            output.write(descriptionValue.getBytes());
+            output.write(descriptionValue.getBytes()); // TODO - charset?
         }
         output.write(CRLF);
         output.write(DASH_DASH);
